/****************************************************
 Test for MH_RQ, WDT, XRESET and FPAGE for for TP64xx boards.
 (C) MicroLAB Systems, 1997-2002

   Description:
   ------------
	This demo continuously tests WDT enable/strobe, generates MH_RQ,
	and generates RESET signals for on-board PIOX and SIOX sites.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in either stand-alone mode or host mode.

	TI CCS6000 tools are required to recompile this demo. Either 
	TP6COFF.EXE utility or MIRAGE JTAG emulator are required to 
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
          - SA-mode or host mode
	  - no BOOT mode

*****************************************************/

#define		__TP6X_SELECT_TP64XX__	1

#include "c6x_def.h"
#include "tp6x_dsp.h"


main()
{ int i,j,n;

	// initialize DSP EMIF areas
   	TP6X_init_dsp_emif();

	while(1)
    {
		TP6X_set_mh_rq();
		TP6X_wdt_reset();


        TP6X_set_wdt_enable(ON);
		if(TP6X_wdt_enable()!=ON) break;
        TP6X_set_wdt_enable(OFF);
		if(TP6X_wdt_enable()!=OFF) break;

        TP6X_set_px_run(ON);
		if(TP6X_px_run()!=ON) break;
        TP6X_set_px_run(OFF);
		if(TP6X_px_run()!=OFF) break;

        TP6X_set_sxa_run(ON);
		if(TP6X_sxa_run()!=ON) break;
        TP6X_set_sxa_run(OFF);
		if(TP6X_sxa_run()!=OFF) break;

        TP6X_set_sxb_run(ON);
		if(TP6X_sxb_run()!=ON) break;
        TP6X_set_sxb_run(OFF);
		if(TP6X_sxb_run()!=OFF) break;

        TP6X_set_pxsx_run(0);
		if(TP6X_pxsx_run()!=0) break;
        TP6X_set_pxsx_run(1);
		if(TP6X_pxsx_run()!=1) break;
        TP6X_set_pxsx_run(2);
		if(TP6X_pxsx_run()!=2) break;
        TP6X_set_pxsx_run(4);
		if(TP6X_pxsx_run()!=4) break;
        TP6X_set_pxsx_run(7);
		if(TP6X_pxsx_run()!=7) break;
        TP6X_set_pxsx_run(0);
		if(TP6X_pxsx_run()!=0) break;

        for(i=0;i<=15;i++)
	 	{
        	TP6X_set_mirq_sel(TP6X_MIRQ0,i);
           	TP6X_set_mirq_sel(TP6X_MIRQ1,15-i);
           	TP6X_set_mirq_sel(TP6X_MIRQ2,(i+8)%16);
           	TP6X_set_mirq_sel(TP6X_MIRQ3,(i+4)%16);

	   		if(TP6X_mirq_sel(TP6X_MIRQ0)!=i) break;
	   		if(TP6X_mirq_sel(TP6X_MIRQ1)!=(15-i)) break;
	   		if(TP6X_mirq_sel(TP6X_MIRQ2)!=((i+8)%16)) break;
	   		if(TP6X_mirq_sel(TP6X_MIRQ3)!=((i+4)%16)) break;
	 	}

		TP6X_set_mirq_sel(TP6X_MIRQ0,TP6X_MIRQ_SEL_OFF);
		TP6X_set_mirq_sel(TP6X_MIRQ1,TP6X_MIRQ_SEL_OFF);
		TP6X_set_mirq_sel(TP6X_MIRQ2,TP6X_MIRQ_SEL_OFF);
		TP6X_set_mirq_sel(TP6X_MIRQ3,TP6X_MIRQ_SEL_OFF);
		TP6X_set_mnmi_sel(TP6X_MIRQ_SEL_OFF);

		if(TP6X_sys_stat_pci_on()!=OFF) // some tests require PCI presence
	 	{
			// set 32-bt word format for access to PCIC
			TP6X_SET_PCIC_FMT_W32;

	   		TP6X_set_pcic_wfifo(0);	// test r/w FIFO
	   		i=TP6X_pcic_rfifo();

	   		TP6X_set_pcic_ombx(TP6X_PCIC_OMBX0,0x12345678);		// test  OMBX
	   		TP6X_set_pcic_ombx(TP6X_PCIC_OMBX1,0x87654321);
	   		TP6X_set_pcic_ombx(TP6X_PCIC_OMBX2,0x9abcdef0);
	   		TP6X_set_pcic_ombx(TP6X_PCIC_OMBX3,0x00edcba9);
	   		if(TP6X_pcic_ombx(TP6X_PCIC_OMBX0)!=0x12345678) break;
	   		if(TP6X_pcic_ombx(TP6X_PCIC_OMBX1)!=0x87654321) break;
	   		if(TP6X_pcic_ombx(TP6X_PCIC_OMBX2)!=0x9abcdef0) break;
	   		if((TP6X_pcic_ombx(TP6X_PCIC_OMBX3) & 0xffffff)!=0x00edcba9) break;
	  	}

	  	// test FPGA register outputs for rev.1B board with 2M/4M/8Mx8 FLASH memory
	  	if(TP64XX_REV1B_BOARD && (TP6X_FLASH_LEN_IS_2M || TP6X_FLASH_LEN_IS_4M || TP6X_FLASH_LEN_IS_8M))
	  	{
        	// this is TP64xx board with either 2Mx8, or 4Mx8, or 8Mx8 FLASH memory, i.e. with FPAGE register.
        	// Number of bits in FPAGE register depends upon the FLASH length.

		  	if(TP6X_FLASH_LEN_IS_2M)
		  		j=1;
		  	else
			  	if(TP6X_FLASH_LEN_IS_4M)
			  		j=2;
			  	else
			  		j=3;

			// test FPAGE to the '0' state
			TP6X_set_flash_page_rg(0);
        	TP6X_set_mirq_sel(TP6X_MIRQ0,0xf);
			if(TP6X_get_flash_page_rg()!=0) break;

			// run '1' thru all FPAGE bits
			for(i=0,n=1;i<j;i++)
			{
				TP6X_set_flash_page_rg(n);

				// 'quad-wide the time for bit '0' for easier oscilloscope synch
				if(i==0)
				{
					TP6X_set_flash_page_rg(n);
					TP6X_set_flash_page_rg(n);
					TP6X_set_flash_page_rg(n);
				}

				// write false bus data
	        	TP6X_set_mirq_sel(TP6X_MIRQ0,~n);
				// verify FPAGE register contents
				if(TP6X_get_flash_page_rg()!=n) break;

				// shift '1' to the upper bit
				n <<= 1;
			}

			// clear FPAGE register at the end of the test
			TP6X_set_flash_page_rg(0);
	  	}
	}


error:	// get here on any error
    while(1) ;

}



