/****************************************************
 Test for MH_RQ, WDT, XRESET for TP6x boards.
 (C) MicroLAB Systems, 1997-2000

   Description:
   ------------
	This demo continuously tests WDT enable/strobe, generates MH_RQ,
	and generates RESET signals for on-board PIOX and SIOX sites.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in either stand-alone mode or host mode.

	TI CCS6000 tools are required to recompile this demo. Either 
	TP6COFF.EXE utility or MIRAGE JTAG emulator are required to 
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
          - SA-mode or host mode
	  - no BOOT mode

*****************************************************/

#include "c6x_def.h"
#include "tp6x_dsp.h"


main()
{ int i;

	// initialize DSP EMIF areas
	// configure on-board RAM as SBSRAM for TP6202/TP6203 (has no matter for this test)
   TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);

   while(1)
    {
	TP6X_set_mh_rq();
	TP6X_wdt_reset();

        TP6X_set_wdt_enable(ON);
	if(TP6X_wdt_enable()!=ON) break;
        TP6X_set_wdt_enable(OFF);
	if(TP6X_wdt_enable()!=OFF) break;

        TP6X_set_px_run(ON);
	if(TP6X_px_run()!=ON) break;
        TP6X_set_px_run(OFF);
	if(TP6X_px_run()!=OFF) break;

        TP6X_set_sxa_run(ON);
	if(TP6X_sxa_run()!=ON) break;
        TP6X_set_sxa_run(OFF);
	if(TP6X_sxa_run()!=OFF) break;

        TP6X_set_sxb_run(ON);
	if(TP6X_sxb_run()!=ON) break;
        TP6X_set_sxb_run(OFF);
	if(TP6X_sxb_run()!=OFF) break;

        TP6X_set_pxsx_run(0);
	if(TP6X_pxsx_run()!=0) break;
        TP6X_set_pxsx_run(1);
	if(TP6X_pxsx_run()!=1) break;
        TP6X_set_pxsx_run(2);
	if(TP6X_pxsx_run()!=2) break;
        TP6X_set_pxsx_run(4);
	if(TP6X_pxsx_run()!=4) break;
        TP6X_set_pxsx_run(7);
	if(TP6X_pxsx_run()!=7) break;
        TP6X_set_pxsx_run(0);
	if(TP6X_pxsx_run()!=0) break;

        for(i=0;i<=15;i++)
	 {
           TP6X_set_mirq_sel(TP6X_MIRQ0,i);
	   if(TP6X_mirq_sel(TP6X_MIRQ0)!=i) break;

           TP6X_set_mirq_sel(TP6X_MIRQ1,15-i);
	   if(TP6X_mirq_sel(TP6X_MIRQ1)!=(15-i)) break;

           TP6X_set_mirq_sel(TP6X_MIRQ2,(i+8)%16);
	   if(TP6X_mirq_sel(TP6X_MIRQ2)!=((i+8)%16)) break;

           TP6X_set_mirq_sel(TP6X_MIRQ3,(i+4)%16);
	   if(TP6X_mirq_sel(TP6X_MIRQ3)!=((i+4)%16)) break;
	 }

	TP6X_set_mirq_sel(TP6X_MIRQ0,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mirq_sel(TP6X_MIRQ1,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mirq_sel(TP6X_MIRQ2,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mirq_sel(TP6X_MIRQ3,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mnmi_sel(TP6X_MIRQ_SEL_OFF);

	if(TP6X_sys_stat_pci_on()!=OFF) // some tests require PCI presence
	 {
	   TP6X_set_pcic_wfifo(0);	// test r/w FIFO
	   i=TP6X_pcic_rfifo();

	   TP6X_set_pcic_ombx(TP6X_PCIC_OMBX0,0x12345678);		// test  OMBX
	   if(TP6X_pcic_ombx(TP6X_PCIC_OMBX0)!=0x12345678) break;
	   TP6X_set_pcic_ombx(TP6X_PCIC_OMBX1,0x87654321);
	   if(TP6X_pcic_ombx(TP6X_PCIC_OMBX1)!=0x87654321) break;
	   TP6X_set_pcic_ombx(TP6X_PCIC_OMBX2,0x9abcdef0);
	   if(TP6X_pcic_ombx(TP6X_PCIC_OMBX2)!=0x9abcdef0) break;
	   TP6X_set_pcic_ombx(TP6X_PCIC_OMBX3,0x00edcba9);
	   if((TP6X_pcic_ombx(TP6X_PCIC_OMBX3) & 0xffffff)!=0x00edcba9) break;

	  }  	 
    }


  error:	// get here on any error
    while(1) ;

}



