/********************************************************************
   TORNADO-P64xx Demo Utility.
   FLASH test utility. rev.1B
   (C) MicroLAB Systems, 2000-2004


   Description:
   ------------
	This demo tests on-board FLASH memory. Plugged-in FLASH memory for TP64xx rev.1A board must be
	either AMD Am29F010B-90JC or Am29F040B-90JC. For TP64xx rev.1B board with embedded FLASH memory,
	this must be either of 512K/1M/2M/4M/8Mx8 FLASH memory.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
		  - no BOOT mode
		  - FLASH write protection switch must be set to OFF
***********************************************************************/
#define		__TP6X_SELECT_TP64XX__				1
#define		__TP6X_INCLUDE_AMD_FLASH_CONTROL__	1

#include	"math.h"
#include	"c6x_def.h"		// include general C6x defs
#include	"tp6x_dsp.h"	// include TP6x defs


#define	FLASH_128K_TEST_CYCLES		11		// 	for 128Kx8 FLASH memory (rev.1A only)
#define	FLASH_512K_TEST_CYCLES		15		//	for 512Kx8 FLASH memory (rev.1A and rev.1B)
#define	FLASH_1M_TEST_CYCLES		19		//	for 1Mx8 FLASH memory (rev.1B)
#define	FLASH_2M_TEST_CYCLES		20		//	for 2Mx8 FLASH memory (rev.1B)
#define	FLASH_4M_TEST_CYCLES		21		//	for 4Mx8 FLASH memory (rev.1B)
#define	FLASH_8M_TEST_CYCLES		23		//	for 8Mx8 FLASH memory (rev.1B)

#define FLASH_CYCLE_LEN				0x100L



unsigned FLASH_SADDR[23] 	= { 0x00000L, 	0x01000L,
                                0x02000L, 	0x04000L,
                                0x08000L, 	0x08800L,
				    			0x10000L, 	0x10400L,
                                0x18000L, 	0x18200L,
                                0x18100L,
				    			0x20000L, 	0x40000L,
				    			0x60000L, 	0x70000L,	// 512Kx8 test ends here
				    			0x80000L, 	0xa0000L,
				    			0xc0000L, 	0xe0000L,	// 1Mx8 test ends here
				    			0x100000L,				// 2Mx8 test ends here
								0x200000L,				// 4Mx8 test ends here
				    			0x400000L, 	0x700000L, 	// 8Mx8 test ends here
				  			  };

unsigned FLASH_SDATA[23]	= { 0x00, 0xe9,
				    			0x3f, 0x55,
				    			0x25, 0xe3,
				    			0x8c, 0xa1,
				    			0x66, 0x99,
				    			0xbf,
				    			0xfa, 0x15,
				    			0x79, 0x59,
				    			0xe9, 0x15,
				    			0x7d, 0xf3,
				    			0x49, 0x5b,
				    			0x27, 0x09,
                              };


unsigned _flash_test_cycles;



void main()
{  int i,j,r;
   unsigned d;
   unsigned long a;
   unsigned manuf_id,dev_id,len,sectors, sector_len;

	// init C64xx DSP EMIF registers,
	TP6X_init_dsp_emif();

	// get FLASH info
	if((r=TP6X_get_flash_info(&manuf_id,&dev_id,&len,&sectors,&sector_len))!=TP6X_OK)
		goto error;

	// set number of FLASH test cycles upon the FLASH length
	switch(dev_id)
	{
		case TP6X_FLASH_AMD_PLUGIN_128KX8_DEV_ID:
        		_flash_test_cycles = FLASH_128K_TEST_CYCLES;
        		break;

		case TP6X_FLASH_AMD_PLUGIN_512KX8_DEV_ID:
		case TP6X_FLASH_AMD_EMBED_512KX8_DEV_ID:
        		_flash_test_cycles = FLASH_512K_TEST_CYCLES;
        		break;

		case TP6X_FLASH_AMD_EMBED_1MX8_DEV_ID:
        		_flash_test_cycles = FLASH_1M_TEST_CYCLES;
        		break;

		case TP6X_FLASH_AMD_EMBED_2MX8_DEV_ID:
        		_flash_test_cycles = FLASH_2M_TEST_CYCLES;
        		break;

		case TP6X_FLASH_AMD_EMBED_4MX8_DEV_ID:
        		_flash_test_cycles = FLASH_4M_TEST_CYCLES;
        		break;

		case TP6X_FLASH_AMD_EMBED_8MX8_DEV_ID:
        		_flash_test_cycles = FLASH_8M_TEST_CYCLES;
        		break;

			// get here on unsupported FLASH memory length
		default:
				// set unused error code for this exceptional situation
				r = -10000;
				goto error;
	}

	// erase entire chip with verification
	if((r=TP6X_erase_flash_chip(TP6X_ON))!=TP6X_OK) goto error;

	// now write shifted RAMPS to all predefied FLASH areas
    for(i=0;i<_flash_test_cycles;i++)
    {
    	for(a=FLASH_SADDR[i], j=0, d=FLASH_SDATA[i];
		    j<FLASH_CYCLE_LEN;
            a++,j++,d++
		   )
		{
		 	if((r=TP6X_write_flash_data(a,d & TP6X_FLASH_DATA_BITMASK))!=TP6X_OK)
				goto error;
	    }
	}

	// now verifies written data
	for(i=0;i<_flash_test_cycles;i++)
    {
    	for(a=FLASH_SADDR[i], j=0, d=FLASH_SDATA[i];
            j<FLASH_CYCLE_LEN;
            a++,j++,d++)
        {
		 	if((r=TP6X_read_flash_data(a))!=(d & TP6X_FLASH_DATA_BITMASK))
				goto error;
	    }
	}

	// finally erase entire chip using sector erase
    for(i=0;i<sectors;i++)
    {
		if((r=TP6X_erase_flash_sector(i*sector_len,TP6X_ON))!=TP6X_OK)
         	goto error;
	}

halt_ok:
	// TRAP here in case programming & verification is OK
	while(1);

error:
	// TRAP here in case of any error
	// variable 'r' contains returned error code
	while(1);
}

