/********************************************************************
   TORNADO-P6x Demo Utility.
   AMD FLASH test utility. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo tests on-board AMD FLASH memory. FLASH memory installed must be
	either AMD Am29F010B-90JC or Am29F0140B-90JC.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
	  - no BOOT mode
***********************************************************************/

#include	"math.h"
#include	"c6x_def.h"	// include general C6x defs

	// allow inclusion of TP6X AMD FLASH programming utilities fm TP6XDSP.H file	
#define		__TP6X_INCLUDE_AMD_FLASH_CONTROL__	1	
#include	"tp6x_dsp.h"	// include TP6x defs


#define	FLASH_1M_TEST_CYCLES		11
#define	FLASH_4M_TEST_CYCLES		15
#define FLASH_CYCLE_LEN			0x100L

unsigned FLASH_SADDR[15] 	= { 0x00000L, 0x01000L,	
                                    0x02000L, 0x04000L,
                                    0x08000L, 0x08800L,
				    0x10000L, 0x10400L,
                                    0x18000L, 0x18200L,
                                    0x18100L,
				    0x20000L, 0x40000L,
				    0x60000L, 0x70000L		
				  };

unsigned FLASH_SDATA[15]	= { 0x00, 0xe9,
				    0x3f, 0x55,
				    0x25, 0xe3,
				    0x8c, 0xa1,
				    0x66, 0x99,
				    0xbf,
				    0xfa, 0x15,
				    0x79, 0x59
                                   };

unsigned _flash_test_cycles = FLASH_1M_TEST_CYCLES;



void main()
{  int i,j,r;
   unsigned d;
   unsigned long a;
   unsigned manuf_id,dev_id,len,sectors, sector_len;

		// init C6x DSP EMIF registers, 
		// configure external memory as SBSRAM for TP6202/TP6203 boards (has no matter for this test) 
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);

		// get FLASH info
	if((r=TP6X_flash_info(&manuf_id,&dev_id,&len,&sectors,&sector_len))!=TP6X_OK)
             goto error;

	_flash_test_cycles = (dev_id==TP6X_FLASH_AM29F010_DEV_ID) ?
				FLASH_1M_TEST_CYCLES:FLASH_4M_TEST_CYCLES ;

		// erase entire chip with verification
	if((r=TP6X_flash_erase_chip(TP6X_ON))!=TP6X_OK) goto error;

		// now write shifted RAMPS to all predefied FLASH areas
        for(i=0;i<_flash_test_cycles;i++)
          {
            for(a=FLASH_SADDR[i], j=0, d=FLASH_SDATA[i];
                j<FLASH_CYCLE_LEN;
                a++,j++,d++)
               {
		 if((r=TP6X_flash_write(a,d & TP6X_FLASH_DATA_BITMASK))!=TP6X_OK) goto error;
	       }
	  }

		// now verifies written data
        for(i=0;i<_flash_test_cycles;i++)
          {
            for(a=FLASH_SADDR[i], j=0, d=FLASH_SDATA[i];
                j<FLASH_CYCLE_LEN;
                a++,j++,d++)
               {
		 if((r=TP6X_flash_read(a))!=(d & TP6X_FLASH_DATA_BITMASK)) goto error;
	       }
	  }

		// finally erase entire chip using sector erase
        for(i=0;i<sectors;i++)
          {
		if((r=TP6X_flash_erase_sector(i*sector_len,TP6X_ON))!=TP6X_OK)
                   goto error;
	  }

halt_ok:
	   /* TRAP here in case programming & verification is OK */
   while(1);	

error:
	   /* TRAP here in case of any error */
   while(1);	


}

