/********************************************************************
   TORNADO-P64xx Demo Utility.
   FLASH Programming (Burn-in) Utility. rev.1B
   (C) MicroLAB Systems, 2000-2004


   Description:
   ------------
	This demo utility is used to program (burn-in) user supplied data
	array into on-board FLASH memory.

		!!! IMPORTANT NOTE !!!
		~~~~~~~~~~~~~~~~~~~~~~
			The on-board FLASH WRITE ENABLE switch must be set
			to the ON state in order to run this demo.

	Input FLASH burn-in data array must be in an include file in 32-bit HEX
	format. This array can be either data or valid on-board C64xx DSP application,
	which can be converted directly from CCS C6000 generaed output.

		!!! IMPORTANT NOTE !!!
		~~~~~~~~~~~~~~~~~~~~~~
			In case TP64xx on-board FLASH must contain FLASH BOOT code, then
			it must be kept in mind that TI C64xx DSP allow only 1kbyte of FLASH data
			to be transferred to DSP on-chip memory during FLASH BOOT procedure (refer to
			TI C64xx DSP documentation for more details).

	Once input data array or application, which has to be programmed to FLASH, has
	been changed, then this utility must be recompiled using TI C6000 Code Composer
	C-compiler and then re-loaded into DSP environment via JTAG emulator and re-run.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo as well as other supplied utilities are distributed free
	of charge and without any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
    on-board configuration switch:
      	- SA-mode
		- no BOOT mode
		- FLASH write is enabled
***********************************************************************/

#define		__TP6X_SELECT_TP64XX__				1
#define		__TP6X_INCLUDE_AMD_FLASH_CONTROL__	1

#include	"math.h"
#include	"c6x_def.h"		// include general C6x defs
#include	"tp6x_dsp.h"	// include TP6x defs


//~~~~~ include FLASH burn-in data array (32-bit C-hex format) ~~~~
// this array will have default unsigned long LUI_DATA[] reference
#include "c64_boot.ci"
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

// FLASH data array size definition
#define  FLASH_ARRAY_LEN	sizeof(LUI_DATA)

void main()
{
	int i,r;
   	unsigned flash_addr;
   	unsigned char *flash_src_data_ptr;

	// init C64xx DSP EMIF registers,
	TP6X_init_dsp_emif();

	// erase FLASH chip with verification
	if((r=TP6X_erase_flash_chip(TP6X_ON))!=TP6X_OK)
		goto flash_error;

	// now program input array to FLASH
	// Note, that input data array must be transferred to FLASH memory in byte increment
    for(flash_addr=0,i=0, flash_src_data_ptr = (unsigned char*)LUI_DATA;
		i<FLASH_ARRAY_LEN;
		i++, flash_addr++, flash_src_data_ptr++)
    {
		 if((r=TP6X_write_flash_data(flash_addr,*flash_src_data_ptr))!=TP6X_OK)
		 	goto flash_error;
    }

	// now verify written data
	// Note, that input data array must be transferred to FLASH memory in byte increment
    for(flash_addr=0,i=0, flash_src_data_ptr = (unsigned char*)LUI_DATA;
		i<FLASH_ARRAY_LEN;
		i++, flash_addr++, flash_src_data_ptr++)
	{
		if(TP6X_read_flash_data(flash_addr)!= *flash_src_data_ptr )
			goto flash_verif_error;
  	}

term_ok:
	// TRAP here in case programming & verification is OK
	while(1);

flash_error:
	// TRAP here in case of any FLASH API error
	// 'r' var will contain error code
	while(1);

flash_verif_error:
	// TRAP here in case of FLASH data verification error
	while(1);
}

