/****************************************************
	FLASH Boot Demo for TORNADO-P6x Boards. v.1.2
	(C) MicroLAB Systems, 2000-2002


   	Description:
   	------------
		This demo boots from on-board FLASH and continuously tests WDT 
		enable/strobe, generates MH_RQ,	and generates RESET signals for 
		on-board PIOX and SIOX sites. This demo is actually TP6X_AUX_IO.C
		demo sample from TP6X DSP demo set.


   	TORNADO platform:
   	-----------------
		TI CCS6000 tools are required to recompile this demo. External
		EPROM/FLASH programmer is required in order to 'burn-in' output
		hex data into 8-bit FLASH.

		Use HEX6X.EXE utility in order to convert compiled .OUT file 
		into HEX data for EPROM programmer:
			HEX6X  tp64xx_boot_rom.h6x


  	Copyright:
  	----------
		This demo is distributed free of charge and as it is without
		any obligation from MicroLAB Systems.


  	Demo and Signal Parameters:
  	---------------------------
        on-board configuration switch:
          - on-board BOOT configuration switch SW2: FLASH8 BOOT mode

*****************************************************/

#include "c6x_def.h"
#include "tp6x_dsp.h"


init_cpu()
{
	*C6X_EMIF_GCR=TP6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=TP6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=TP6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=TP6X_EMIF_CE2_DATA;

	*C6X_EMIF_SDRAM=TP6X_EMIF_SDRAM_DATA;
	*C6X_EMIF_SDRAM_TIMER=TP6X_EMIF_SDRAM_TIMER_DATA;	
	*C6X_EMIF_CE3=TP6X_EMIF_CE3_DATA;

}



main()
{ int i;

   init_cpu();

   while(1)
    {
	TP6X_set_mh_rq();
	TP6X_wdt_reset();

        TP6X_set_wdt_enable(ON);
	if(TP6X_wdt_enable()!=ON) break;
        TP6X_set_wdt_enable(OFF);
	if(TP6X_wdt_enable()!=OFF) break;

        TP6X_set_px_run(ON);
	if(TP6X_px_run()!=ON) break;
        TP6X_set_px_run(OFF);
	if(TP6X_px_run()!=OFF) break;

        TP6X_set_sxa_run(ON);
	if(TP6X_sxa_run()!=ON) break;
        TP6X_set_sxa_run(OFF);
	if(TP6X_sxa_run()!=OFF) break;

        TP6X_set_sxb_run(ON);
	if(TP6X_sxb_run()!=ON) break;
        TP6X_set_sxb_run(OFF);
	if(TP6X_sxb_run()!=OFF) break;

        TP6X_set_pxsx_run(0);
	if(TP6X_pxsx_run()!=0) break;
        TP6X_set_pxsx_run(1);
	if(TP6X_pxsx_run()!=1) break;
        TP6X_set_pxsx_run(2);
	if(TP6X_pxsx_run()!=2) break;
        TP6X_set_pxsx_run(4);
	if(TP6X_pxsx_run()!=4) break;
        TP6X_set_pxsx_run(7);
	if(TP6X_pxsx_run()!=7) break;
        TP6X_set_pxsx_run(0);
	if(TP6X_pxsx_run()!=0) break;

        for(i=0;i<=15;i++)
	 {
           TP6X_set_mirq_sel(TP6X_MIRQ0,i);
	   if(TP6X_mirq_sel(TP6X_MIRQ0)!=i) break;

           TP6X_set_mirq_sel(TP6X_MIRQ1,15-i);
	   if(TP6X_mirq_sel(TP6X_MIRQ1)!=(15-i)) break;

           TP6X_set_mirq_sel(TP6X_MIRQ2,(i+8)%16);
	   if(TP6X_mirq_sel(TP6X_MIRQ2)!=((i+8)%16)) break;

           TP6X_set_mirq_sel(TP6X_MIRQ3,(i+4)%16);
	   if(TP6X_mirq_sel(TP6X_MIRQ3)!=((i+4)%16)) break;
	 }

	TP6X_set_mirq_sel(TP6X_MIRQ0,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mirq_sel(TP6X_MIRQ1,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mirq_sel(TP6X_MIRQ2,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mirq_sel(TP6X_MIRQ3,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mnmi_sel(TP6X_MIRQ_SEL_OFF);

	if(TP6X_sys_stat_pci_on()!=OFF) // some tests require PCI presence
	 {
	   TP6X_set_pcic_wfifo(0);	// test r/w FIFO
	   i=TP6X_pcic_rfifo();

	   TP6X_set_pcic_ombx(TP6X_PCIC_OMBX0,0x12345678);		// test  OMBX
	   if(TP6X_pcic_ombx(TP6X_PCIC_OMBX0)!=0x12345678) break;
	   TP6X_set_pcic_ombx(TP6X_PCIC_OMBX1,0x87654321);
	   if(TP6X_pcic_ombx(TP6X_PCIC_OMBX1)!=0x87654321) break;
	   TP6X_set_pcic_ombx(TP6X_PCIC_OMBX2,0x9abcdef0);
	   if(TP6X_pcic_ombx(TP6X_PCIC_OMBX2)!=0x9abcdef0) break;
	   TP6X_set_pcic_ombx(TP6X_PCIC_OMBX3,0x00edcba9);
	   if((TP6X_pcic_ombx(TP6X_PCIC_OMBX3) & 0xffffff)!=0x00edcba9) break;

	  }  	 
    }


  error:
    while(1) ;

}



