/*************************************************************************
  External XIRQ-0..3 Interrupt Request Demo for TP6x.
  (C) MicroLAB Systems, 1999-2002

   Description:
   ------------
	This demo performs DSP external interrupt requests (XIRQ-0..3) test for
	TP6x boards. XIRQ-0..3 DSP external interrupt requests are shared between
	TP6x on-board SIOX and PIOX interface sites.

		Note!!!:
		--------
			This test has limited functionality at TP64xx boards, since these 
			boards use dedicated non-shares PIOX and SIOX interrupt request inputs. 
			Use TP64XX_PXIRQ.C and TP64XX_SXIRQ.C demos instead.

	This test assumes that external signal generator will be connected to
  	XIRQ-0..3 pins of SIOX or PIOX I/F connectors in order to apply external
	interrupt source. Correct operation is indicated by continuous toggling of
	the corresponding indicator signal each time the interrupt is recognized.

		XIRQ signal:	C6x DSP EXT_INTx routed to:			IRQ indicator signal:
		~~~~~~~~~~~		~~~~~~~~~~~~~~~~~~~~~~~~~~~			~~~~~~~~~~~~~~~~~~~~~
	    XIRQ0 			EXT_INT4        				TM0 output of SIOX/PIOX site is toggled
    	XIRQ1 			EXT_INT5        				TM1 output of SIOX/PIOX site is toggled
     	XIRQ2 			EXT_INT6        				McBSP-0 DX0 output of SIOX-A site is toggled
     	XIRQ3 			EXT_INT7        				McBSP-1 DX1 output of SIOX-A site is toggled

	The following on-board DSP TIMERs configuration jumpers and switches 
	shall be installed and configured properly for correct operation of 
	this test:
			- TP62/TP67 boards: J5 and J6 shall be installed
			- TP6202/TP6203 boards: SW4-3 and SW4-4 shall be set to the 'ON' state

   TORNADO platform:
   -----------------
	This demo does not utilize any host resources.
	TI CCS6000 tools are required to recompile this demo. Either 
	TP6COFF.EXE utility or MIRAGE JTAG emulator are required to 
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
          	- SA-mode or host mode
	  		- no BOOT mode
**************************************************************************/

#include "c6x_def.h"
#include "tp6x_dsp.h"

#define DX_OUT_0      0x3f00
#define DX_OUT_1      0x3f20

#define TM_OUT_0  0
#define TM_OUT_1  4


int count0=0,
    count1=0,
    count2=0,
    count3=0;



// interrupt handler for EXT_INT4
interrupt void c_mirq0()	// toggle TOUT pin of TIMER-0
{
	if(count0==0)	
    {
    	*C6X_TM0_CONTROL = TM_OUT_0;   // set TM0_TOUT=0
	 	count0=1;
    }
    else
    {
    	*C6X_TM0_CONTROL = TM_OUT_1;   // set TM0_OUT=1
	 	count0=0;
    }
}

// interrupt handler for EXT_INT5
interrupt void c_mirq1()	// toggle TOUT pin of TIMER-1
{
 	if(count1==0)	
    {
    	*C6X_TM1_CONTROL = TM_OUT_0;   // set TM1_TOUT=0
	 	count1=1;
    }
    else
    {
    	*C6X_TM1_CONTROL = TM_OUT_1;   // set TM1_TOUT=1 
	 	count1=0;
    }
}

// interrupt handler for EXT_INT6
interrupt void c_mirq2()				// toggle DX of McBSP-0
{
   	if(count2==0)	
    {
    	*C6X_MCBSP0_PCR = DX_OUT_0;   	// set DX0=0 of McBSP-0
	 	count2=1;
    }
    else
    {
    	*C6X_MCBSP0_PCR = DX_OUT_1;   	// set DX0=1 of McBSP-0
	 	count2=0;
 	}
}

// interrupt handler for EXT_INT7
interrupt void c_mirq3()  				// toggle DX of McBSP-1
{
   	if(count3==0)	
    {
    	*C6X_MCBSP1_PCR = DX_OUT_0;   	// set DX0=0 of McBSP-1
	 	count3=1;
    }
    else
    {
    	*C6X_MCBSP1_PCR = DX_OUT_1;   	// set DX0=1 of McBSP-1
	 	count3=0;
    }
}


main()
{
	// initialize DSP EMIF areas
	// configure on-board RAM as SBSRAM for TP6202/TP6203 (has no matter for this test)
   	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);

  	*C6X_TM0_CONTROL = 0;   		// configure TIMER-0 as I/O
  	*C6X_TM1_CONTROL = 0;   		// configure TIMER-1 as I/O
   	*C6X_MCBSP0_PCR = DX_OUT_0;   	// set DX0=0 of McBSP-0 
   	*C6X_MCBSP1_PCR = DX_OUT_0;   	// set DX0=0 of McBSP-1

	C6X_XINT_ALL_POLARITY_SET(C6X_XINT_POLARITY_POS);
	C6X_INT_DISABLE_ALL;

	// now configure external IRQ sources
	TP6X_set_ext_int_sel(TP6X_EXT_INT4_SEL,TP6X_EXT_INT_SEL_XIRQ0);
	TP6X_set_ext_int_sel(TP6X_EXT_INT5_SEL,TP6X_EXT_INT_SEL_XIRQ1);
	TP6X_set_ext_int_sel(TP6X_EXT_INT6_SEL,TP6X_EXT_INT_SEL_XIRQ2);
	TP6X_set_ext_int_sel(TP6X_EXT_INT7_SEL,TP6X_EXT_INT_SEL_XIRQ3);
	TP6X_set_mnmi_sel(TP6X_EXT_INT_SEL_OFF);

	// now clear and enable external interrupt sources
	C6X_INT_CLEAR_ALL;
   	C6X_INT_ENABLE(4);
   	C6X_INT_ENABLE(5);
   	C6X_INT_ENABLE(6);
   	C6X_INT_ENABLE(7);
   	C6X_GINT_ENABLE;    			

	// loop infinitely with interrupts enabled
	while(1);

}


