/****************************************************
 PIOX r/w test for TP64xx boards.
 (C) MicroLAB Systems, 1997-2002

   Description:
   ------------
    	This test checks for R/W buffers of PIOX data bus
	   	of TORNADO-P64xx boards.

			!!!CAUTION!!!
       			DO NOT APPLY ANY OSCILLOSCOPE NODES to the PIOX
       			DATA BUS! (written data must stay precharged on
				PIOX data buffers).

		External grounding jumper is required for PX_READY input pin of
		PIOX/PIOX-16 interface site connector, which must be installed into
		pin #41 of on-board JP1 connector in order to permanently confirm
		PIOX/PIOX-16 parallel data bus cycles.

		'C6x DSP on-chip TIMER-0 output is used as the progress indicator and is 
		toggling in case of correct test procurement. In order to enable TM0 output,
		the	following on-board DSP TIMERs configuration jumpers and switches 
		shall be installed and configured properly:
			- SW5-1 must be set to the 'ON' state


   TORNADO platform:
   -----------------
	This demo does not utilize any host resources.
	TI CCS6000 tools are required to recompile this demo. Either 
	TP6COFF.EXE utility or MIRAGE JTAG emulator are required to 
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
    on-board configuration switch:
  		- SA-mode or host mode
	  	- no BOOT mode
*****************************************************/

#define		__TP6X_SELECT_TP64XX__	1

#include "c6x_def.h"
#include "tp6x_dsp.h"


#define TM_OUT_0  0
#define TM_OUT_1  4



main()
{
   	unsigned  int d, i, count;

	// initialize DSP EMIF areas
   	TP6X_init_dsp_emif();

	// select 32-bit PIOX data access format for address scan
	TP6X_SET_PX_FMT_W32;

   	count=0;

   	while(1)
    {
      	for(d=0x1L, i=0; i<32 ; i++)
		{ 
			// write to PIOX/PIOX-16
			TP6X_set_piox_rg(0,d);

			//  verify read-back data
			//  !! do not apply any oscilloscope probe to the PX_D[] pins in order
			//     to keep PX_D[] data uncheged for reads
	  		if(TP6X_get_piox_rg(0)!=d) goto error;
	  		d= (d<<1);
		}

		// use TIMER-0 output as the progress indicator
	   	if(count==0)	
      	{
      	 	*C6X_TM0_CONTROL = TM_OUT_0;
	 		count=1;
      	}
     	else
      	{
      	 	*C6X_TM0_CONTROL = TM_OUT_1;
	 		count=0;
      	}

	}

error:
	// get here on error condition
    while(1);

}



