/****************************************************
 McBSP pins scan test for TP64xx boards.
 (C) MicroLAB Systems, 1999-2002

   Description:
   ------------
	This demo tests DSP on-chip McBSP-0/1/2 ports. 

	This test configures all McBSP pins as OUTPUT pins except for
	DR pin, which is configured as INPUT, and continuously generates 
	'running '1'' over DX/DR, FSX, FSR, CLKX and CLKR pins
	for each McBSP port. Oscilloscope can be used to view pulses at
	pins of McBSP-0/1/2 ports.

	External DX-to-DR-to-CLKS shorting jumpers are equired for each of the
	McBSP-0/1/2 ports in order to run this demo. These shorting 
	jumpers shall be connected to SIO-0/1 ports of SIOX-A rev.B site 
	(McBSP-0/1 ports), and to SIO-1 port of SIOX-B rev.C site (McBSP-2 port). 
	Refer to TORNADO-P6x User's Guide for more details about SIOX site headers.
   	The following external plug-in jumpers are required for correct running of 
	this demo:
		1. External jumper DX0=>DR0/DX0=>CLKS0. Must install into pins #12 and 
		   #11 of SIOX-A rev.B site and pin#3 of on-board S2 socket.
		2. External jumper DX1=>DR1/DX1=>CLKS1. Must install into pins #26 and 
		   #25 of SIOX-A rev.B site and pin#3 of on-board S3 socket.
		3. External jumper DX2=>DX2/DX2=>CLKS2. Installs into pins #1 and #2 
		   of SIOX-B rev.C site and pin#3 of on-board S4 socket.

	On-board SIOX ports configuration SW4 switch have to be configured as 
	the following:
		- SW4-1 must be set to the 'ON' state in order to enable
		  'C64xx McBSP-1 port via SIO-1 port of SIOX-A rev.B site
		- SW4-2 must be set to the 'ON' state in order to enable
		  'C64xx McBSP-2 port via SIO-1 port of SIOX-B rev.C site
		- SW4-3 must be set to the 'ON' state in order to enable
		  'C64xx McBSP-1 port instead of UTOPIA interface


   TORNADO platform:
   -----------------
	This demo does not utilize any host resources.
	TI CCS6000 tools are required to recompile this demo. Either 
	TP6COFF.EXE utility or MIRAGE JTAG emulator are required to 
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
          	- SA-mode or host mode
	  		- no BOOT mode
*****************************************************/

#define		__TP6X_SELECT_TP64XX__	1

#include "c6x_def.h"
#include "tp6x_dsp.h"


#define DX_OUT_0      0x3f00
#define DX_OUT_1      0x3f20
#define FSX_OUT_0     0x3f00
#define FSX_OUT_1     0x3f08
#define FSR_OUT_0     0x3f00
#define FSR_OUT_1     0x3f04
#define CLKX_OUT_0    0x3f00
#define CLKX_OUT_1    0x3f02
#define CLKR_OUT_0    0x3f00
#define CLKR_OUT_1    0x3f01

#define DR_BIT_MASK   0x10
#define CLKS_BIT_MASK 0x40
#define CLKR_BIT_MASK 0x1
#define CLKX_BIT_MASK 0x2
#define FSX_BIT_MASK  0x8
#define FSR_BIT_MASK  0x4


unsigned pass_counter=0;

main()
{
	// initialize DSP EMIF areas
   	TP6X_init_dsp_emif();

   	*C6X_MCBSP0_SPCR = 0;   // configure McBSP-0 for I/O pins
   	*C6X_MCBSP1_SPCR = 0;   // configure McBSP-1 for I/O pins
   	*C6X_MCBSP2_SPCR = 0;   // configure McBSP-2 for I/O pins
	
	// reconfigure CLKS2/GPIO-8 as input pin with CLKS2 function
	C6X_SET_GPIO_ENABLE_RG(0);
	C6X_SET_GPIO_DIR_RG(0);
	

   	while(1)
    {
      	// test McBSP-0 
			// set DX=1
      	*C6X_MCBSP0_PCR = DX_OUT_1;

      	if(((*C6X_MCBSP0_PCR) & DR_BIT_MASK  ) ==0) break; 
      	if(((*C6X_MCBSP0_PCR) & CLKS_BIT_MASK) ==0) break; 

			// set DX=0 
      	*C6X_MCBSP0_PCR = DX_OUT_0;

      	if((*C6X_MCBSP0_PCR & DR_BIT_MASK  ) !=0) break; 
      	if((*C6X_MCBSP0_PCR & CLKS_BIT_MASK) !=0) break; 

      	*C6X_MCBSP0_PCR = FSX_OUT_1;
      	*C6X_MCBSP0_PCR = FSX_OUT_0;

      	*C6X_MCBSP0_PCR = FSR_OUT_1;
      	*C6X_MCBSP0_PCR = FSR_OUT_0;

      	*C6X_MCBSP0_PCR = CLKX_OUT_1;
      	*C6X_MCBSP0_PCR = CLKX_OUT_0;

      	*C6X_MCBSP0_PCR = CLKR_OUT_1;
      	*C6X_MCBSP0_PCR = CLKR_OUT_0;


		// test McBSP-1
			// set DX=1 
      	*C6X_MCBSP1_PCR = DX_OUT_1;

      	if((*C6X_MCBSP1_PCR & DR_BIT_MASK  ) ==0) break; 
      	if((*C6X_MCBSP1_PCR & CLKS_BIT_MASK) ==0) break; 

			// set DX=0 
      	*C6X_MCBSP1_PCR = DX_OUT_0;

      	if((*C6X_MCBSP1_PCR & DR_BIT_MASK  ) !=0) break; 
      	if((*C6X_MCBSP1_PCR & CLKS_BIT_MASK) !=0) break; 

      	*C6X_MCBSP1_PCR = FSX_OUT_1;
      	*C6X_MCBSP1_PCR = FSX_OUT_0;

      	*C6X_MCBSP1_PCR = FSR_OUT_1;
      	*C6X_MCBSP1_PCR = FSR_OUT_0;

      	*C6X_MCBSP1_PCR = CLKX_OUT_1;
      	*C6X_MCBSP1_PCR = CLKX_OUT_0;

      	*C6X_MCBSP1_PCR = CLKR_OUT_1;
      	*C6X_MCBSP1_PCR = CLKR_OUT_0;

      	// test McBSP-2 
			// set DX=1 
      	*C6X_MCBSP2_PCR = DX_OUT_1;

      	if(((*C6X_MCBSP2_PCR) & DR_BIT_MASK  ) ==0) break; 
      	if(((*C6X_MCBSP2_PCR) & CLKS_BIT_MASK) ==0) break; 

			// set DX=0 
      	*C6X_MCBSP2_PCR = DX_OUT_0;

      	if((*C6X_MCBSP2_PCR & DR_BIT_MASK  ) !=0) break; 
      	if((*C6X_MCBSP2_PCR & CLKS_BIT_MASK) !=0) break; 

      	*C6X_MCBSP2_PCR = FSX_OUT_1;
      	*C6X_MCBSP2_PCR = FSX_OUT_0;

      	*C6X_MCBSP2_PCR = FSR_OUT_1;
      	*C6X_MCBSP2_PCR = FSR_OUT_0;

      	*C6X_MCBSP2_PCR = CLKX_OUT_1;
      	*C6X_MCBSP2_PCR = CLKX_OUT_0;

      	*C6X_MCBSP2_PCR = CLKR_OUT_1;
      	*C6X_MCBSP2_PCR = CLKR_OUT_0;

		pass_counter++;
	}


error:
	// trap here on any error
    while(1) ;

}



