/************************************************************************
  DSP on-chip TIMERs test for TP6x.
  (C) MicroLAB Systems, 1999-2002

	Description:
	------------	
		This test uses 'C6x DSP on-chip TIMER I/O pins as I/O port. This
		test verifies read-back TIN input port data against set TOUT output
		port data. TM0 has longer '0' state duration for better 
		synchronization.

  		This test requires the TOUT enabled for each DSP on-chip timer via
		on-board jumpers/switches.

		The following on-board DSP TIMERs configuration jumpers and switches 
		shall be installed and configured properly for correct operation of 
		this test:
			- TP62/TP67 boards: J5 and J6 shall be installed
			- TP6202/TP6203 boards: SW4-3 and SW4-4 shall be set to the 'ON' state


   TORNADO platform:
   -----------------
		This demo does not utilize any host resources.
		TI CCS6000 tools are required to recompile this demo. Either 
		TP6COFF.EXE utility or MIRAGE JTAG emulator are required to 
		upload code to the on-board DSP environment.

  Copyright:
  ----------
		This demo is distributed free of charge and as it is without
		any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
          	- SA-mode or host mode
	  		- no BOOT mode
*************************************************************************/

#include "c6x_def.h"
#include "tp6x_dsp.h"


#define TM_OUT_0  0
#define TM_OUT_1  4

unsigned pass_counter=0;

main()
{
	// initialize DSP EMIF areas
	// configure on-board RAM as SBSRAM for TP6202/TP6203 (has no matter for this test)
   	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);

   	*C6X_TM0_CONTROL=0;
   	*C6X_TM1_CONTROL=0;

   	while(1)
    {
    	// test TIMER-0.
		// Repeat this two times for external synchronization.
			// set TOUT=0
      	*C6X_TM0_CONTROL = TM_OUT_0;
			// check for TINP==TOUT
      	if((*C6X_TM0_CONTROL & 8) !=0) break; 
      	*C6X_TM0_CONTROL = TM_OUT_0;
			// check for TINP==TOUT
      	if((*C6X_TM0_CONTROL & 8) !=0) break; 
			// set TOUT=1 
      	*C6X_TM0_CONTROL = TM_OUT_1;
			// check for TINP==TOUT 
      	if((*C6X_TM0_CONTROL & 8) ==0) break; 


		// test TIMER-1
			// set TOUT=0
      	*C6X_TM1_CONTROL = TM_OUT_0;
			// check for TINP==TOUT 
      	if((*C6X_TM1_CONTROL & 8) !=0) break; 
			// set TOUT=1
      	*C6X_TM1_CONTROL = TM_OUT_1;
			// check for TINP==TOUT 
      	if((*C6X_TM1_CONTROL & 8) ==0) break; 


		pass_counter++;
    }


error:
	// get here on any error
    while(1) ;

}


