/**********************************************************************

	TP6x DPRAM test from host PC.
	(C) MicroLAB Systems, 1999-2002

	Notes:
		1. This demo runs under DOS and Windows 9x only with DPRAM
		   area allocation into PC UMB memory area.
		2. For TP64xx boards, this tests performs testng of allocated
		   8Kx32 DPRAM sub-area only.
		3. This test automatcally selects TP6x board #0 for testing.

***********************************************************************/

#include <stdio.h>
#include <dos.h>
#include <stdlib.h>

#include <tp6x_h.h>

#include <ips_ut.h>

unsigned long	START_SADDR = 0L,
                LAST_SADDR  = 0L;

unsigned long sa;
unsigned int page;
unsigned pass=0, mode=0;
unsigned long	data,*D,data_start, data_rd,
                initial_data = 0xa5f1e251L,
				data_inc     = 0x13579bdfL;
int ex_status;

#define CLEAR_CURRENT_LINE  printf("\r%79s\r"," ");


char err_mess[100];

#define BOARD_ID	0

int boards;
unsigned long _io0,_io1, _io3, _io4;
unsigned dev_id, index;
char _io0_name[40],_io1_name[40],_io3_name[40],_io4_name[40], dev_name[40];

unsigned long dpram_len;


 main()

 {
   int r,i;
   unsigned smp_r,smp_w,smp_len;


   	printf("\nTORNADO-P6x Host PCI I/F DPRAM Test. v.1.2\n");
   	printf("Copyright (C) MicroLAB Systems Ltd, 1999-2002\n\n");

	if(TP6X_open()!=TP6X_OK)
   	{
exit_err:
		TP6X_last_err_message(err_mess);
        printf("err: %i  (%s)\n",TP6X_last_err(),err_mess);
		goto exit;
    }
   	printf("boards installed: %i.  Selecting board #0...\n", TP6X_boards());

   	TP6X_sel_board_id(BOARD_ID);
   	TP6X_board_info(BOARD_ID,&dev_id,&index,NULL,NULL,&_io0,&_io1,NULL,&_io3,&_io4);
   	TP6X_dev_name(dev_name);
   	TP6X_area_alloc_name(TP6X_PCIC_AREA,_io0_name);
   	TP6X_area_alloc_name(TP6X_ECC_AREA, _io1_name);
   	TP6X_area_alloc_name(TP6X_DPM_AREA, _io3_name);
   	TP6X_area_alloc_name(TP6X_HIF_AREA, _io4_name);
   	if(TP6X_err_flag()) goto exit_err;

   	printf("TP6x device id = %x (%s)     index=%x\n",dev_id, dev_name, index);
   	printf("TP6x AMCC registers: %lxH (%s)\n",_io0,_io0_name);
   	printf("TP6x ECC area      : %lxH (%s)\n",_io1,_io1_name);
   	printf("TP6x DPRAM area    : %lxH (%s)\n",_io3,_io3_name);
   	printf("TP6x Host I/O area : %lxH (%s)\n",_io4,_io4_name);

	printf("TP6x on-board DPRAM: %lxH %s\n",TP6X_dpram_len(),TP64XX_BOARD ? "(only 8Kx32 sub-area will be tested)":"");
   	printf("TP6x on-board SRAM : %lxH\n",TP6X_sram_len());
   	printf("TP6x on-board SDRAM: %lxH\n",TP6X_dram_len());


   	printf("\n");


	// set DPRAM end address upon the board type. For TP64XX, only allocated 8Kx32 DPRAM 
	// sub-area can be processed by TP6X DOS utilities.
	dpram_len = (TP64XX_BOARD ? TP64XX_DPM_DPRAM_LEN_MAX:TP6X_dpram_len());

   	printf("start DPRAM address [00000H..%lxH]: ",dpram_len-1L);
   	START_SADDR=gethex(0L,(dpram_len-1L),0L,CR_ENABLE,ESC_MASK,&ex_status);

   	START_SADDR= START_SADDR & ~0x3L;	// allloc at DWORD boundary
   	if(ex_status==ESC_MASK)
    {  
		CLEAR_CURRENT_LINE;
        goto exit;
    }

   	printf("end DPRAM address   [%5lxH..%lxH]: ",START_SADDR,(dpram_len-1L));
   	LAST_SADDR=gethex(START_SADDR,(TP6X_dpram_len()-1L),(dpram_len-1L),CR_ENABLE,ESC_MASK,&ex_status);

   	LAST_SADDR= LAST_SADDR | 0x3L;	// allloc at DWORD boundary

   	if(ex_status==ESC_MASK)
    {  
		CLEAR_CURRENT_LINE;
        goto exit;
	}

   	printf("\npress ESC to abort and exit to DOS ...\n\n");

   	if(TP62XX_TP67_BOARD) TP6X_CLR_DPRAM_ERR;
   	if(TP6X_err_flag()) goto exit_err;

   	data_start = initial_data;

   	pass=0;	// set params for infinite loop
   	mode=0;

	while(1)
   	{
		CLEAR_CURRENT_LINE;

		// For TP62xx/TP67 boards with SMP register, select different SMP page modes
   		if(TP62XX_TP67_BOARD) 
		{
       		switch(mode)
 			{
	  			case 0:
	      			printf("SPA mode via SMP-0: ");
	      			smp_r=TP6X_SMP_SMP0_SEL;
	      			smp_w=TP6X_SMP_SMP0_SEL;
	      			smp_len=TP6X_SMP_SPA_SMP_LEN;
	      			TP6X_SET_SMP_MODE_SPA;
	      			break;

          		case 1:
	      			printf("DPA mode via SMP-0: ");
	      			smp_r=TP6X_SMP_SMP0_SEL;
	      			smp_w=TP6X_SMP_SMP0_SEL;
	      			smp_len=TP6X_SMP_DPA_SMP_LEN;
	      			TP6X_SET_SMP_MODE_DPA;
	      			break;

          		case 2:
	      			printf("DPA mode via SMP-1: ");
	      			smp_r=TP6X_SMP_SMP1_SEL;
	      			smp_w=TP6X_SMP_SMP1_SEL;
	      			smp_len=TP6X_SMP_DPA_SMP_LEN;
	      			TP6X_SET_SMP_MODE_DPA;
	      			break;

          		case 3:
	      			printf("DPA mode via SMP-0/1: ");
	      			smp_r=TP6X_SMP_SMP0_SEL;
	      			smp_w=TP6X_SMP_SMP1_SEL;
	      			smp_len=TP6X_SMP_DPA_SMP_LEN;
	      			TP6X_SET_SMP_MODE_DPA;
	      			break;
	 		}
		}


		//--- write data
		data = data_start;
        TP6X_set_smp_ptr(START_SADDR,TP6X_SMP_DPRAM_AREA,smp_w,&D);
	   	if(TP6X_err_flag()) goto exit_err;

		// Provide two different write paths separately for TP62xx/TP67 and TP64xx boards 
		// in order to exclude any extra operations for TP64XX boards and have TP64xx
		// boards to use PCI BURSTS during writes.
   		if(TP62XX_TP67_BOARD) 
    	{
			// this code is executed for TP62xx/TP67 boards
			for (sa=START_SADDR;sa<=LAST_SADDR;sa+=4L)
	    	{
				if((sa%smp_len)==0)	
		  		{
		     		TP6X_set_smp_ptr(sa,TP6X_SMP_DPRAM_AREA,smp_w,&D);
		     		if((sa%(smp_len/2))==0) printf("w");
		  		}

				*(D++)=data++;

				if(TP6X_DPRAM_ERR_DATA!=0)
		  		{
		    		printf("\nerror: DPRAM error on write to @%lxH\n\n",sa);
		    		goto exit;
		  		}
			}

			if(TP6X_err_flag()) goto exit_err;
		}
		else
		{
			// this code is executed for TP64xx boards
			printf("w");
			for (sa=START_SADDR;sa<=LAST_SADDR;sa+=4L, *(D++)=data++);
		}

		// check for whether ESC has been pressed	   
		if( key_pressed() && read_char()==KEY_ESC)
       	{
user_aborted:		  
         	printf("\nUser aborted. status: OK\n\n");
         	goto exit;
       	}

		//----- verify written data
		data = data_start;
        TP6X_set_smp_ptr(START_SADDR,TP6X_SMP_DPRAM_AREA,smp_r,&D);
	   	if(TP6X_err_flag()) goto exit_err;
        
		// Provide two different read paths separately for TP62xx/TP67 and TP64xx boards 
		// in order to exclude any extra operations for TP64XX boards and have TP64xx
		// boards to use PCI BURSTS during reads.
   		if(TP62XX_TP67_BOARD) 
    	{
			// this code is executed for TP62xx/TP67 boards
			for (sa=START_SADDR;sa<=LAST_SADDR;sa+=4L)
	    	{
				if((sa%smp_len)==0)	
		  		{
		     		TP6X_set_smp_ptr(sa,TP6X_SMP_DPRAM_AREA,smp_r,&D);
		     		if((sa%(smp_len/2))==0) printf("r");
		  		}

				data_rd=*(D++);

				if(TP6X_DPRAM_ERR_DATA!=0)
		  		{
		    		printf("\nerror: DPRAM error at read from @%lxH\n\n",sa);
		    		goto exit;
		  		}

				if(data_rd!=data++)
		  		{
verify_error:
		    		printf("\nerror: A=%lxH data_wr=%lxH  data_rd=%lxH\n\n",sa, --data, data_rd );
		    		goto exit;
		  		}
			}
		}
		else
		{
			// this code is executed for TP64xx boards
			printf("r");
			for (sa=START_SADDR;sa<=LAST_SADDR;sa+=4L)
	    	{
				data_rd=*(D++);
				if(data_rd!=data++) goto verify_error;
			}
		}

		// check for ESC key has been pressed
		if( key_pressed() && read_char()==KEY_ESC)
			goto user_aborted;

		if(++pass>=10)
	  	{
	    	pass=0;
	    	if(++mode>3) mode=0;
	  	}

       	data_start += data_inc ;
	}
	
exit:
	if(TP62XX_TP67_BOARD) 
   	{
    	TP6X_SET_SMP0_RG(0);
    	TP6X_SET_SMP1_RG(0);
    	TP6X_CLR_DPRAM_ERR;
	}
}





