/**********************************************************************************
   	Host-to-DSP Communication Test for TP64XX.
   	(C) MicroLAB Systems, 1998-2002

   	file: C64xx DSP kernel

   	This utility performs test of communication between on0board DSP and host PCI 
   	interface of TORNADO-P6x board.

	Notes:
		1. For TP64xx boards, using host TP6x DOS utilities, allocation of
		   DPRAM sub-window, which is accessed from host PCI-bus, is not known in 
		   advance for on-board DSP. Therefore, DSP memory location (@0x1fc)
		   is reserved for host-to-DSP transfer of DPRAM sub-area base address.

		2. This executable does not run for TP62xx/TP67 boards.

***********************************************************************************/

#define	__TP6X_SELECT_TP64XX__		1
#define	__TP6X_DPRAM256K__			1
#include "c6x_def.h"
#include "tp6x_dsp.h"



#define	DPRAM_SUBAREA_BADDR_ADDR	0x1fc	// DSP address of location for passing DPRAM sub-area base address
#define	MCD_ADDR_WIN				(TP64XX_DPRAM_SADDR+0x200)	// MCD address for TSDK tests

#define	M_ID						0x110
#define	PRG_ID						0x111

#define	MCMND_NOP					0
#define	MCMND_SRAM_COMM				1
#define	MCMND_IRQ_DPRAM_COMM		2
#define	MCMND_IRQ_HM0_COMM			3
#define	MCMND_IRQ_HM1_COMM			4
#define	MCMND_IRQ_AOB_COMM			5
#define	MCMND_HPI_HINT_COMM			6
#define	MCMND_DPSEM_COMM			7
#define	MCMND_IRQ_RFIFO_COMM		8
#define	MCMND_IRQ_WFIFO_COMM		9

#define	MSTAT_SYNC					3
#define	MSTAT_BUSY					2
#define	MSTAT_ATTN					1
#define	MSTAT_RDY					0

#define	ERR_OK						0
#define	ERR_INV_CMND				-2
#define	ERR_INV_PARAM				-1
#define	ERR_CMND_EXEC_ERR			-3
#define	ERR_INV_INIT				-100


struct MCD_DD
{ 
	int  		m_id,
				prg_id,
				cmnd,
				status,
				error;

   	unsigned    up[4];
   	unsigned    dpram_len;
 };

struct MCD_DD *MCD;

unsigned 		*dpram_hm_rq=0, 
				*dpram_mh_rq=0,
				dpram_irq_support=TP6X_OFF;



//--------------------- IRQs -----------------------

unsigned nmi_count=0;
				
//!!!--- NMI IRQ declaration via pragma NMI_INTERRUPT is required
//!!!--- in order to exit via NRP for NMI instead of IRP for maskable IRQ
//#pragma NMI_INTERRUPT (c_nmi)

interrupt void c_nmi()		// AOB-IRQ via IMBX-0
{ 
	unsigned d;

  	if(MCD->cmnd==MCMND_IRQ_AOB_COMM)
	{
    	MCD->up[0]=0x01234567;
	  	MCD->up[1]=0x89abcdef;

		d=TP6X_pcic_imbx(0);

	  	TP6X_set_pcic_aint(0x10013); // clear AOB-INT on IMBX-0

	  	C6X_PGINT_ENABLE;	//!!! remove this if NMI_INTERRUPT pragma is supported
	  	C6X_NMI_ENABLE;	//!!! remove this if NMI_INTERRUPT pragma is supported

	  	MCD->up[2]=d;
	  	MCD->up[3]= ++nmi_count;
	  	MCD->error=ERR_OK;
	  	TP6X_set_pcic_ombx(0,~d);
	}
}



interrupt void c_mirq0()	// hm_rq0/1
{ 
	unsigned d;

  	if(MCD->cmnd==MCMND_IRQ_HM0_COMM)
	{
    	MCD->up[0]=0x76543210;
	  	MCD->up[1]=0xfedcba98;
        d=TP6X_hm_rq0();
        TP6X_clr_hm_rq0();
	  	MCD->up[2]=d;
	  	MCD->up[3]=(~d & 0xf);
	  	MCD->error=ERR_OK;
	  	TP6X_set_mh_rq();
	}

  	if(MCD->cmnd==MCMND_IRQ_HM1_COMM)
	{
    	MCD->up[0]=0xfedcba98;
	  	MCD->up[1]=0x76543210;
        d=TP6X_hm_rq1();
        TP6X_clr_hm_rq1();
	  	MCD->up[2]=(~d & 0xf);
	  	MCD->up[3]=d;
	  	MCD->error=ERR_OK;
	  	TP6X_set_mh_rq();
	}
}



int rfifo_count=0;

interrupt void c_mirq1()	// FIFO-RD/WR
{ 
	unsigned d;

  	if(MCD->cmnd==MCMND_IRQ_RFIFO_COMM)
	{
		MCD->up[0]=0x55555555;
	  	MCD->up[1]=0xaaaaaaaa;
	  	d=TP6X_pcic_rfifo();
	  	TP6X_set_pcic_wfifo(~d);
	  	MCD->up[2]=d;
	  	MCD->up[3]= (rfifo_count++);
	  	MCD->error=ERR_OK;
	}
}


int wfifo_count=0;

interrupt void c_mirq2()        // FIFO-WR
{ 
	unsigned d;

  	if(MCD->cmnd==MCMND_IRQ_WFIFO_COMM)
	{
	  	MCD->up[0]=0xaaaaaaaa;
        MCD->up[1]=0x55555555;
	  	d= (0xcdef0000 | wfifo_count);
	  	TP6X_set_pcic_wfifo(d);
	  	MCD->up[2]=d;
	  	MCD->up[3]=(wfifo_count++);
	  	MCD->error=ERR_OK;
	}
}

// DPRAM HM IRQ
// This interrupt is enabled for TP64xx boards with 256Kx32 DPRAM only.
interrupt void c_mirq3()	
{
  	if(MCD->cmnd==MCMND_IRQ_DPRAM_COMM)
	{ 
		MCD->up[0]=0x01230000;
	  	MCD->up[1]=0x4567;
	  	MCD->up[2]=0x89ab0000;
	  	MCD->up[3]=0xcdef;
	  	*dpram_mh_rq = (~*dpram_hm_rq);
	  	MCD->error=ERR_OK;
	}
}
//---------------------------------------------------------------




int init_cpu()
{ 
	// init C6x EMIF, configure external memory as SBSRAM (has no matter for this test)
	TP6X_init_dsp_emif();

	// clear DSPINT bit
	TP6X_clr_dspint();			
	return(0);
}



int init_irq()
{ 
	C6X_XINT_ALL_POLARITY_SET(C6X_XINT_POLARITY_POS);
	C6X_INT_DISABLE_ALL;

	// clear all MBX/FIFO flags
	TP6X_set_pcic_agcsts(0x0e000000);	

	// check for AOB-IN is active and clear its source
	if((TP6X_pcic_aint() & 0x800000)!=0)	
	TP6X_set_pcic_aint(TP6X_pcic_aint() & 0x3f0000);

	if((TP6X_pcic_aint() & 0x800000)!=0) return(-1); // incorrect init	

	// clear DPRAM_HM_IRQ if these are supported
	if(dpram_irq_support==TP6X_ON)
		*dpram_hm_rq = ~(*dpram_hm_rq);	

	// enable AOB_INT on IMBX-0
	TP6X_set_pcic_aint(0x13); 

	// now configure external IRQ sources
	TP6X_set_mirq_sel(TP6X_MIRQ0,TP6X_MIRQ_SEL_HM_RQ0);	// HM_RQ0/1
	TP6X_set_mirq_sel(TP6X_MIRQ1,TP6X_MIRQ_SEL_RFIFO);	// RFIFO
	TP6X_set_mirq_sel(TP6X_MIRQ2,TP6X_MIRQ_SEL_OFF);	// WFIFO

		// enable MDPIRQ if it is supported
	if(dpram_irq_support==TP6X_ON)
		TP6X_set_mirq_sel(TP6X_MIRQ3,TP6X_MIRQ_SEL_MDPIRQ);
	else
		TP6X_set_mirq_sel(TP6X_MIRQ3,TP6X_MIRQ_SEL_OFF);

	TP6X_set_mnmi_sel(TP6X_MIRQ_SEL_PCIC);		// AOB@IMBX0

	C6X_INT_CLEAR_ALL;
   	C6X_INT_ENABLE(4);
   	C6X_INT_ENABLE(5);
   	C6X_INT_ENABLE(6);
   	C6X_INT_ENABLE(7);
   	C6X_GINT_ENABLE;    // set GIE bit

	return(0);
}


main()
{
	unsigned *dpram_subarea_addr_ptr;

	// perform DSP initialization, exit if error
   	if(init_cpu()==-1) goto error;	

	// now wait until host will pass calculated base address for DPRAM sub-area
	// using HPI port. Initially clear this memory location, and then poll this
	// location for !=0 value, which is set by host PC
	dpram_subarea_addr_ptr = (unsigned *)DPRAM_SUBAREA_BADDR_ADDR;
	*dpram_subarea_addr_ptr = 0;
	while(*dpram_subarea_addr_ptr ==0);

	// host PC looks to pass DPRAM sub-area base address to DSP
	// assign it t the MCP pointer
	MCD = (struct MCD_DD *) (*dpram_subarea_addr_ptr);

	// now fill-in MCD for back-PC validation
	MCD->m_id=M_ID;
    MCD->prg_id=PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->up[0]=0x98765432;
	MCD->up[1]=0xabcdef01;

	// now check whether DPRAM IRQs are supported via MCD->dpram_len parameter
	if(MCD->dpram_len>0x80000)
	{	
		// this is 256Kx32 DPRAM and higher, DPRAM IRQs are supported. Assign DPRAM IRQ pointers.
		dpram_irq_support=TP6X_ON;
		dpram_hm_rq = (unsigned *)(unsigned)(TP64XX_DPRAM_SADDR+ MCD->dpram_len -4L);
		dpram_mh_rq = (unsigned *)(unsigned)(TP64XX_DPRAM_SADDR+ MCD->dpram_len -8L);
	}

	// now initialize interrupts
	if(init_irq()==-1) goto error;	
	
	// finally, set status to READY for host PC
	MCD->status=MSTAT_RDY;

	// enter command loop
   	while(1)
    {
     	while(MCD->status !=MSTAT_ATTN);
       	MCD->status=MSTAT_BUSY;

       	switch(MCD->cmnd)
		{
          	case MCMND_NOP:
	       		MCD->error=ERR_OK;
	       		break;

          	case MCMND_SRAM_COMM:
	       		MCD->up[0]=0xfedc0000;
	       		MCD->up[1]=0xba98;
	       		MCD->up[2]=0x76540000;
	       		MCD->up[3]=0x3210;
	       		MCD->error=ERR_OK;
	       		break;

	  		case MCMND_IRQ_HM0_COMM:
				// wait for synch flag
	       		while(MCD->status !=MSTAT_SYNC) ;	
	       		TP6X_set_mirq_sel(TP6X_MIRQ0,TP6X_MIRQ_SEL_HM_RQ1);
	       		break;


          	case MCMND_HPI_HINT_COMM:
	       		TP6X_clr_dspint();
	       		TP6X_set_hint();
	       		MCD->up[0]=0x77221144;
	       		MCD->up[1]=~MCD->up[0];
	       		MCD->up[2]=0x90785634;
	       		MCD->up[3]=0x21436587;
               	MCD->error=ERR_OK;
	       		break;


	  		case MCMND_IRQ_WFIFO_COMM:
	       		TP6X_set_mirq_sel(TP6X_MIRQ2,TP6X_MIRQ_SEL_WFIFO);
				// wait for synch flag
	       		while(MCD->status !=MSTAT_SYNC) ;	
	       		TP6X_set_mirq_sel(TP6X_MIRQ2,TP6X_MIRQ_SEL_OFF);
	       		break;

			default:
	       		MCD->error=ERR_INV_CMND;
	       		break;

		}
       	MCD->status=MSTAT_RDY;
	}

error:
      while(1);
}



