/***************************************************************************
	Host HPI test for TP6x boards. v.2.2
	(C) MicroLAB Systems, 1998-2004

	Notes:
		1. This test performs test of memory inside the DSP environment
		   by host via HPI port of TORNADO-P6x board.
		2. TP62xx/TP67 boards shall be configured in NO-BOOT mode via
		   on-board jumpers/switch settings.
		3. C64xx DSP at TP64xx boards will be put into HPI BOOTMODE in
		   order to perform this test.

****************************************************************************/

#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <conio.h>

#include <ips_ut.h>
#include "tp6x_h.h"

unsigned long	START_SADDR = 0x0L,
                LAST_SADDR  = 0x0L;

unsigned long sa;
unsigned long 	data,data_start, data_rd,
	 			data_x,data_start_x, data_rd_x,data_ccl_x,
         		initial_data   = 0xe97fa5f7l,
         		initial_data_x = 0x39f0e7ael,
	 			data_inc     	= 0xb5f13fe9l,
	 			data_inc_x     = 0x491fe3adl;
int ex_status;

#define CLEAR_CURRENT_LINE  printf("\r%79s\r"," ");

unsigned sn;


char err_mess[100];

#define BOARD_ID	0

int boards;
unsigned long _io0,_io1, _io3, _io4;
unsigned dev_id, index;
char _io0_name[40],_io1_name[40],_io3_name[40],_io4_name[40], dev_name[40];
unsigned char rev;
unsigned fw_rev;

unsigned long *D;
unsigned smp;
unsigned long ccl;

unsigned long dm_ccl, dm_addr_step,DM_CCL_TOTAL;


	//-- DSP SYS STAT register, which is used to select mem type for TP6202/TP6203 via bit XMEM (D2)
	//   SYS STATUS reg @TP6202/TP6203: XMEM (D2) bit: (r/w): SBSRAM (=0) or SDRAM (=1) external memory selector
#define TP6X_DSP_SYS_STAT_ADDR			0x000c0058L
#define	TP6x_DSP_SYS_STAT_XMEM_BITMASK	4L			
#define	TP6x_DSP_XMEM_SBSRAM			0L	
#define	TP6x_DSP_XMEM_SDRAM				4L


_write_hpi(unsigned long a,unsigned long d)
{ 
	unsigned long a_rd;

   	TP6X_SET_HPIA(a);
   	if((a_rd=TP6X_HPIA_DATA)!=a)
    { 
		printf("\nerror-CNF: error setting HPIA register to %lXH (reads as %lxH)\n",a,a_rd);
       	return(-1);
    }
   	TP6X_SET_HPID(d);
   	if(TP6X_HPI_ERR_DATA!=0)
   	{ 
		printf("\nerror-CNF: HPI timeout error at write to HPI @%lXH\n",a);
       	return(-1);
   	}
   	return(0);
}


_print_hpi_data(unsigned long a)
{ 
	unsigned long a_rd;

   	TP6X_SET_HPIA(a);
   	if((a_rd=TP6X_HPIA_DATA)!=a)
    { 
		printf("\nerror-CNF: error setting HPIA register to %lXH (reads as %lxH)\n",a,a_rd);
       	return(-1);
    }

   	data_rd=TP6X_HPID_DATA;
   	if(TP6X_HPI_ERR_DATA!=0)
    { 
		printf("\nerror-CNF: HPI timeout error at read fm HPI @%lXH\n",a);
       	return(-1);
    }
	printf("  HPI-A=%.8lx  data=%.8lx\n",a,data_rd);
   	return(0);
}


main()
{
	int r;
	unsigned ux;

   	printf("\nTORNADO-P6x HPI test. v.2.2\n");
   	printf("Copyright (C) MicroLAB Systems Ltd, 1996-2004\n\n");

   	if(TP6X_open()!=TP6X_OK)
    {
exit_err:
		TP6X_last_err_message(err_mess);
        printf("err: %i  (%s)\n",TP6X_last_err(),err_mess);
		goto exit;
    }
   
	printf("boards installed: %i.  Selecting board #0...\n", TP6X_boards());

   	TP6X_sel_board_id(BOARD_ID);
   	TP6X_board_info(BOARD_ID,&dev_id,&index,&rev,NULL,&_io0,&_io1,NULL,&_io3,&_io4);
   	TP6X_dev_name(dev_name);
   	fw_rev=TP6X_dev_fw_rev();
   	TP6X_area_alloc_name(TP6X_PCIC_AREA,_io0_name);
   	TP6X_area_alloc_name(TP6X_ECC_AREA, _io1_name);
   	TP6X_area_alloc_name(TP6X_DPM_AREA, _io3_name);
   	TP6X_area_alloc_name(TP6X_HIF_AREA, _io4_name);
   	if(TP6X_err_flag()) goto exit_err;

   	printf("TP6x device id = %x (%s rev.%x-%x)     index=%x\n",dev_id, dev_name, (unsigned)rev,fw_rev,index);
   	printf("TP6x AMCC registers: %lxH (%s)\n",_io0,_io0_name);
   	printf("TP6x ECC area      : %lxH (%s)\n",_io1,_io1_name);
   	printf("TP6x DPRAM area    : %lxH (%s)\n",_io3,_io3_name);
   	printf("TP6x Host I/O area : %lxH (%s)\n",_io4,_io4_name);
   	printf("TP6x on-board DPRAM: %lxH\n",TP6X_dpram_len());
   	printf("TP6x on-board SRAM : %lxH\n",TP6X_sram_len());
   	printf("TP6x on-board SDRAM: %lxH\n",TP6X_dram_len());


   	TP6X_SET_MRESET_MODE_HOST;
   	TP6X_SET_MRESET;
   	if((r=TP6X_SET_EMU_SEL_XEMU)!=TP6X_OK ) goto exit_err;


	// TP62xx/TP67 boards must be configured via on-board jumpers/swict to have on-board DSP
	// to start in NO-BOOT mode. In order DSP on-chip HPI port could run, special LOOP code
	// below must be loaded into DSP and have DSP to execute this loop code inside program cache.
	// Load DSP LOOP program (C6X_LOOP.ASM). This program simply loops with CACHE enabled
	if(TP62XX_TP67_BOARD)
	{
   		TP6X_CLR_DPRAM_ERR;
   		TP6X_SET_SMP_MODE_SPA;
   		TP6X_set_smp_ptr(0L,TP6X_SMP_DPRAM_AREA,TP6X_SMP_SMP0_SEL,&D);
   		if(TP6X_err_flag()) goto exit_err;

		*D++=0xa02a;	// DPRAM addr#0
		*D++=0x6a;
		*D++=0x008003a2;
		*D++=0x0;
		*D++=0x0;       // DPRAM addr#10
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;       // DPRAM addr#20
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;       // DPRAM addr#30
		*D++=0x0;
		*D++=0x0;
		*D++=0x12;
		*D++=0x0;       // DPRAM addr#40
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;       // DPRAM addr#50
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;

		if(TP6X_DPRAM_ERR_DATA!=0)
		{
			printf("\nerror: DPRAM access error\n");
		    goto exit;
		}
	}
	else
	{
		// TP64xx on-board DSP must be configured to start in HPI BOOTMODE via
		// HIF_DSP_BMODE_RG register in order to make HPI port running after release of
		// DSP RESET.
		TP6X_SET_DSP_BMODE_HPI;

		if(!TP6X_DSP_BMODE_IS_HPI)
		{
			printf("\nerror: cannot set DSP HPI bootmode for TORNADO-P64xx board\n");
		    goto exit;
		}
	}


	//---------- release DSP reset and initialize HPI port
   	TP6X_SET_MGO;
	TP6X_HPI_INIT;

   	if(		TP62_TP67_BOARD && ((ux=TP6X_HPIC_DATA)!=0x0009)
		||	TP64XX_BOARD && ((ux=TP6X_HPIC_DATA)!=0x0008)
	  )
	{ 
			printf("\nerror: error initializing HPIC register (reads as %xH)\n",ux);
		    goto exit;
	}
  	if(TP6X_HPI_ERR_DATA!=0)
	{ 
		printf("\nerror: HPI timeout error at HPI init\n");
	    goto exit;
	}

	//--- now configure EMIF registers dependng upon the TORNADO board type 
	//    for proper access via DSP to EMIF 

   	if(TP62_BOARD)	// configure EMIF for TP62 board
    {
		if(    _write_hpi(C6X_EMIF_GCR_RG,TP62_DSP_EMIF_GCR_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE0_RG,TP62_DSP_EMIF_CE0_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE1_RG,TP62_DSP_EMIF_CE1_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE2_RG,TP62_DSP_EMIF_CE2_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE3_RG,TP62_DSP_EMIF_CE3_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_SDRAM_RG,TP62_DSP_EMIF_SDRAM_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_SDRAM_TIMER_RG,TP62_DSP_EMIF_SDRAM_TIMER_DATA)!=0
   	  	  ) 
			goto exit;
    }

   	if(TP67_BOARD)	// configure EMIF for TP67 board
    {
		if(    _write_hpi(C6X_EMIF_GCR_RG,TP67_DSP_EMIF_GCR_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE0_RG,TP67_DSP_EMIF_CE0_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE1_RG,TP67_DSP_EMIF_CE1_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE2_RG,TP67_DSP_EMIF_CE2_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE3_RG,TP67_DSP_EMIF_CE3_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_SDRAM_RG,TP67_DSP_EMIF_SDRAM_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_SDRAM_TIMER_RG,TP67_DSP_EMIF_SDRAM_TIMER_DATA)!=0
   	  	  ) 
			goto exit;
    }

   	if(TP6202_TP6203_BOARD)	// configure configure CE-2/3 for SBSAM initially for TP6202/TP6203 board
    {
		if(    _write_hpi(TP6X_DSP_SYS_STAT_ADDR,TP6x_DSP_XMEM_SBSRAM)!=0) goto exit;
    }


   	if(TP6202_BOARD)	// configure EMIF for TP6202 board (configure CE-2/3 for SBSAM initially)
    {
		if(    _write_hpi(C6X_EMIF_GCR_RG,TP6202_DSP_EMIF_GCR_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE0_RG,TP6202_DSP_EMIF_CE0_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE1_RG,TP6202_DSP_EMIF_CE1_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE2_RG,TP6202_DSP_EMIF_CE2_SBSRAM_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE3_RG,TP6202_DSP_EMIF_CE3_SBSRAM_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_SDRAM_RG,TP6202_DSP_EMIF_SDRAM_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_SDRAM_TIMER_RG,TP6202_DSP_EMIF_SDRAM_TIMER_DATA)!=0
   	  	  ) 
			goto exit;
    }

   	if(TP6203_BOARD)	// configure EMIF for TP6202 board (configure CE-2/3 for SBSAM initially)
    {
		if(    _write_hpi(C6X_EMIF_GCR_RG,TP6203_DSP_EMIF_GCR_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE0_RG,TP6203_DSP_EMIF_CE0_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE1_RG,TP6203_DSP_EMIF_CE1_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE2_RG,TP6203_DSP_EMIF_CE2_SBSRAM_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_CE3_RG,TP6203_DSP_EMIF_CE3_SBSRAM_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_SDRAM_RG,TP6203_DSP_EMIF_SDRAM_DATA)!=0
	    	|| _write_hpi(C6X_EMIF_SDRAM_TIMER_RG,TP6203_DSP_EMIF_SDRAM_TIMER_DATA)!=0
   	  	  )
			goto exit;
    }

   	if(TP64XX_BOARD)	// configure EMIF-A/B registers for TP64xx boards upon the f/w rev and SDRAM capacity
    {
    	// configure EMIF-A/B registers, which do not depend upon the f/w rev. and SDRAM capacity
		if(	   _write_hpi(C6X_EMIFA_GCR_RG,TP64XX_DSP_EMIFA_GCR_DATA)!=0
	    	|| _write_hpi(C6X_EMIFA_CE0_RG,TP64XX_DSP_EMIFA_CE0_DATA)!=0
	    	|| _write_hpi(C6X_EMIFA_CE0_SEC_RG,TP64XX_DSP_EMIFA_CE0_SEC_DATA)!=0
	    	|| _write_hpi(C6X_EMIFA_CE1_RG,TP64XX_DSP_EMIFA_CE1_DATA)!=0
	    	|| _write_hpi(C6X_EMIFA_CE1_SEC_RG,TP64XX_DSP_EMIFA_CE1_SEC_DATA)!=0
	    	|| _write_hpi(C6X_EMIFA_CE2_RG,TP64XX_DSP_EMIFA_CE2_DATA)!=0
	    	|| _write_hpi(C6X_EMIFA_CE2_SEC_RG,TP64XX_DSP_EMIFA_CE2_SEC_DATA)!=0
	    	|| _write_hpi(C6X_EMIFA_CE3_RG,TP64XX_DSP_EMIFA_CE3_DATA)!=0
			|| _write_hpi(C6X_EMIFB_GCR_RG,TP64XX_DSP_EMIFB_GCR_DATA)!=0
	    	|| _write_hpi(C6X_EMIFB_CE2_RG,TP64XX_DSP_EMIFB_CE2_DATA)!=0
   	  	  )
			goto exit;

			// process the f/w rev. and the SDRAM capacity
			// check the TP64xx f/w revision
		if(!TP64XX_BOARD_FW_REV1A && !TP64XX_BOARD_FW_REV1A1 && !TP64XX_BOARD_FW_REV1A2 & !TP64XX_BOARD_FW_REV1B)
			goto exit;

		if(TP64XX_BOARD_FW_REV1A)
		{

				// configure EMIF-A SDRAM control regs upon the SDRAM size installed
			if( 	(		TP6X_dram_len()==0x2000000L 	// this configuration is for 4Mx64 SDRAM
					  &&	(	 _write_hpi(C6X_EMIFA_SDRAM_RG,TP64XX_DSP_EMIFA_SDRAM4M_DATA_REV1A_600MHZ)!=0
	    					  || _write_hpi(C6X_EMIFA_SDRAM_TIMER_RG,TP64XX_DSP_EMIFA_SDRAM4M_TIMER_DATA_REV1A_600MHZ)!=0
							)
			   		)
		    	|| (		TP6X_dram_len()==0x8000000L 	// this configuration is for 16Mx64 SDRAM
					  &&	(	 _write_hpi(C6X_EMIFA_SDRAM_RG,TP64XX_DSP_EMIFA_SDRAM16M_DATA_REV1A_600MHZ)!=0
	    					  || _write_hpi(C6X_EMIFA_SDRAM_TIMER_RG,TP64XX_DSP_EMIFA_SDRAM16M_TIMER_DATA_REV1A_600MHZ)!=0
							)
			   		)
	    		|| _write_hpi(C6X_EMIFA_SDRAM_EXT_RG,TP64XX_DSP_EMIFA_SDRAM_EXT_DATA_REV1A_600MHZ)!=0
		    	|| _write_hpi(C6X_EMIFB_CE0_RG,TP64XX_DSP_EMIFB_CE0_DATA_REV1A_600MHZ)!=0
	    		|| _write_hpi(C6X_EMIFB_CE1_RG,TP64XX_DSP_EMIFB_CE1_DATA_REV1A_600MHZ)!=0
	    		|| _write_hpi(C6X_EMIFB_CE3_RG,TP64XX_DSP_EMIFB_CE3_DATA_REV1A_600MHZ)!=0
   	  	  	  )
				goto exit;
		}

		if(TP64XX_BOARD_FW_REV1A1)
		{

				// configure EMIF-A SDRAM control regs upon the SDRAM size installed
			if( 	(		TP6X_dram_len()==0x2000000L 	// this configuration is for 4Mx64 SDRAM
					  &&	(	 _write_hpi(C6X_EMIFA_SDRAM_RG,TP64XX_DSP_EMIFA_SDRAM4M_DATA_REV1A1)!=0
	    					  || _write_hpi(C6X_EMIFA_SDRAM_TIMER_RG,TP64XX_DSP_EMIFA_SDRAM4M_TIMER_DATA_REV1A1)!=0
							)
			   		)
		    	|| (		TP6X_dram_len()==0x8000000L 	// this configuration is for 16Mx64 SDRAM
					  &&	(	 _write_hpi(C6X_EMIFA_SDRAM_RG,TP64XX_DSP_EMIFA_SDRAM16M_DATA_REV1A1)!=0
	    					  || _write_hpi(C6X_EMIFA_SDRAM_TIMER_RG,TP64XX_DSP_EMIFA_SDRAM16M_TIMER_DATA_REV1A1)!=0
							)
			   		)
	    		|| _write_hpi(C6X_EMIFA_SDRAM_EXT_RG,TP64XX_DSP_EMIFA_SDRAM_EXT_DATA_REV1A1)!=0
		    	|| _write_hpi(C6X_EMIFB_CE0_RG,TP64XX_DSP_EMIFB_CE0_DATA_REV1A1)!=0
	    		|| _write_hpi(C6X_EMIFB_CE1_RG,TP64XX_DSP_EMIFB_CE1_DATA_REV1A1)!=0
	    		|| _write_hpi(C6X_EMIFB_CE3_RG,TP64XX_DSP_EMIFB_CE3_DATA_REV1A1)!=0
   	  	  	  )
				goto exit;
		}


		if(TP64XX_BOARD_FW_REV1A2)
		{

				// configure EMIF-A SDRAM control regs upon the SDRAM size installed
			if( 	(		TP6X_dram_len()==0x2000000L 	// this configuration is for 4Mx64 SDRAM
					  &&	(	 _write_hpi(C6X_EMIFA_SDRAM_RG,TP64XX_DSP_EMIFA_SDRAM4M_DATA_REV1A2)!=0
	    					  || _write_hpi(C6X_EMIFA_SDRAM_TIMER_RG,TP64XX_DSP_EMIFA_SDRAM4M_TIMER_DATA_REV1A2)!=0
							)
			   		)
		    	|| (		TP6X_dram_len()==0x8000000L 	// this configuration is for 16Mx64 SDRAM
					  &&	(	 _write_hpi(C6X_EMIFA_SDRAM_RG,TP64XX_DSP_EMIFA_SDRAM16M_DATA_REV1A2)!=0
	    					  || _write_hpi(C6X_EMIFA_SDRAM_TIMER_RG,TP64XX_DSP_EMIFA_SDRAM16M_TIMER_DATA_REV1A2)!=0
							)
			   		)
	    		|| _write_hpi(C6X_EMIFA_SDRAM_EXT_RG,TP64XX_DSP_EMIFA_SDRAM_EXT_DATA_REV1A2)!=0
		    	|| _write_hpi(C6X_EMIFB_CE0_RG,TP64XX_DSP_EMIFB_CE0_DATA_REV1A2)!=0
	    		|| _write_hpi(C6X_EMIFB_CE1_RG,TP64XX_DSP_EMIFB_CE1_DATA_REV1A2)!=0
	    		|| _write_hpi(C6X_EMIFB_CE3_RG,TP64XX_DSP_EMIFB_CE3_DATA_REV1A2)!=0
   	  	  	  )
				goto exit;
		}

		if(TP64XX_BOARD_FW_REV1B)
		{

				// configure EMIF-A SDRAM control regs upon the SDRAM size installed
			if( 	(		TP6X_dram_len()==0x2000000L 	// this configuration is for 4Mx64 SDRAM
					  &&	(	 _write_hpi(C6X_EMIFA_SDRAM_RG,TP64XX_DSP_EMIFA_SDRAM4M_DATA_REV1B)!=0
	    					  || _write_hpi(C6X_EMIFA_SDRAM_TIMER_RG,TP64XX_DSP_EMIFA_SDRAM4M_TIMER_DATA_REV1B)!=0
							)
			   		)
		    	|| (		TP6X_dram_len()==0x8000000L 	// this configuration is for 16Mx64 SDRAM
					  &&	(	 _write_hpi(C6X_EMIFA_SDRAM_RG,TP64XX_DSP_EMIFA_SDRAM16M_DATA_REV1B)!=0
	    					  || _write_hpi(C6X_EMIFA_SDRAM_TIMER_RG,TP64XX_DSP_EMIFA_SDRAM16M_TIMER_DATA_REV1B)!=0
							)
			   		)
	    		|| _write_hpi(C6X_EMIFA_SDRAM_EXT_RG,TP64XX_DSP_EMIFA_SDRAM_EXT_DATA_REV1B)!=0
		    	|| _write_hpi(C6X_EMIFB_CE0_RG,TP64XX_DSP_EMIFB_CE0_DATA_REV1B)!=0
	    		|| _write_hpi(C6X_EMIFB_CE1_RG,TP64XX_DSP_EMIFB_CE1_DATA_REV1B)!=0
	    		|| _write_hpi(C6X_EMIFB_CE3_RG,TP64XX_DSP_EMIFB_CE3_DATA_REV1B)!=0
   	  	  	  )
				goto exit;
		}

    }


	//--- now start test of DSP memory areas via HPI
   	printf("\npress ESC to abort and exit to DOS ...\n\n");

   	data_start = initial_data;
   	data_start_x = initial_data_x;

   	while(1)
    {
		//---- test DSP on-chip data memory
		CLEAR_CURRENT_LINE;
		printf("\rtesting DSP on-chip RAM...   ");

		// below is the LOOP2 data
		START_SADDR	= (TP62XX_TP67_BOARD ? TP6X_DSP_ONCHIP_DRAM_SADDR: TP64XX_DSP_ONCHIP_RAM_SADDR);
		LAST_SADDR	= ((TP62_BOARD) ? TP62_DSP_ONCHIP_DRAM_EADDR:
			   			((TP67_BOARD) ? TP67_DSP_ONCHIP_DRAM_EADDR:
			    			((TP6202_BOARD) ? TP6202_DSP_ONCHIP_DRAM_EADDR:
			    				((TP6203_BOARD) ? TP6203_DSP_ONCHIP_DRAM_EADDR:TP64XX_DSP_ONCHIP_RAM_EADDR))));

		//-- fill HPI data using direct address mode
        data = data_start;

		if( key_pressed() && read_char()==KEY_ESC)
	    {
abort_exit:
			printf("\nUser aborted. status: OK\n\n");
		    goto exit;
	    }

		// clear AINC bit for TP6202/TP6203 boards
	   	if(TP6202_TP6203_BOARD) TP6X_CLR_HPI32_AINC;

		for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data++)
	    {
			if((sa%((LAST_SADDR-START_SADDR+1)/16L))==0)   printf("w");
	        TP6X_SET_HPIA(sa);
			if((data_rd=TP6X_HPIA_DATA)!=sa)
		   	{ 
				printf("\nerror: error setting HPIA register to %lXH (reads as %lxH)\n",sa,data_rd);
		     	goto exit;
		   	}

			TP6X_SET_HPID(data);
			if(TP6X_HPI_ERR_DATA!=0)
		   	{ 
				printf("\nerror: HPI timeout error at write to HPI @%lXH\n",sa);
		     	goto exit;
		   	}
		}

		// verify HPI data using address autoincrement mode
        data = data_start;
        TP6X_SET_HPIA(START_SADDR);

		// set AINC bit for TP6202/TP6203 boards
	   	if(TP6202_TP6203_BOARD) TP6X_SET_HPI32_AINC;

			// perform verification up to the -1 word from the end in order to
			// exclude HPI DMA problems when crossing internal DSP on-chip DRAM/RAM
			// up boundary, which can result in stalling HPI DMA upon DSP reset 
			// (this is true for C6203 DSP at least)
		for (sa=START_SADDR;sa<(LAST_SADDR-4L);sa+=4L,data++)
	    {
			if((sa%((LAST_SADDR-START_SADDR+1)/16L))==0)   printf("r");
			data_rd= (TP6202_TP6203_BOARD) ? TP6X_HPID_DATA:TP6X_HPID_AINC_DATA;

			if(TP6X_HPI_ERR_DATA!=0)
		   	{ 
				printf("\nerror: HPI timeout error at read from HPI @%lxH\n",sa);
		     	goto exit;
		    }
			
			if(data_rd!=data)
		  	{
		    	printf("\nerror: HPI-A=%lxH data_wr=%lxH  data_rd=%lxH\n\n",sa, data, data_rd );
		    	goto exit;

		  	}
		}

		//----- test external DPRAM
		CLEAR_CURRENT_LINE;
		printf("\rtesting DPRAM...   ");

		START_SADDR	= (TP62XX_TP67_BOARD) ? TP6X_DSP_DPRAM_SADDR:TP64XX_DSP_DPRAM_SADDR;
		LAST_SADDR	= START_SADDR+TP6X_dpram_len()-1L;

		//-- fill HPI data using direct address mode
        data_x = data_start_x;

		if( key_pressed() && read_char()==KEY_ESC)	goto abort_exit;

		// clear AINC bit for TP6202/TP6203 boards
	   	if(TP6202_TP6203_BOARD) TP6X_CLR_HPI32_AINC;
		
		for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data_x++)
	    {
			if((sa%((LAST_SADDR-START_SADDR+1)/16L))==0)   printf("w");
	        TP6X_SET_HPIA(sa);
			if(TP6X_HPI_ERR_DATA!=0)
		   	{ 
				printf("\nerror: HPI timeout error at write to HPIA @%lXH\n",sa);
		     	goto exit;
		   	}
			if((data_rd=TP6X_HPIA_DATA)!=sa)
		   	{ 
				printf("\nerror: error setting HPIA register to %lXH (reads as %lxH)\n",sa,data_rd);
		     	goto exit;
		   	}

			TP6X_SET_HPID(data_x);

			if(TP6X_HPI_ERR_DATA!=0)
		   	{ 
				printf("\nerror: HPI timeout error at write to HPID @%lXH\n",sa);
		     	goto exit;
		   	}

		}

		// verify HPI data using address auto increment mode
        data_x = data_start_x;
        TP6X_SET_HPIA(START_SADDR);

		// set AINC bit for TP6202/TP6203 boards
	   	if(TP6202_TP6203_BOARD) TP6X_SET_HPI32_AINC;

		for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data_x++)
	    {
			if((sa%((LAST_SADDR-START_SADDR+1)/16L))==0)   printf("r");
			data_rd_x= TP6X_HPID_AINC_DATA;

			if(TP6X_HPI_ERR_DATA!=0)
		   	{ 
				printf("\nerror: HPI timeout error at read from HPI @%lxH\n",sa);
		     	goto exit;
		    }

			if(data_rd_x!=data_x)
		  	{
		    	printf("\nerror: HPI-A=%lxH data_wr=%lxH  data_rd=%lxH\n\n",sa, data_x, data_rd_x );
		    	goto exit;
		  	}
		}

		//--- test external SBSRAM
		CLEAR_CURRENT_LINE;

   		if(TP6X_sram_len()!=0L)	// test for SBSRAM presents
    	{
			printf("\rtesting SBSRAM...   ");

			START_SADDR	= (TP62XX_TP67_BOARD) ? TP6X_DSP_SBSRAM_SADDR:TP64XX_DSP_SBSRAM_SADDR;
			LAST_SADDR	= START_SADDR+TP6X_sram_len()-1L;

 			if(TP6202_TP6203_BOARD)	// re-configure CE-2/3 for SBSAM for TP6202/TP6203 board
  	  		{
				if(_write_hpi(TP6X_DSP_SYS_STAT_ADDR,TP6x_DSP_XMEM_SBSRAM)!=0) goto exit;
				
				if(TP6202_BOARD)	
  		  		{
					if(    _write_hpi(C6X_EMIF_CE2_RG,TP6202_DSP_EMIF_CE2_SBSRAM_DATA)!=0
			    		|| _write_hpi(C6X_EMIF_CE3_RG,TP6202_DSP_EMIF_CE3_SBSRAM_DATA)!=0
			    		|| _write_hpi(C6X_EMIF_SDRAM_RG,TP6202_DSP_EMIF_SDRAM_DATA)!=0
			    		|| _write_hpi(C6X_EMIF_SDRAM_TIMER_RG,TP6202_DSP_EMIF_SDRAM_TIMER_DATA)!=0
		   	  		  ) 
						goto exit;
  		  		}

		 		if(TP6203_BOARD)	
  		  		{
					if(    _write_hpi(C6X_EMIF_CE2_RG,TP6203_DSP_EMIF_CE2_SBSRAM_DATA)!=0
			    		|| _write_hpi(C6X_EMIF_CE3_RG,TP6203_DSP_EMIF_CE3_SBSRAM_DATA)!=0
			    		|| _write_hpi(C6X_EMIF_SDRAM_RG,TP6203_DSP_EMIF_SDRAM_DATA)!=0
			    		|| _write_hpi(C6X_EMIF_SDRAM_TIMER_RG,TP6203_DSP_EMIF_SDRAM_TIMER_DATA)!=0
		   	  		  ) 
						goto exit;
  		  		}
	   		}


           	//-- fill HPI data using direct address mode
           	data_x = data_start_x;

           	if( key_pressed() && read_char()==KEY_ESC)	goto abort_exit;

			// clear AINC bit for TP6202/TP6203 boards
	   		if(TP6202_TP6203_BOARD) TP6X_CLR_HPI32_AINC;

           	for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data_x++)
	      	{
				if((sa%((LAST_SADDR-START_SADDR+1)/16L))==0)   printf("w");
	        	TP6X_SET_HPIA(sa);
				TP6X_SET_HPID(data_x);

				if(TP6X_HPI_ERR_DATA!=0)
		   		{ 
					printf("\nerror: HPI timeout error at write to HPI @%lXH\n",sa);
		     		goto exit;
		   		}
				if((data_rd=TP6X_HPIA_DATA)!=sa)
		   		{ 
					printf("\nerror: error setting HPIA register to %lXH (reads as %lxH)\n",sa,data_rd);
		     		goto exit;
		   		}
	      	}

			// verify HPI data using address auto increment mode
           	data_x = data_start_x;
           	TP6X_SET_HPIA(START_SADDR);

			// set AINC bit for TP6202/TP6203 boards
	   		if(TP6202_TP6203_BOARD) TP6X_SET_HPI32_AINC;

           	for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data_x++)
	      	{
				if((sa%((LAST_SADDR-START_SADDR+1)/16L))==0)   printf("r");
				data_rd_x= TP6X_HPID_AINC_DATA;

				if(TP6X_HPI_ERR_DATA!=0)
		   		{ 
					printf("\nerror: HPI timeout error at read from HPI @%lxH\n",sa);
		     		goto exit;
		    	}
				if(data_rd_x!=data_x)
		  		{
		    		printf("\nerror: HPI-A=%lxH data_wr=%lxH  data_rd=%lxH\n\n",sa, data_x, data_rd_x );
		    		goto exit;
		  		}
	      	}
     	}



		//--- test external SDRAM

   		if(TP6X_dram_len()!=0L)	// test for SDRAM presents
    	{
 			if(TP6202_TP6203_BOARD)	// re-configure configure CE-2/3 for SDRAM for TP6202/TP6203 board
  	  		{
				if(    _write_hpi(TP6X_DSP_SYS_STAT_ADDR,TP6x_DSP_XMEM_SDRAM)!=0) goto exit;

	 			if(TP6202_BOARD)	
  		  		{
					if(    _write_hpi(C6X_EMIF_CE2_RG,TP6202_DSP_EMIF_CE2_SDRAM_DATA)!=0
			    		|| _write_hpi(C6X_EMIF_CE3_RG,TP6202_DSP_EMIF_CE3_SDRAM_DATA)!=0
			    		|| _write_hpi(C6X_EMIF_SDRAM_RG,TP6202_DSP_EMIF_SDRAM_DATA)!=0
			    		|| _write_hpi(C6X_EMIF_SDRAM_TIMER_RG,TP6202_DSP_EMIF_SDRAM_TIMER_DATA)!=0
		   	  		  ) 
						goto exit;
  		  		}

		 		if(TP6203_BOARD)	
  		  		{
					if(    _write_hpi(C6X_EMIF_CE2_RG,TP6203_DSP_EMIF_CE2_SDRAM_DATA)!=0
			    		|| _write_hpi(C6X_EMIF_CE3_RG,TP6203_DSP_EMIF_CE3_SDRAM_DATA)!=0
			    		|| _write_hpi(C6X_EMIF_SDRAM_RG,TP6203_DSP_EMIF_SDRAM_DATA)!=0
			    		|| _write_hpi(C6X_EMIF_SDRAM_TIMER_RG,TP6203_DSP_EMIF_SDRAM_TIMER_DATA)!=0
		   	  		  ) 
						goto exit;
  		  		}
	   		}

			dm_addr_step	= 0x80000L;
			START_SADDR	= ((TP62XX_TP67_BOARD) ? TP6X_DSP_SDRAM_SADDR:TP64XX_DSP_SDRAM_SADDR)-dm_addr_step;
        	data_x = data_start_x;
        	DM_CCL_TOTAL    = TP6X_dram_len()/dm_addr_step;

      		for(dm_ccl=0L;dm_ccl<DM_CCL_TOTAL;dm_ccl++)
       		{
				START_SADDR	+= dm_addr_step;

				// switch to the 2nd SDRAM bank at CE-2 for TP6202/TP6203 boards with 8M SDRAM installed
				if(    	TP6202_TP6203_BOARD 
	   				&&  START_SADDR==(TP6X_DSP_SDRAM_EADDR_MAX+1L)
	  			  ) 
					START_SADDR=TP6X_DSP_SDRAM2_SADDR;

				LAST_SADDR	= START_SADDR+dm_addr_step-1L;

				CLEAR_CURRENT_LINE;
				printf("\rtesting SDRAM bank #%li (addr=%.8lxH) ...   ",dm_ccl,START_SADDR);

             	if( key_pressed() && read_char()==KEY_ESC)	goto abort_exit;

				// clear AINC bit for TP6202/TP6203 boards
	   			if(TP6202_TP6203_BOARD) TP6X_CLR_HPI32_AINC;

           		//-- fill HPI data using direct address mode
           		data_x = data_ccl_x;

           		for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data_x++)
	      		{
					if((sa%((LAST_SADDR-START_SADDR+1)/16L))==0)   printf("w");
	        		TP6X_SET_HPIA(sa);
					TP6X_SET_HPID(data_x);

					if(TP6X_HPI_ERR_DATA!=0)
		   			{ 
						printf("\nerror: HPI timeout error at write to HPI @%lXH\n",sa);
		     			goto exit;
		   			}
					if((data_rd=TP6X_HPIA_DATA)!=sa)
		   			{ 
						printf("\nerror: error setting HPIA register to %lXH (reads as %lxH)\n",sa,data_rd);
		     			goto exit;
		   			}
	      		}

             	
				// verify HPI data using address auto increment mode
				if( key_pressed() && read_char()==KEY_ESC)	goto abort_exit;

           		data_x = data_ccl_x;
           		TP6X_SET_HPIA(START_SADDR);

				// set AINC bit for TP6202/TP6203 boards
	   			if(TP6202_TP6203_BOARD) TP6X_SET_HPI32_AINC;

           		for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data_x++)
	      		{
					if((sa%((LAST_SADDR-START_SADDR+1)/16L))==0) printf("r");

					data_rd_x= TP6X_HPID_AINC_DATA;

					if(TP6X_HPI_ERR_DATA!=0)
		   			{ 
						printf("\nerror: HPI timeout error at read from HPI @%lxH\n",sa);
		     			goto exit;
		    		}
					if(data_rd_x!=data_x)
		  			{
		    			printf("\nerror: HPI-A=%lxH data_wr=%lxH  data_rd=%lxH\n\n",sa,data_x, data_rd_x );
		    			goto exit;
		  			}
	      		}
	      		data_ccl_x = data_x;
			}	//-- end of SDRAM loop
    	}

       data_start	+= data_inc ;
       data_start_x	+= data_inc_x ;
  }


exit:
   	TP6X_SET_MRESET;
	if(TP64XX_BOARD)  TP6X_SET_DSP_BMODE_NONE;
	
}




