/************************************************************************************
   DSP PCI-bus mastering demo for TP62xx/TP67/TP64xx (host PC control utility). v.2.1
   (C) MicroLAB Systems, 1998-2002

   file: host PC

   This utility performs test of DSP PCI-bus mastering for TORNADO-P6x board. 
   This PC executable runs with both TP62xx/TP67 and TP64xx DSP kernels.


   release notes:
   --------------
	1. In order to fix the PC-to-DSP communication problem via DPRAM for TP64xx as it was
	   in the rev.2.0 demo, the HM_RQ0_RG register is used to pass the PC-to-DSP attention command
	   rather then the MCD->status location in the DPRAM.

*************************************************************************************/


#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <malloc.h>

#include <ips_ut.h>
#include <mlu.h>

#include <tp6x_h.h>


#define	BUF_LEN				0x800L

unsigned long sa;
unsigned int page;
unsigned long	*BUF=NULL, buf_ptr,
				*D, data_rd, data,
                initial_data = 0xa5f1e251L,
				data_inc = 0x13579bdfL;
int ex_status;

#define CLEAR_CURRENT_LINE  printf("\r%79s\r"," ");


char err_mess[100];

#define BOARD_ID	0

int boards,pass;
unsigned long _io0,_io1, _io3, _io4;
unsigned dev_id, index;
char _io0_name[40],_io1_name[40],_io3_name[40],_io4_name[40], dev_name[40];

MLU_TBDD TB;  // time bell descriptor


#define	DPRAM_SUBAREA_BADDR_ADDR	0x1fc	// DSP address of location for passing DPRAM sub-area base address

#define	MCD_ADDR					(0x200L)

#define	M_ID						0x110L
#define	PRG_ID						0x111L

#define	MCMND_NOP					0
#define	MCMND_FILL					1
#define	MCMND_VERIFY				2

#define	MSTAT_BUSY					2
#define	MSTAT_ATTN					1
#define	MSTAT_RDY					0

#define	ERR_OK						0
#define	ERR_HW						-3
#define	ERR_INV_CMND				-2
#define	ERR_INV_PARAM				-1
#define	ERR_AMEN_OFF				-100
#define	ERR_VERIF					-101


struct MCD_DD
{ 
	long  			m_id,
					prg_id,
					cmnd,
					status,
					error;

   	unsigned long	saddr, 
					len, 
					init_data ,
					err_step;	// simplifies error source tracing

   	unsigned long 	xd[8];		// used for tracking real-time params
};


struct MCD_DD *MCD;


void print_dsp_err()
{
	switch((int)MCD->error)
	{
    	case ERR_OK:
	    	printf("\nDSP-error>> OK\n");
			break;

		case ERR_HW:
	    	printf("\nDSP-error>> hardware error, step #%lx\n",MCD->err_step);
			break;

		case ERR_INV_CMND:
	    	printf("\nDSP-error>> invalid command, step #%lx\n",MCD->err_step);
			break;

		case ERR_INV_PARAM:
	    	printf("\nDSP-error>> invalid parameter, step #%lx\n",MCD->err_step);
			break;

		case ERR_AMEN_OFF:
	    	printf("\nDSP-error>> AMEN is OFF, step #%lx\n",MCD->err_step);
			break;

		case ERR_VERIF:
	    	printf("\nDSP-error>> verification error, step #%lx\n",MCD->err_step);
			break;

	    default:
	    	printf("\nDSP-error>> unknown error, step #%lx\n",MCD->err_step);
			break;
	}
}


#define _PCIC_ADDON_TO_PCI_FIFO_EMPTY	((TP6X_PCIC_MCSR_DATA & 0x20L)==0x20L)
#define _PCIC_PCI_TO_ADDON_FIFO_EMPTY	((TP6X_PCIC_MCSR_DATA &  0x4L)==0x4L)
#define _PCIC_RESET_FIFO_FLAGS			(TP6X_SET_PCIC_MCSR(0x6000000L))
#define _PCIC_RESET_MBX_FLAGS			(TP6X_SET_PCIC_MCSR(0x8000000L))
#define _PCIC_RESET_FIFO_MBX_FLAGS		(TP6X_SET_PCIC_MCSR(0xe000000L))




dump_mcd()
{
	printf(" err_step=%lx\n",MCD->err_step);
	printf(" xd[0]=%lx xd[1]=%lx xd[2]=%lx xd[3]=%lx\n",
	MCD->xd[0],MCD->xd[1],MCD->xd[2],MCD->xd[3]);
	printf(" xd[4]=%lx xd[5]=%lx xd[6]=%lx xd[7]=%lx\n",
	MCD->xd[4],MCD->xd[5],MCD->xd[6],MCD->xd[7]);
}



main()
{
	int r,i;
   	unsigned smp_r,smp_w,smp_len;
	long il;


   	printf("\nTORNADO-P6x DSP PCI-mastering Demo. v.2.0\n");
   	printf("Copyright (C) MicroLAB Systems Ltd, 1999-2002\n\n");

   	if(TP6X_open()!=TP6X_OK)
    {
exit_err:
		TP6X_last_err_message(err_mess);
        printf("err: %i  (%s)\n",TP6X_last_err(),err_mess);
		goto exit;
    }
   	printf("boards installed: %i.  Selecting board #0...\n", TP6X_boards());

   	TP6X_sel_board_id(BOARD_ID);
   	TP6X_board_info(BOARD_ID,&dev_id,&index,NULL,NULL,&_io0,&_io1,NULL,&_io3,&_io4);
   	TP6X_dev_name(dev_name);
   	TP6X_area_alloc_name(TP6X_PCIC_AREA,_io0_name);
   	TP6X_area_alloc_name(TP6X_ECC_AREA, _io1_name);
   	TP6X_area_alloc_name(TP6X_DPM_AREA, _io3_name);
   	TP6X_area_alloc_name(TP6X_HIF_AREA, _io4_name);
   	if(TP6X_err_flag()) goto exit_err;

   	printf("TP6x device id = %x (%s)     index=%x\n",dev_id, dev_name, index);
   	printf("TP6x AMCC registers: %lxH (%s)\n",_io0,_io0_name);
   	printf("TP6x ECC area      : %lxH (%s)\n",_io1,_io1_name);
   	printf("TP6x DPRAM area    : %lxH (%s)\n",_io3,_io3_name);
   	printf("TP6x Host I/O area : %lxH (%s)\n",_io4,_io4_name);
   	printf("TP6x on-board DPRAM: %lxH\n",TP6X_dpram_len());
   	printf("TP6x on-board SRAM : %lxH\n",TP6X_sram_len());
   	printf("TP6x on-board SDRAM: %lxH\n",TP6X_dram_len());

   	if(TP6X_err_flag()) goto exit_err;

   	TP6X_SET_MRESET;
   	TP6X_SET_MRESET_MODE_HOST;
   	TP6X_SET_DSP_AMEN_OFF;
   	TP6X_SET_IM_RG(0);
   	TP6X_SET_PCIC_INTCSR(0);

	if(TP62XX_TP67_BOARD)  	
	{
		TP6X_CLR_DPRAM_ERR;
        TP6X_SET_SMP_MODE_SPA;
	}

	if(TP64XX_BOARD)
	{
		TP6X_SET_DSP_BMODE_NONE;
	}

   	if(TP6X_err_flag()) goto exit_err;


	//--- init communication with DSP
   	TP6X_set_smp_ptr(MCD_ADDR,TP6X_SMP_DPRAM_AREA,TP6X_SMP_SMP0_SEL,&MCD);
   	if(TP6X_err_flag()) goto exit_err;

   	printf("testing DSP initialization ... ");

   	MCD->m_id=-1L;
   	MCD->prg_id=-1L;
   	MCD->cmnd=-1L;
   	MCD->status=-1L;
   	MCD->error=-1L;
   	MCD->saddr=0xffffffffL;
   	MCD->len=0xffffffffL;
   	MCD->init_data=0xffffffffL;
   	MCD->err_step=0xffffffffL;

	// initialize HM-RQ#0 register, which will be used to pass the ready flag between PC and DSP
	TP6X_SET_HM_RQ0_RG(1);

	if(TP62XX_TP67_BOARD && TP6X_DPRAM_ERR_DATA!=0)
	{
		printf("\nerror: DPRAM error on init comm\n");
		goto exit;
	}

	// start DSP and init HPI
   	TP6X_SET_MGO;		
   	TP6X_HPI_INIT;

	if(TP64XX_BOARD)
	{
		// for TP64xx boards, we must pass base address of DPRAM sub-area in
		// DSP address space via location @DPRAM_SUBAREA_BADDR_ADDR in DSP memory.
		// DSP s/w will initially clear this location and then poll it for !=0 value.
		// We can write to this location after some pause.

		// wait sometime in order to ensure that DSP has cleared this location
		for(il=0L;il<50000L;il++);

		// now write to this location
		TP6X_SET_HPIA(DPRAM_SUBAREA_BADDR_ADDR);
		TP6X_SET_HPID(TP6X_dpram_area_dsp_baddr()+MCD_ADDR);

		if(TP6X_HPI_ERR_DATA!=0)
   		{
       		printf("\nerror: HPI error is set at passing base address for DPRAM");
       		goto exit;
   		}
	}


	// set time control =1s and wait for DSP will init or timeout
 	MLU_set_bell(1,&TB);  

	// await for the DSP reply via the HM_RQ0 register
	while (TP6X_HM_RQ0_RG_DATA!=0)
//   	while (MCD->status!=MSTAT_RDY)
	{
    	if(MLU_check_bell(&TB)<0)
		{ 
no_reply:
			printf("\nno reply from TP6X\n");
		    dump_mcd();
	       	goto exit;
		}
		if(TP62XX_TP67_BOARD && TP6X_DPRAM_ERR_DATA!=0)
		{ 
dpram_err:
			printf("\nerror: DPRAM error\n");
			goto exit;
		}
	}

	// validate MCD returned
   	if(    	MCD->m_id!=M_ID	
      	||  MCD->prg_id!=PRG_ID
      	||  MCD->cmnd!=MCMND_NOP
      	||  MCD->status!=MSTAT_RDY
      	||  MCD->error !=ERR_OK
      	||  MCD->saddr !=0x55555555L
      	||  MCD->len   !=0xaaaaaaaaL
      	||  MCD->init_data  !=0x12345678L
      	||  MCD->err_step   !=0x87654321L
      )
	{ 
		printf("\ninvalid TP6X initialization\n");
	    goto exit;
	}
    
	if(TP62XX_TP67_BOARD && TP6X_DPRAM_ERR_DATA!=0)	goto dpram_err;

	//--- clear FIFO flags and configure FIFO advance scheme
   _PCIC_RESET_FIFO_MBX_FLAGS;

   	if(   	!_PCIC_ADDON_TO_PCI_FIFO_EMPTY
      	|| 	!_PCIC_PCI_TO_ADDON_FIFO_EMPTY
      )
   	{
    	printf("unable to reset FIFO flags\n");
		goto exit;
    }

   	if(   	TP6X_SET_PCIC_INTCSR(0x0c000000L)!=TP6X_OK
      	|| 	TP6X_PCIC_INTCSR_DATA!=0x0c000000L
      )
	{
    	printf("error: cannot configure FIFO advance control\n ");
	    goto exit;
    }


	// allocate memory for PCI-mastering transfer
	if((BUF=(unsigned long*) halloc(BUF_LEN+0x10L,1))==NULL)
	{ 
		printf("\nerror: not enough memory\n");
	    goto exit;
	}

   	buf_ptr = (unsigned long) BUF;
	buf_ptr = ((buf_ptr>>12)&0xffff0L) + (buf_ptr&0xffffL);

	printf("\n\npress any key to abort ...\n\n");

	data = initial_data;

	pass=0;
	CLEAR_CURRENT_LINE;

   	while(1)
    {
		if(pass==0)  CLEAR_CURRENT_LINE;
		// exit on key pressed
		if(key_pressed()) break;

		//-------- FILL-in data by DSP -----------
		printf("w");

		// start FILL with AMEN bit not set (ensure the error)
   		TP6X_SET_DSP_AMEN_OFF;
        if(TP6X_err_flag()) goto exit_err;

		// clear allocated buffer
		for(i=0,D=BUF;i<BUF_LEN/4L;i++) *D++=0L;

		MCD->cmnd = MCMND_FILL;
		MCD->saddr = buf_ptr;
        MCD->len   = BUF_LEN;
		MCD->init_data = data;
		MCD->status = MSTAT_ATTN;

		// set ATTENTION command to the DSP via the HM-RQ#0 register
		TP6X_SET_HM_RQ0_RG(1);

        if(TP62XX_TP67_BOARD && TP6X_DPRAM_ERR_DATA!=0)	goto dpram_err;
        if(TP6X_err_flag()) goto exit_err;

		// set time control =1s and wait for DSP will init or timeout
 		MLU_set_bell(1,&TB);
		// await for the DSP reply via the HM_RQ0 register
		while (TP6X_HM_RQ0_RG_DATA!=0)
//   		while (MCD->status!=MSTAT_RDY)
		{
       		if(MLU_check_bell(&TB)<0) goto no_reply;
	        if(TP6X_DPRAM_ERR_DATA!=0)	goto dpram_err;
	  	}

		if(MCD->error!=ERR_AMEN_OFF)
		{
amen_off_err:
			printf("\nerror: DSP is unable to detect AMEN OFF\n");
			print_dsp_err();
            goto exit;
		}

		// enable AMEN and repeat CMND
   		TP6X_SET_DSP_AMEN_ON;
        if(TP6X_err_flag()) goto exit_err;

		MCD->cmnd = MCMND_FILL;
		MCD->saddr = buf_ptr;
        MCD->len   = BUF_LEN;
		MCD->init_data = data;
		MCD->status = MSTAT_ATTN;

		// set ATTENTION command to the DSP via the HM-RQ#0 register
		TP6X_SET_HM_RQ0_RG(1);

        if(TP62XX_TP67_BOARD && TP6X_DPRAM_ERR_DATA!=0)	goto dpram_err;
        if(TP6X_err_flag()) goto exit_err;

		// set time control and wait for DSP will init or timeout
 		MLU_set_bell(2,&TB);
		// await for the DSP reply via the HM_RQ0 register
		while (TP6X_HM_RQ0_RG_DATA!=0)
//   		while (MCD->status!=MSTAT_RDY)
     	{
       		if(MLU_check_bell(&TB)<0)  goto no_reply;
       		if(TP62XX_TP67_BOARD && TP6X_DPRAM_ERR_DATA!=0) goto dpram_err;
	  	}

		if(MCD->error!=ERR_OK)
		{
dsp_err:
			print_dsp_err();
			dump_mcd();
            goto exit;
		}

        if(TP62XX_TP67_BOARD && TP6X_DPRAM_ERR_DATA!=0)	goto dpram_err;

		// verify filled-in data
		for(i=0,D=BUF,data_rd=data;i<BUF_LEN/4L;i++,D++,data_rd++)
	  	{
        	if(*D!=data_rd)
	      	{
            	printf("\nerror: verification error for filled-in data\n");
                goto exit;
	      	}
	  	}


		//-------- DSP data verification -----------
		printf("r");

		MCD->cmnd = MCMND_VERIFY;
		MCD->saddr = buf_ptr;
        MCD->len   = BUF_LEN;
		MCD->init_data = data;
		MCD->status = MSTAT_ATTN;

		// set ATTENTION command to the DSP via the HM-RQ#0 register
		TP6X_SET_HM_RQ0_RG(1);

        if(TP6X_DPRAM_ERR_DATA!=0)	goto dpram_err;
        if(TP6X_err_flag()) goto exit_err;

		// set time control and wait for DSP will init or timeout
 		MLU_set_bell(2,&TB);
		// await for the DSP reply via the HM_RQ0 register
		while (TP6X_HM_RQ0_RG_DATA!=0)
//   		while (MCD->status!=MSTAT_RDY)
     	{
       		if(MLU_check_bell(&TB)<0) goto no_reply;
	        if(TP62XX_TP67_BOARD && TP6X_DPRAM_ERR_DATA!=0) goto dpram_err;
	  	}
        if(MCD->error!=ERR_OK) goto dsp_err;

        if(TP62XX_TP67_BOARD && TP6X_DPRAM_ERR_DATA!=0)	goto dpram_err;

       	data += data_inc ;

       	if(++pass>=10)
	  	{
	    	pass=0;
	  	}
  	}

exit:
    CLEAR_CURRENT_LINE;
    TP6X_SET_MRESET;
    TP6X_SET_DSP_AMEN_OFF;

	if(TP62XX_TP67_BOARD)
	{
    	TP6X_SET_SMP0_RG(0);
    	TP6X_SET_SMP1_RG(0);
    	TP6X_CLR_DPRAM_ERR;
	}

	if(TP64XX_BOARD)
		TP6X_CLR_HPI_ERR;
}





