/******************************************************************************
  TMS320C6x (C620x/C670x/C621x/C671x/C64xx DSP General Definitions. rev.2B
  (C) MicroLAB Systems Ltd, 1998-2002

  Revision history:
  -----------------
	rev.1A	- 1998, initial release

	rev.2A	- 2002
				- completely redesigned by adding replacements of (unsigned *C6X_...= (unsigned *)0x...);)
				  definitions by (#define..) in order to exclude useless pointer variable allocation
				  in the .BSS area and to avoid multiple allocated variable re-definition in case 
				  C6x application consists of multiple .C modules and each includes this header file.
				  User syntax remains the same and user applications shall be
				  recompiled using this new header file. 

				  For exact user code compatibility, old C620x/C670x definitions with 
				  DSP on-chip register allocated vars are also available if 
				  __C6X_SELECT_C620X_C670X_DSP_RG_PTR_VAR__ definition is defined 
				  in user C-code prior inclusion of this file.

				- added support for C621x/C671x/C64xx DSP using run-time compiler keys

  Notes:
  ------
	1. 	This C-header file contains general C6x DSP definitions and is include 
		file for TI C6x compile tools. 
	2.	This header file is externally controlled from user C-code by run-time 
		compiler keys definitions in order to apply DSP-type specific difinitions
		to refer to particular definitions included for different DSP type:
			__C6X_SELECT_C64XX_DSP__, or __TE6X_SELECT_E64XX__, or __TP6X_SELECT_TP64XX__
				- if defined in user code prior inclusion of this header file,
				  then DSP-type specific definitions are applied to C64xx DSP

			__C6X_SELECT_C620X_C670X_DSP_RG_PTR_VAR__
				- if defined in user code prior inclusion of this header file,
				  then old C620x/C670x definitions with DSP on-chip register pointers
				  allocation in the .BSS section will be included 
		
			__C6X_SELECT_C621X_C671X_DSP__
				- if defined in user code prior inclusion of this header file,
				  then DSP-type specific definitions are applied to C21x/C671x DSP

		If neither of the above run-time definitions are defined, then new DSP-type
		specific definitions without allocation of vars to DSP on-chip peripheral 
		registers are assumed to be applied for C620x/C670x DSP.

*******************************************************************************/

#ifndef __C6X_DEF__		// check for this file has been already included
#define __C6X_DEF__	1


//---- DSP-type specific run-time compiler keys processing -------
// (this is required in order to exclude key confusions)

//	Note:!!
//		If none of valid compiler keys are defined in user code, then
//		new defs without register allocation are applied to C620x/C670x DSP.

// initially, undefine common defs
#ifndef	__C6X_SELECT_C64XX_DSP__
#ifndef	__C6X_SELECT_C621X_C671X_DSP__
#ifndef	__TE6X_SELECT_E64XX__
#ifndef	__TP6X_SELECT_TP64XX__
#undef	__C620X_C670X_DSP_DEF__
#define	__C620X_C670X_DSP_DEF__					1	
#ifndef	__C6X_SELECT_C620X_C670X_DSP_RG_PTR_VAR__
#undef	__C6X_DSP_NEW_PERIPH_RG_DEF__
#define	__C6X_DSP_NEW_PERIPH_RG_DEF__			1	
#endif
#endif
#endif
#endif
#endif

// check for old C6X defs for C620x/C670x DSP
#ifdef	__C6X_SELECT_C620X_C670X_DSP_RG_PTR_VAR__
#undef	__C6X_SELECT_C64XX_DSP__
#undef	__C6X_SELECT_C621X_C671X_DSP__
#undef	__C6X_DSP_NEW_PERIPH_RG_DEF__
#undef	__C620X_C670X_DSP_DEF__
#define	__C620X_C670X_DSP_DEF__					1	
#undef	__TE6X_SELECT_E64XX__
#undef	__TP6X_SELECT_TP64XX__
#endif

// now define common defs
#ifdef	__C6X_SELECT_C64XX_DSP__
#undef	__C6X_SELECT_C621X_C671X_DSP__
#undef	__C620X_C670X_DSP_DEF__
#undef	__C6X_DSP_NEW_PERIPH_RG_DEF__
#define	__C6X_DSP_NEW_PERIPH_RG_DEF__			1	
#undef	__TE6X_SELECT_E64XX__
#define	__TE6X_SELECT_E64XX__					1
#undef	__TP6X_SELECT_TP64XX__
#define	__TP6X_SELECT_TP64XX__					1
#endif

#ifdef	__TE6X_SELECT_E64XX__
#undef	__C6X_SELECT_C621X_C671X_DSP__
#undef	__C620X_C670X_DSP_DEF__
#undef	__C6X_DSP_NEW_PERIPH_RG_DEF__
#define	__C6X_DSP_NEW_PERIPH_RG_DEF__			1	
#undef	__C6X_SELECT_C64XX_DSP__
#define	__C6X_SELECT_C64XX_DSP__				1
#endif

#ifdef	__TP6X_SELECT_TP64XX__
#undef	__C6X_SELECT_C621X_C671X_DSP__
#undef	__C620X_C670X_DSP_DEF__
#undef	__C6X_DSP_NEW_PERIPH_RG_DEF__
#define	__C6X_DSP_NEW_PERIPH_RG_DEF__			1	
#undef	__C6X_SELECT_C64XX_DSP__
#define	__C6X_SELECT_C64XX_DSP__				1
#endif

#ifdef	__C6X_SELECT_C621X_C671X_DSP__
#undef	__C620X_C670X_DSP_DEF__
#undef	__C6X_DSP_NEW_PERIPH_RG_DEF__
#define	__C6X_DSP_NEW_PERIPH_RG_DEF__			1	
#undef	__TE6X_SELECT_E64XX__
#endif
//----------------------------------------------------------------


#ifdef	__C6X_DSP_NEW_PERIPH_RG_DEF__
//============= C6xxx peripheral regs definitions ================
// DSP on-chip register allocated defs for all C6xxx DSP without
// pointers allocation in the .BSS section

//-------------- memory mapped peripheral registers --------------

//>>>>>>>>>> EMIF control RGs

#define	C6X_EMIF_GCR        	((unsigned *) 0x01800000)
#define	C6X_EMIF_CE0        	((unsigned *) 0x01800008)
#define	C6X_EMIF_CE1        	((unsigned *) 0x01800004)
#define	C6X_EMIF_CE2        	((unsigned *) 0x01800010)
#define	C6X_EMIF_CE3        	((unsigned *) 0x01800014)
#define	C6X_EMIF_SDRAM      	((unsigned *) 0x01800018)
#define	C6X_EMIF_SDRAM_TIMER	((unsigned *) 0x0180001c)

	// C64xx specific extension regs
#define	C6X_EMIF_SDRAM_EXT     	((unsigned *) 0x01800020)
#define	C6X_EMIF_CE0_SEC     	((unsigned *) 0x01800048)
#define	C6X_EMIF_CE1_SEC     	((unsigned *) 0x01800044)
#define	C6X_EMIF_CE2_SEC     	((unsigned *) 0x01800050)
#define	C6X_EMIF_CE3_SEC     	((unsigned *) 0x01800054)

	// aliases for C64xx DSP EMIF-A
#define	C6X_EMIFA_GCR        	C6X_EMIF_GCR
#define	C6X_EMIFA_CE0        	C6X_EMIF_CE0
#define	C6X_EMIFA_CE1        	C6X_EMIF_CE1
#define	C6X_EMIFA_CE2        	C6X_EMIF_CE2
#define	C6X_EMIFA_CE3        	C6X_EMIF_CE3
#define	C6X_EMIFA_SDRAM      	C6X_EMIF_SDRAM
#define	C6X_EMIFA_SDRAM_TIMER	C6X_EMIF_SDRAM_TIMER

#define	C6X_EMIFA_SDRAM_EXT     C6X_EMIF_SDRAM_EXT
#define	C6X_EMIFA_CE0_SEC     	C6X_EMIF_CE0_SEC
#define	C6X_EMIFA_CE1_SEC     	C6X_EMIF_CE1_SEC
#define	C6X_EMIFA_CE2_SEC     	C6X_EMIF_CE2_SEC
#define	C6X_EMIFA_CE3_SEC     	C6X_EMIF_CE3_SEC

#define	C64XX_EMIFA_GCR        	C6X_EMIF_GCR
#define	C64XX_EMIFA_CE0        	C6X_EMIF_CE0
#define	C64XX_EMIFA_CE1        	C6X_EMIF_CE1
#define	C64XX_EMIFA_CE2        	C6X_EMIF_CE2
#define	C64XX_EMIFA_CE3        	C6X_EMIF_CE3
#define	C64XX_EMIFA_SDRAM      	C6X_EMIF_SDRAM
#define	C64XX_EMIFA_SDRAM_TIMER	C6X_EMIF_SDRAM_TIMER

#define	C64XX_EMIFA_SDRAM_EXT   C6X_EMIF_SDRAM_EXT
#define	C64XX_EMIFA_CE0_SEC     C6X_EMIF_CE0_SEC
#define	C64XX_EMIFA_CE1_SEC     C6X_EMIF_CE1_SEC
#define	C64XX_EMIFA_CE2_SEC     C6X_EMIF_CE2_SEC
#define	C64XX_EMIFA_CE3_SEC     C6X_EMIF_CE3_SEC


	// C64xx EMIF-B defs
#define	C6X_EMIFB_GCR        	((unsigned *) 0x01A80000)
#define	C6X_EMIFB_CE0        	((unsigned *) 0x01A80008)
#define	C6X_EMIFB_CE1        	((unsigned *) 0x01A80004)
#define	C6X_EMIFB_CE2        	((unsigned *) 0x01A80010)
#define	C6X_EMIFB_CE3        	((unsigned *) 0x01A80014)
#define	C6X_EMIFB_SDRAM      	((unsigned *) 0x01A80018)
#define	C6X_EMIFB_SDRAM_TIMER	((unsigned *) 0x01A8001c)

#define	C6X_EMIFB_SDRAM_EXT    	((unsigned *) 0x01A80020)
#define	C6X_EMIFB_CE0_SEC     	((unsigned *) 0x01A80048)
#define	C6X_EMIFB_CE1_SEC     	((unsigned *) 0x01A80044)
#define	C6X_EMIFB_CE2_SEC     	((unsigned *) 0x01A80050)
#define	C6X_EMIFB_CE3_SEC     	((unsigned *) 0x01A80054)


#define	C64XX_EMIFB_GCR        	C6X_EMIFB_GCR
#define	C64XX_EMIFB_CE0        	C6X_EMIFB_CE0
#define	C64XX_EMIFB_CE1        	C6X_EMIFB_CE1
#define	C64XX_EMIFB_CE2        	C6X_EMIFB_CE2
#define	C64XX_EMIFB_CE3        	C6X_EMIFB_CE3
#define	C64XX_EMIFB_SDRAM      	C6X_EMIFB_SDRAM
#define	C64XX_EMIFB_SDRAM_TIMER	C6X_EMIFB_SDRAM_TIMER

#define	C64XX_EMIFB_SDRAM_EXT  	C6X_EMIFB_SDRAM_EXT
#define	C64XX_EMIFB_CE0_SEC    	C6X_EMIFB_CE0_SEC
#define	C64XX_EMIFB_CE1_SEC    	C6X_EMIFB_CE1_SEC
#define	C64XX_EMIFB_CE2_SEC    	C6X_EMIFB_CE2_SEC
#define	C64XX_EMIFB_CE3_SEC    	C6X_EMIFB_CE3_SEC


	// TI register convention aliases
#define	C6X_EMIF_GBLCTL        	C6X_EMIF_GCR        	
#define	C6X_EMIF_CE0CTL        	C6X_EMIF_CE0        	
#define	C6X_EMIF_CE1CTL        	C6X_EMIF_CE1
#define	C6X_EMIF_CE2CTL        	C6X_EMIF_CE2        	
#define	C6X_EMIF_CE3CTL        	C6X_EMIF_CE3
#define	C6X_EMIF_CE0SEC     	C6X_EMIF_CE0_SEC     	
#define	C6X_EMIF_CE1SEC     	C6X_EMIF_CE1_SEC     	
#define	C6X_EMIF_CE2SEC     	C6X_EMIF_CE2_SEC     	
#define	C6X_EMIF_CE3SEC     	C6X_EMIF_CE3_SEC     	
#define	C6X_EMIF_SDCTL	     	C6X_EMIF_SDRAM      	
#define	C6X_EMIF_SDTIM			C6X_EMIF_SDRAM_TIMER	
#define	C6X_EMIF_SDEXT     		C6X_EMIF_SDRAM_EXT     	

#define	C6X_EMIFA_GBLCTL       	C6X_EMIF_GCR        	
#define	C6X_EMIFA_CE0CTL       	C6X_EMIF_CE0        	
#define	C6X_EMIFA_CE1CTL       	C6X_EMIF_CE1
#define	C6X_EMIFA_CE2CTL       	C6X_EMIF_CE2        	
#define	C6X_EMIFA_CE3CTL       	C6X_EMIF_CE3        	
#define	C6X_EMIFA_CE0SEC     	C6X_EMIF_CE0_SEC     	
#define	C6X_EMIFA_CE1SEC     	C6X_EMIF_CE1_SEC     	
#define	C6X_EMIFA_CE2SEC     	C6X_EMIF_CE2_SEC     	
#define	C6X_EMIFA_CE3SEC     	C6X_EMIF_CE3_SEC     	
#define	C6X_EMIFA_SDCTL	     	C6X_EMIF_SDRAM      	
#define	C6X_EMIFA_SDTIM			C6X_EMIF_SDRAM_TIMER	
#define	C6X_EMIFA_SDEXT    		C6X_EMIF_SDRAM_EXT     	

#define	C6X_EMIFB_GBLCTL       	C6X_EMIFB_GCR        	
#define	C6X_EMIFB_CE0CTL       	C6X_EMIFB_CE0        	
#define	C6X_EMIFB_CE1CTL       	C6X_EMIFB_CE1
#define	C6X_EMIFB_CE2CTL       	C6X_EMIFB_CE2        	
#define	C6X_EMIFB_CE3CTL       	C6X_EMIFB_CE3        	
#define	C6X_EMIFB_CE0SEC     	C6X_EMIFB_CE0_SEC     	
#define	C6X_EMIFB_CE1SEC     	C6X_EMIFB_CE1_SEC     	
#define	C6X_EMIFB_CE2SEC     	C6X_EMIFB_CE2_SEC     	
#define	C6X_EMIFB_CE3SEC     	C6X_EMIFB_CE3_SEC     	
#define	C6X_EMIFB_SDCTL	     	C6X_EMIFB_SDRAM      	
#define	C6X_EMIFB_SDTIM			C6X_EMIFB_SDRAM_TIMER	
#define	C6X_EMIFB_SDEXT    		C6X_EMIFB_SDRAM_EXT     	


//>>>>>>>>>>>>>> TIMER Control
#define	C6X_TM0_CONTROL     	((unsigned *) 0x01940000)
#define	C6X_TM0_PERIOD      	((unsigned *) 0x01940004)
#define	C6X_TM0_COUNTER     	((unsigned *) 0x01940008)
#define	C6X_TM1_CONTROL     	((unsigned *) 0x01980000)
#define	C6X_TM1_PERIOD      	((unsigned *) 0x01980004)
#define	C6X_TM1_COUNTER     	((unsigned *) 0x01980008)

	// C64xx specific TIMER-2
#define	C6X_TM2_CONTROL     	((unsigned *) 0x01AC0000)
#define	C6X_TM2_PERIOD      	((unsigned *) 0x01AC0004)
#define	C6X_TM2_COUNTER     	((unsigned *) 0x01AC0008)


//>>>>>>>>>>>>>> MCBSP Registers
#define	C6X_MCBSP0_DRR      	((unsigned *) 0x018C0000)
#define	C6X_MCBSP0_DXR      	((unsigned *) 0x018C0004)
#define	C6X_MCBSP0_SPCR     	((unsigned *) 0x018C0008)
#define	C6X_MCBSP0_RCR      	((unsigned *) 0x018C000C)
#define	C6X_MCBSP0_XCR      	((unsigned *) 0x018C0010)
#define	C6X_MCBSP0_SRGR     	((unsigned *) 0x018C0014)
#define	C6X_MCBSP0_MCR      	((unsigned *) 0x018C0018)
#define	C6X_MCBSP0_RCER     	((unsigned *) 0x018C001C)
#define	C6X_MCBSP0_XCER     	((unsigned *) 0x018C0020)
#define	C6X_MCBSP0_PCR      	((unsigned *) 0x018C0024)

#define	C6X_MCBSP1_DRR      	((unsigned *) 0x01900000)
#define	C6X_MCBSP1_DXR      	((unsigned *) 0x01900004)
#define	C6X_MCBSP1_SPCR     	((unsigned *) 0x01900008)
#define	C6X_MCBSP1_RCR      	((unsigned *) 0x0190000C)
#define	C6X_MCBSP1_XCR      	((unsigned *) 0x01900010)
#define	C6X_MCBSP1_SRGR     	((unsigned *) 0x01900014)
#define	C6X_MCBSP1_MCR      	((unsigned *) 0x01900018)
#define	C6X_MCBSP1_RCER     	((unsigned *) 0x0190001C)
#define	C6X_MCBSP1_XCER     	((unsigned *) 0x01900020)
#define	C6X_MCBSP1_PCR      	((unsigned *) 0x01900024)

	// available in C6202/C6203/C64xx
#define	C6X_MCBSP2_DRR      	((unsigned *) 0x01a40000)	
#define	C6X_MCBSP2_DXR      	((unsigned *) 0x01a40004)
#define	C6X_MCBSP2_SPCR     	((unsigned *) 0x01a40008)
#define	C6X_MCBSP2_RCR      	((unsigned *) 0x01a4000C)
#define	C6X_MCBSP2_XCR      	((unsigned *) 0x01a40010)
#define	C6X_MCBSP2_SRGR     	((unsigned *) 0x01a40014)
#define	C6X_MCBSP2_MCR      	((unsigned *) 0x01a40018)
#define	C6X_MCBSP2_RCER     	((unsigned *) 0x01a4001C)
#define	C6X_MCBSP2_XCER     	((unsigned *) 0x01a40020)
#define	C6X_MCBSP2_PCR      	((unsigned *) 0x01a40024)

	// C64xx specific McBSP regs
#define	C6X_MCBSP0_RCERE1     	((unsigned *) 0x018C0028)
#define	C6X_MCBSP0_XCERE1     	((unsigned *) 0x018C002C)
#define	C6X_MCBSP0_RCERE2     	((unsigned *) 0x018C0030)
#define	C6X_MCBSP0_XCERE2     	((unsigned *) 0x018C0034)
#define	C6X_MCBSP0_RCERE3     	((unsigned *) 0x018C0038)
#define	C6X_MCBSP0_XCERE3     	((unsigned *) 0x018C003C)

#define	C6X_MCBSP1_RCERE1     	((unsigned *) 0x01900028)
#define	C6X_MCBSP1_XCERE1     	((unsigned *) 0x0190002C)
#define	C6X_MCBSP1_RCERE2     	((unsigned *) 0x01900030)
#define	C6X_MCBSP1_XCERE2     	((unsigned *) 0x01900034)
#define	C6X_MCBSP1_RCERE3     	((unsigned *) 0x01900038)
#define	C6X_MCBSP1_XCERE3     	((unsigned *) 0x0190003C)

#define	C6X_MCBSP2_RCERE1     	((unsigned *) 0x01a40028)
#define	C6X_MCBSP2_XCERE1     	((unsigned *) 0x01a4002C)
#define	C6X_MCBSP2_RCERE2     	((unsigned *) 0x01a40030)
#define	C6X_MCBSP2_XCERE2     	((unsigned *) 0x01a40034)
#define	C6X_MCBSP2_RCERE3     	((unsigned *) 0x01a40038)
#define	C6X_MCBSP2_XCERE3     	((unsigned *) 0x01a4003C)


//>>>>>>>>>>>>> HPI/XBUS registers
#define	C6X_HPIC	      		((unsigned *) 0x01880000)
#define	C6X_XBGC	      		((unsigned *) 0x01880000)	// C6202/C6203 X-bus
#define	C6X_XCE0_SCR	      	((unsigned *) 0x01880008)	// C6202/C6203 X-bus
#define	C6X_XCE1_SCR	      	((unsigned *) 0x01880004)	// C6202/C6203 X-bus
#define	C6X_XCE2_SCR	      	((unsigned *) 0x01880010)	// C6202/C6203 X-bus
#define	C6X_XCE3_SCR	      	((unsigned *) 0x01880014)	// C6202/C6203 X-bus
#define	C6X_XBHC	      		((unsigned *) 0x0188000c)	// C6202/C6203 X-bus
#define	C6X_XBIMA	      		((unsigned *) 0x01880020)	// C6202/C6203 X-bus
#define	C6X_XBEA	      		((unsigned *) 0x01880024)	// C6202/C6203 X-bus

	// C64xx extension regs for HPI (HPIA splitted into READ/WRITE)
#define	C6X_HPIAW	      		((unsigned *) 0x01880004)
#define	C6X_HPIAR	      		((unsigned *) 0x01880008)


//>>>>>>>>>>>>> INT specific rgs
#define C6X_INTMUX_HIGH     	((unsigned *) 0x019C0000)
#define C6X_INTMUX_LOW      	((unsigned *) 0x019C0004)
#define C6X_XINT_POLARITY   	((unsigned *) 0x019C0008)


//>>>>>>>>>>>>> C620x/C670x DMA control registers
#define C6X_DMA0_PCRG			((unsigned *) 0x01840000)
#define C6X_DMA0_SCRG			((unsigned *) 0x01840008)
#define C6X_DMA0_SARG			((unsigned *) 0x01840010)
#define C6X_DMA0_DARG			((unsigned *) 0x01840018)
#define C6X_DMA0_TCRG			((unsigned *) 0x01840020)

#define C6X_DMA1_PCRG			((unsigned *) 0x01840040)
#define C6X_DMA1_SCRG			((unsigned *) 0x01840048)
#define C6X_DMA1_SARG			((unsigned *) 0x01840050)
#define C6X_DMA1_DARG			((unsigned *) 0x01840058)
#define C6X_DMA1_TCRG			((unsigned *) 0x01840060)

#define C6X_DMA2_PCRG			((unsigned *) 0x01840004)
#define C6X_DMA2_SCRG			((unsigned *) 0x0184000c)
#define C6X_DMA2_SARG			((unsigned *) 0x01840014)
#define C6X_DMA2_DARG			((unsigned *) 0x0184001c)
#define C6X_DMA2_TCRG			((unsigned *) 0x01840024)

#define C6X_DMA3_PCRG			((unsigned *) 0x01840044)
#define C6X_DMA3_SCRG			((unsigned *) 0x0184004c)
#define C6X_DMA3_SARG			((unsigned *) 0x01840054)
#define C6X_DMA3_DARG			((unsigned *) 0x0184005c)
#define C6X_DMA3_TCRG			((unsigned *) 0x01840064)

#define C6X_DMA_GAARG			((unsigned *) 0x01840038)
#define C6X_DMA_GABRG			((unsigned *) 0x0184003C)
#define C6X_DMA_GACRG			((unsigned *) 0x01840068)
#define C6X_DMA_GADRG			((unsigned *) 0x0184006C)
#define C6X_DMA_GCRARG			((unsigned *) 0x01840028)
#define C6X_DMA_GCRBRG			((unsigned *) 0x0184002C)
#define C6X_DMA_GIARG			((unsigned *) 0x01840030)
#define C6X_DMA_GIBRG			((unsigned *) 0x01840034)
		// aliases
#define C6X_DMA0_GAARG			C6X_DMA_GAARG
#define C6X_DMA0_GABRG			C6X_DMA_GABRG
#define C6X_DMA0_GACRG			C6X_DMA_GACRG
#define C6X_DMA0_GADRG			C6X_DMA_GADRG
#define C6X_DMA0_GCRARG			C6X_DMA_GCRARG
#define C6X_DMA0_GCRBRG			C6X_DMA_GCRBRG
#define C6X_DMA0_GIARG			C6X_DMA_GIARG
#define C6X_DMA0_GIBRG			C6X_DMA_GIBRG

#define C6X_DMA1_GAARG			C6X_DMA_GAARG
#define C6X_DMA1_GABRG			C6X_DMA_GABRG
#define C6X_DMA1_GACRG			C6X_DMA_GACRG
#define C6X_DMA1_GADRG			C6X_DMA_GADRG
#define C6X_DMA1_GCRARG			C6X_DMA_GCRARG
#define C6X_DMA1_GCRBRG			C6X_DMA_GCRBRG
#define C6X_DMA1_GIARG			C6X_DMA_GIARG
#define C6X_DMA1_GIBRG			C6X_DMA_GIBRG

#define C6X_DMA2_GAARG			C6X_DMA_GAARG
#define C6X_DMA2_GABRG			C6X_DMA_GABRG
#define C6X_DMA2_GACRG			C6X_DMA_GACRG
#define C6X_DMA2_GADRG			C6X_DMA_GADRG
#define C6X_DMA2_GCRARG			C6X_DMA_GCRARG
#define C6X_DMA2_GCRBRG			C6X_DMA_GCRBRG
#define C6X_DMA2_GIARG			C6X_DMA_GIARG
#define C6X_DMA2_GIBRG			C6X_DMA_GIBRG

#define C6X_DMA3_GAARG			C6X_DMA_GAARG
#define C6X_DMA3_GABRG			C6X_DMA_GABRG
#define C6X_DMA3_GACRG			C6X_DMA_GACRG
#define C6X_DMA3_GADRG			C6X_DMA_GADRG
#define C6X_DMA3_GCRARG			C6X_DMA_GCRARG
#define C6X_DMA3_GCRBRG			C6X_DMA_GCRBRG
#define C6X_DMA3_GIARG			C6X_DMA_GIARG
#define C6X_DMA3_GIBRG			C6X_DMA_GIBRG

#define C6X_DMAAUX_CRG			((unsigned *) 0x01840070)


//>>>>>>>>>>>>> C621x/C671x/C64xx EDMA control registers
	// event LOW/HIGH registers (HIGH part for C64xx only)
#define C6X_EDMA_ER				((unsigned *) 0x01A0fff0)
#define C6X_EDMA_ERL			((unsigned *) 0x01A0fff0)
#define C6X_EDMA_ERH			((unsigned *) 0x01A0ffb0)
	
	// event enable LOW/HIGH (HIGH part for C64xx only)
#define C6X_EDMA_EER			((unsigned *) 0x01A0fff4)
#define C6X_EDMA_EERL			((unsigned *) 0x01A0fff4)
#define C6X_EDMA_EERH			((unsigned *) 0x01A0ffb4)

	// event clear LOW/HIGH (HIGH part for C64xx only)
#define C6X_EDMA_ECR			((unsigned *) 0x01A0fff8)
#define C6X_EDMA_ECRL			((unsigned *) 0x01A0fff8)
#define C6X_EDMA_ECRH			((unsigned *) 0x01A0ffb8)

	// event set LOW/HIGH (HIGH part for C64xx only)
#define C6X_EDMA_ESR			((unsigned *) 0x01A0fffc)
#define C6X_EDMA_ESRL			((unsigned *) 0x01A0fffc)
#define C6X_EDMA_ESRH			((unsigned *) 0x01A0ffbc)

	// event polarity LOW/HIGH (HIGH part for C64xx only)
#define C6X_EDMA_EPR			((unsigned *) 0x01A0ffdc)
#define C6X_EDMA_EPRL			((unsigned *) 0x01A0ffdc)
#define C6X_EDMA_EPRH			((unsigned *) 0x01A0ff9c)

	// channel interrupt enable LOW/HIGH (HIGH part for C64xx only)
#define C6X_EDMA_CIER			((unsigned *) 0x01A0ffe8)
#define C6X_EDMA_CIERL			((unsigned *) 0x01A0ffe8)
#define C6X_EDMA_CIERH			((unsigned *) 0x01A0ffa8)

	// channel interrupt pending LOW/HIGH (HIGH part for C64xx only)
#define C6X_EDMA_CIPR			((unsigned *) 0x01A0ffe4)
#define C6X_EDMA_CIPRL			((unsigned *) 0x01A0ffe4)
#define C6X_EDMA_CIPRH			((unsigned *) 0x01A0ffa4)

	// channel chain enable LOW/HIGH (HIGH part for C64xx only)
#define C6X_EDMA_CCER			((unsigned *) 0x01A0ffec)
#define C6X_EDMA_CCERL			((unsigned *) 0x01A0ffec)
#define C6X_EDMA_CCERH			((unsigned *) 0x01A0ffac)

	// priority queue status
#define C6X_EDMA_PQSR			((unsigned *) 0x01A0ffe0)

	// C64xx only: priority queue allocation regs
#define C6X_EDMA_PQAR0			((unsigned *) 0x01A0ffc0)
#define C6X_EDMA_PQAR1			((unsigned *) 0x01A0ffc4)
#define C6X_EDMA_PQAR2			((unsigned *) 0x01A0ffc8)
#define C6X_EDMA_PQAR3			((unsigned *) 0x01A0ffcc)


	//	EDMA channel descriptors (located in PaRAM)
		// EDMA parameter RAM channel and reload-link channel descriptor def
typedef	struct
{
	unsigned 		opt;
	unsigned 		src_addr;
	unsigned short	element_count;
	unsigned short	array_frame_count;
	unsigned 		dst_addr;
	unsigned short	element_index;
	unsigned short	array_frame_index;
	unsigned short	link_addr;
	unsigned short	element_count_reload;
}
C6X_EDMA_CH_DD;


#define C6X_EDMA_PRAM_BADDR				0x01A00000		// base address for EDMA parameter RAM

#define C64XX_EDMA_MAX_CH_COUNT			64				// C64xx: max number of EDMA channels
#define C64XX_EDMA_MAX_RLCH_COUNT		21				// C64xx: max number of EDMA reload-link channel descriptors

#define C621X_C671X_EDMA_MAX_CH_COUNT	16		// C621x/C671x: max number of EDMA channels
#define C641X_C671X_EDMA_MAX_RLCH_COUNT	69		// C621x/C671x: max number of EDMA reload-link channel descriptors

			// macros for getting pointer for EDMA channel and reload-link channel descriptors
			// user must use the follow syntax to access channel descitor:
			//		C6X_EDMA_CH_DPTR(ch)->opt = ...;
			//			etc...
#define C6X_EDMA_CH_DPTR(ch)			((C6X_EDMA_CH_DD *) (C6X_EDMA_PRAM_BADDR+ (ch%85)*24))

			// direct access to channel and reload-link cannel descriptor regs
#define C6X_EDMA_ORG(ch)				((unsigned *) (C6X_EDMA_PRAM_BADDR+ (ch%85)*24))
#define C6X_EDMA_SARG(ch)				((unsigned *) (C6X_EDMA_PRAM_BADDR+ (ch%85)*24) +0x4))
#define C6X_EDMA_CRG(ch)				((unsigned *) (C6X_EDMA_PRAM_BADDR+ (ch%85)*24) +0x8))
#define C6X_EDMA_DARG(ch)				((unsigned *) (C6X_EDMA_PRAM_BADDR+ (ch%85)*24) +0xc))
#define C6X_EDMA_IRG(ch)				((unsigned *) (C6X_EDMA_PRAM_BADDR+ (ch%85)*24) +0x10))
#define C6X_EDMA_RLRG(ch)				((unsigned *) (C6X_EDMA_PRAM_BADDR+ (ch%85)*24) +0x14))

#define C6X_EDMA_PARAM_SCRPAD0_RG		((unsigned *) (C6X_EDMA_PRAM_BADDR+ 85*24))
#define C6X_EDMA_PARAM_SCRPAD1_RG		((unsigned *) (C6X_EDMA_PRAM_BADDR+ 85*24 + 4))


#ifdef	__C6X_SELECT_C64XX_DSP__
//>>>>>>> C64xx cache control registers
#define C6X_CCFG				((unsigned *) 0x01840000)
#define C6X_L2ALLOC0			((unsigned *) 0x01842000)
#define C6X_L2ALLOC1			((unsigned *) 0x01842004)
#define C6X_L2ALLOC2			((unsigned *) 0x01842008)
#define C6X_L2ALLOC3			((unsigned *) 0x0184200c)
#define C6X_L2FBAR				((unsigned *) 0x01844000)
#define C6X_L2FWC				((unsigned *) 0x01844004)
#define C6X_L2CBAR				((unsigned *) 0x01844010)
#define C6X_L2CWC				((unsigned *) 0x01844014)
#define C6X_L1PFBAR				((unsigned *) 0x01844020)
#define C6X_L1PFWC				((unsigned *) 0x01844024)
#define C6X_L1DFBAR				((unsigned *) 0x01844030)
#define C6X_L2FLUSH				((unsigned *) 0x01845000)
#define C6X_L2CLEAN				((unsigned *) 0x01845004)


	// DEVICE_REV register.
#define C6X_DEVICE_REV			((unsigned *) 0x01B00200)

	// MAR0..255 registers.
	// Note, that C64x has MAR96..111 and MAR128..191 valid regs only.
#define C6X_MAR(n)				((unsigned *) (0x01848000+n*4))


//>>>>>>>>>>>>> C64xx on-chip GPIO control registers
// (notation of GPIO control registers corresponds to TI notation)
#define C6X_GPEN				((unsigned *) 0x01B00000)
#define C6X_GPDIR				((unsigned *) 0x01B00004)
#define C6X_GPVAL				((unsigned *) 0x01B00008)
#define C6X_GPDH				((unsigned *) 0x01B00010)
#define C6X_GPHM				((unsigned *) 0x01B00014)
#define C6X_GPDL				((unsigned *) 0x01B00018)
#define C6X_GPLM				((unsigned *) 0x01B0001c)
#define C6X_GPGC				((unsigned *) 0x01B00020)
#define C6X_GPPOL				((unsigned *) 0x01B00024)

#endif
//-----------------------------------------------------------------
#endif
//=====================================================================



#ifdef	__C6X_SELECT_C620X_C670X_DSP_RG_PTR_VAR__
//============= old C620x/C670x peripheral regs definitions ===========
// old DSP on-chip peripheral register allocated defs for C620x/C670x
// with poiners allocation in the .BSS section

//-------------- memory mapped peripheral registers --------------

//>>>>>>>>>> EMIF control RGs
unsigned *C6X_EMIF_GCR        	= (unsigned *) 0x01800000;
unsigned *C6X_EMIF_CE0        	= (unsigned *) 0x01800008;
unsigned *C6X_EMIF_CE1        	= (unsigned *) 0x01800004;
unsigned *C6X_EMIF_CE2        	= (unsigned *) 0x01800010;
unsigned *C6X_EMIF_CE3        	= (unsigned *) 0x01800014;
unsigned *C6X_EMIF_SDRAM      	= (unsigned *) 0x01800018;
unsigned *C6X_EMIF_SDRAM_TIMER	= (unsigned *) 0x0180001c;

//>>>>>>>>>>>>>> TIMER Control
unsigned *C6X_TM0_CONTROL     	= (unsigned *) 0x01940000;
unsigned *C6X_TM0_PERIOD      	= (unsigned *) 0x01940004;
unsigned *C6X_TM0_COUNTER     	= (unsigned *) 0x01940008;
unsigned *C6X_TM1_CONTROL     	= (unsigned *) 0x01980000;
unsigned *C6X_TM1_PERIOD      	= (unsigned *) 0x01980004;
unsigned *C6X_TM1_COUNTER     	= (unsigned *) 0x01980008;

//>>>>>>>>>>>>>> MCBSP Registers
unsigned *C6X_MCBSP0_DRR      	= (unsigned *) 0x018C0000;
unsigned *C6X_MCBSP0_DXR      	= (unsigned *) 0x018C0004;
unsigned *C6X_MCBSP0_SPCR     	= (unsigned *) 0x018C0008;
unsigned *C6X_MCBSP0_RCR      	= (unsigned *) 0x018C000C;
unsigned *C6X_MCBSP0_XCR      	= (unsigned *) 0x018C0010;
unsigned *C6X_MCBSP0_SRGR     	= (unsigned *) 0x018C0014;
unsigned *C6X_MCBSP0_MCR      	= (unsigned *) 0x018C0018;
unsigned *C6X_MCBSP0_RCER     	= (unsigned *) 0x018C001C;
unsigned *C6X_MCBSP0_XCER     	= (unsigned *) 0x018C0020;
unsigned *C6X_MCBSP0_PCR      	= (unsigned *) 0x018C0024;

unsigned *C6X_MCBSP1_DRR      	= (unsigned *) 0x01900000;
unsigned *C6X_MCBSP1_DXR      	= (unsigned *) 0x01900004;
unsigned *C6X_MCBSP1_SPCR     	= (unsigned *) 0x01900008;
unsigned *C6X_MCBSP1_RCR      	= (unsigned *) 0x0190000C;
unsigned *C6X_MCBSP1_XCR      	= (unsigned *) 0x01900010;
unsigned *C6X_MCBSP1_SRGR     	= (unsigned *) 0x01900014;
unsigned *C6X_MCBSP1_MCR      	= (unsigned *) 0x01900018;
unsigned *C6X_MCBSP1_RCER     	= (unsigned *) 0x0190001C;
unsigned *C6X_MCBSP1_XCER     	= (unsigned *) 0x01900020;
unsigned *C6X_MCBSP1_PCR      	= (unsigned *) 0x01900024;

unsigned *C6X_MCBSP2_DRR      	= (unsigned *) 0x01a40000;	// available in C6202/C6203 only
unsigned *C6X_MCBSP2_DXR      	= (unsigned *) 0x01a40004;
unsigned *C6X_MCBSP2_SPCR     	= (unsigned *) 0x01a40008;
unsigned *C6X_MCBSP2_RCR      	= (unsigned *) 0x01a4000C;
unsigned *C6X_MCBSP2_XCR      	= (unsigned *) 0x01a40010;
unsigned *C6X_MCBSP2_SRGR     	= (unsigned *) 0x01a40014;
unsigned *C6X_MCBSP2_MCR      	= (unsigned *) 0x01a40018;
unsigned *C6X_MCBSP2_RCER     	= (unsigned *) 0x01a4001C;
unsigned *C6X_MCBSP2_XCER     	= (unsigned *) 0x01a40020;
unsigned *C6X_MCBSP2_PCR      	= (unsigned *) 0x01a40024;

//>>>>>>>>>>>>> HPI registers
unsigned *C6X_HPIC	      		= (unsigned *) 0x01880000;
unsigned *C6X_XBGC	      		= (unsigned *) 0x01880000;
unsigned *C6X_XCE0_SCR	      	= (unsigned *) 0x01880008;
unsigned *C6X_XCE1_SCR	      	= (unsigned *) 0x01880004;
unsigned *C6X_XCE2_SCR	      	= (unsigned *) 0x01880010;
unsigned *C6X_XCE3_SCR	      	= (unsigned *) 0x01880014;
unsigned *C6X_XBHC	      		= (unsigned *) 0x0188000c;
unsigned *C6X_XBIMA	      		= (unsigned *) 0x01880020;
unsigned *C6X_XBEA	      		= (unsigned *) 0x01880024;

//>>>>>>>>>>>>> INT specific rgs
unsigned *C6X_INTMUX_HIGH     	= (unsigned *) 0x019C0000;
unsigned *C6X_INTMUX_LOW      	= (unsigned *) 0x019C0004;
unsigned *C6X_XINT_POLARITY   	= (unsigned *) 0x019C0008;

//>>>>>>>>>>>>> DMA control registers
unsigned *C6X_DMA0_PCRG			= (unsigned *) 0x01840000;
unsigned *C6X_DMA0_SCRG			= (unsigned *) 0x01840008;
unsigned *C6X_DMA0_SARG			= (unsigned *) 0x01840010;
unsigned *C6X_DMA0_DARG			= (unsigned *) 0x01840018;
unsigned *C6X_DMA0_TCRG			= (unsigned *) 0x01840020;

unsigned *C6X_DMA1_PCRG			= (unsigned *) 0x01840040;
unsigned *C6X_DMA1_SCRG			= (unsigned *) 0x01840048;
unsigned *C6X_DMA1_SARG			= (unsigned *) 0x01840050;
unsigned *C6X_DMA1_DARG			= (unsigned *) 0x01840058;
unsigned *C6X_DMA1_TCRG			= (unsigned *) 0x01840060;

unsigned *C6X_DMA2_PCRG			= (unsigned *) 0x01840004;
unsigned *C6X_DMA2_SCRG			= (unsigned *) 0x0184000c;
unsigned *C6X_DMA2_SARG			= (unsigned *) 0x01840014;
unsigned *C6X_DMA2_DARG			= (unsigned *) 0x0184001c;
unsigned *C6X_DMA2_TCRG			= (unsigned *) 0x01840024;

unsigned *C6X_DMA3_PCRG			= (unsigned *) 0x01840044;
unsigned *C6X_DMA3_SCRG			= (unsigned *) 0x0184004c;
unsigned *C6X_DMA3_SARG			= (unsigned *) 0x01840054;
unsigned *C6X_DMA3_DARG			= (unsigned *) 0x0184005c;
unsigned *C6X_DMA3_TCRG			= (unsigned *) 0x01840064;

unsigned *C6X_DMA0_GAARG		= (unsigned *) 0x01840038;
unsigned *C6X_DMA0_GABRG		= (unsigned *) 0x0184003C;
unsigned *C6X_DMA0_GACRG		= (unsigned *) 0x01840068;
unsigned *C6X_DMA0_GADRG		= (unsigned *) 0x0184006C;
unsigned *C6X_DMA0_GCRARG		= (unsigned *) 0x01840028;
unsigned *C6X_DMA0_GCRBRG		= (unsigned *) 0x0184002C;
unsigned *C6X_DMA0_GIARG		= (unsigned *) 0x01840030;
unsigned *C6X_DMA0_GIBRG		= (unsigned *) 0x01840034;

unsigned *C6X_DMAAUX_CRG		= (unsigned *) 0x01840070;
#endif
//=====================================================================



//========================== CPU REGISTERS ============================
extern cregister volatile unsigned int AMR;     // Address Mode Register      
extern cregister volatile unsigned int CSR;     // Control Status Register    
extern cregister volatile unsigned int IFR;     // Interrupt Flag Register    
extern cregister volatile unsigned int ISR;     // Interrupt Set Register     
extern cregister volatile unsigned int ICR;     // Interrupt Clear Register   
extern cregister volatile unsigned int IER;     // Interrupt Enable Register  
extern cregister volatile unsigned int ISTP;    // Interrupt Service Tbl Ptr  
extern cregister volatile unsigned int IRP;     // Interrupt Return Pointer   
extern cregister volatile unsigned int NRP;     // Non-maskable Int Return Ptr
extern cregister volatile unsigned int IN;      // General Purpose Input Reg  
extern cregister volatile unsigned int OUT;     // General Purpose Output Reg 
//=====================================================================


//=========== C6x HPIC register bits defs =============================
#define	C6X_HPIC_DSPINT_BITMASK			0x2
#define	C6X_HPI_DSPINT_BITMASK			0x2
#define	C6201_C6701_HPIC_HINT_BITMASK	0x4	// C6201/C6701 only
#define	C6201_C6701_HPI_HINT_BITMASK	0x4	// C6201/C6701 only
//=====================================================================


//========================= C6x macros ================================

// 	Cache control macros. Cache control is different for C620x/C670x and C64x.
//	C620x/C670x: program cache only.
//	C64xx: program and data cache.
#ifdef	__C620X_C670X_DSP_DEF__
#define C6X_CACHE_ENABLE				CSR |=0x40		// enable program cache only
#define C6X_CACHE_DISABLE				CSR &=~0x40		// disable program cache
	// aliases to meet SET/GET API
#define C6X_SET_CACHE_ENABLE_ON			C6X_CACHE_ENABLE
#define C6X_SET_CACHE_ENABLE_OFF		C6X_CACHE_DISABLE
#endif

#ifdef	__C6X_SELECT_C64XX_DSP__
#define C6X_CACHE_ENABLE				CSR |=0x48
#define C6X_CACHE_DISABLE				CSR &=~0x48		// disable program/data cache
	// aliases to meet SET/GET API
#define C6X_SET_CACHE_ENABLE_ON			C6X_CACHE_ENABLE
#define C6X_SET_CACHE_ENABLE_OFF		C6X_CACHE_DISABLE

#define C6X_SET_L1P_CACHE_ENABLE_ON		CSR |=0x40		// enable program cache
#define C6X_SET_L1P_CACHE_ENABLE_OFF	CSR &=~0x40		// disable program cache

#define C6X_SET_L1D_CACHE_ENABLE_ON		CSR |=0x8		// enable data cache
#define C6X_SET_L1D_CACHE_ENABLE_OFF	CSR &=~0x8		// disable data cache

#define C6X_SET_L1P_CACHE_INVALIDATE	*C6X_CCFG |=0x200	// set L1 program cache invalidation
#define C6X_SET_L1P_CACHE_NORMAL		*C6X_CCFG &=~0x200	// set L1 program cache normal operation

#define C6X_SET_L1D_CACHE_INVALIDATE	*C6X_CCFG |=0x100	// set L1 data cache invalidation
#define C6X_SET_L1D_CACHE_NORMAL		*C6X_CCFG &=~0x100	// set L1 data cache normal operation

#define C6X_GET_L2_CACHE_MODE			((*C6X_CCFG) &0x7)	// returns L2 cache mode
#define C6X_SET_L2_CACHE_MODE(mode)		*C6X_CCFG = ((*C6X_CCFG) & ~0x7) | (mode&0x7)) // set L2 cache mode

#define	C6X_L2_CACHE_MODE_NO_CACHE		0		// no cache, full SRAM allocation
#define	C6X_L2_CACHE_MODE_1_32_CACHE	1		// cache occupies 1/32 of SRAM
#define	C6X_L2_CACHE_MODE_1_16_CACHE	2		// cache occupies 1/16 of SRAM
#define	C6X_L2_CACHE_MODE_1_8_CACHE		3		// cache occupies 1/8 of SRAM
#define	C6X_L2_CACHE_MODE_1_4_CACHE		7		// cache occupies 1/4 of SRAM

#endif


// 	Silicon Revision ID access macros. (available for C64xx DSP only)
#ifdef	__C6X_SELECT_C64XX_DSP__
#define C64XX_SILREV_BITMASK			0xf0000
#define C64XX_GET_SILREV				((*C6X_DEVICE_REV) & C64XX_SILREV_BITMASK)

#define C64XX_SILREV_IS_SILREV103			(C64XX_GET_SILREV == 0x10000)
#define C64XX_SILREV_IS_SILREV11			(C64XX_GET_SILREV == 0x20000)
//#define C64XX_SILREV_IS_EARLIERSILREV103	((C64XX_GET_SILREV == 0xf0000) || (C64XX_GET_SILREV == 0x00000))
#define C64XX_SILREV_IS_EARLIERSILREV103	(!(C64XX_SILREV_IS_SILREV103 || C64XX_SILREV_IS_SILREV11))
#endif

//=====================================================================


//========================= IRQ control ===============================

#define C6X_GINT_DISABLE				{CSR &=~1;}
#define C6X_GINT_ENABLE					{IER |=2; CSR |=1;}
#define C6X_NMI_ENABLE					{IER |=2;}
#define C6X_PGINT_ENABLE				{CSR |=2;}
#define C6X_PGINT_DISABLE				{CSR &=~2;}

#define C6X_INT_CLEAR_ALL       		{ICR = 0xfff0 ;}

#define C6X_INT_CLEAR(n)        		{ICR = 1<<(n) ;}
#define C6X_INT_SET(n)          		{ ISR = 1<<(n) ;}

#define C6X_INT_ENABLE(n)       		{ IER |= (1<<(n)) ;}
#define C6X_INT_DISABLE(n)      		{ IER &= ~(1<<(n)) ;}
#define C6X_INT_DISABLE_ALL     		{ IER = 0;}

	// aliases to meet SET/GET API
#define C6X_SET_GINT_ENABLE_OFF			C6X_GINT_DISABLE
#define C6X_SET_GINT_ENABLE_ON			C6X_GINT_ENABLE
#define C6X_SET_NMI_ENABLE_ON			C6X_NMI_ENABLE
#define C6X_SET_PGINT_ENABLE_ON			C6X_PGINT_ENABLE
#define C6X_SET_PGINT_ENABLE_OFF		C6X_PGINT_DISABLE

#define C6X_SET_INT_OFF_ALL       		C6X_INT_CLEAR_ALL

#define C6X_SET_INT_OFF(n)        		C6X_INT_CLEAR(n)
#define C6X_SET_INT_ON(n)          		C6X_INT_SET(n)

#define C6X_INT_IS_OFF(n)        		((IFR&(1<<(n)))==0)
#define C6X_INT_IS_ON(n)        		((IFR&(1<<(n)))!=0)

#define C6X_SET_INT_ENABLE_ON(n)       	C6X_INT_ENABLE(n)
#define C6X_SET_INT_ENABLE_OFF(n)      	C6X_INT_DISABLE(n)
#define C6X_SET_INT_ENABLE_OFF_ALL     	C6X_INT_DISABLE_ALL

#define C6X_INT_ENABLE_IS_OFF(n)       	((IER&(1<<(n)))==0)
#define C6X_INT_ENABLE_IS_ON(n)       	((IER&(1<<(n)))!=0)


// 	this function sets INT MUX regsiter in order to map particular source
//	interrupt to C6x interrupt line
int C6X_INTMUX_SET(i,x)
{
	if(  (i)<4	|| (i)>15	|| (x)>15)
		return -1;	// exit with error on erroneus params

	if((i)>=4 && (i)<=9)
	{
       	(*C6X_INTMUX_LOW)  &= ~(0xf       <<((((i)-4)*5)+(((i)>=7)?1:0)));
       	(*C6X_INTMUX_LOW)  |=  (((x) & 0xf) <<((((i)-4)*5)+(((i)>=7)?1:0)));
    }
	else
	{
		(*C6X_INTMUX_HIGH) &= ~(0xf       <<((((i)-10)*5)+(((i)>=13)?1:0)));
		(*C6X_INTMUX_HIGH) |=  (((x) & 0xf) <<((((i)-10)*5)+(((i)>=13)?1:0)));
	}

    return(0);	// exit OK
}

	// aliases to meet SET/GET API
#define	C6X_SET_INTMUX(i,x)				C6X_INTMUX_SET(i,x)


	// INT selector codes for INT-MUX mapping
#define C6X_INTSEL_DSPINT				0
#define C6X_INTSEL_TINT0				1
#define C6X_INTSEL_TINT1				2
#define C6X_INTSEL_SDINT				3
#define C6X_INTSEL_SDINTA				3	// C64xx only
#define C6X_INTSEL_XINT4				4
#define C6X_INTSEL_GPINT4				4	// C64xx only
#define C6X_INTSEL_XINT5				5
#define C6X_INTSEL_GPINT5				5	// C64xx only
#define C6X_INTSEL_XINT6				6
#define C6X_INTSEL_GPINT6				6	// C64xx only
#define C6X_INTSEL_XINT7				7
#define C6X_INTSEL_GPINT7				7	// C64xx only
#define C6X_INTSEL_DMA0					8
#define C6X_INTSEL_EDMA					8	// C621x/C671x/C64xx only
#define C6X_INTSEL_DMA1					9
#define C6X_INTSEL_DMA2					10
#define C6X_INTSEL_DMA3					11
#define C6X_INTSEL_MCBSP0_XINT			12
#define C6X_INTSEL_MCBSP0_RINT			13
#define C6X_INTSEL_MCBSP1_XINT			14
#define C6X_INTSEL_MCBSP1_RINT			15
#define C6X_INTSEL_GPINT0				16	// C64xx only
#define C6X_INTSEL_MCBSP2_XINT			17
#define C6X_INTSEL_PCI_WAKEUP			17	// C6205
#define C6X_INTSEL_MCBSP2_RINT			18
#define C6X_INTSEL_ADMA_HLT				18	// C6205
#define C6X_INTSEL_TINT2				19	// C64xx only
#define C6X_INTSEL_SDINTB				20	// C64xx only
#define C6X_INTSEL_C64XX_PCI_WAKEUP		21	// C64xx only
#define C6X_INTSEL_UINT					23	// C64xx only

#define C6X_XINT_POLARITY_POS			0
#define C6X_XINT_POLARITY_NEG			1
#define C6X_XINT4						4
#define C6X_XINT5						5
#define C6X_XINT6						6
#define C6X_XINT7						7

#define C6X_XINT_POLARITY_SET(i,p)  	{*C6X_XINT_POLARITY &=~(1<<((i)-4));*C6X_XINT_POLARITY |=(((p)&1)<<((i)-4));}
#define C6X_XINT_ALL_POLARITY_SET(p)	{*C6X_XINT_POLARITY = ((p)==0)? 0:0xf;}
	// aliases to meet SET/GET API
#define C6X_SET_XINT_POLARITY(i,p)		C6X_XINT_POLARITY_SET(i,p)
#define C6X_SET_XINT_POLARITY_ALL(p)	C6X_XINT_ALL_POLARITY_SET(p)
//=====================================================================



//========================= C64xx GPIO control ========================
#ifdef	__C6X_SELECT_C64XX_DSP__

// GPIO enable (GPEN reg)
#define C6X_GPIO_ENABLE_OFF				0
#define C6X_GPIO_ENABLE_ON				1

// used for GPDIR register to define GPIO direction
#define C6X_GPIO_DIR_INPUT				0
#define C6X_GPIO_DIR_OUTPUT				1

// used for GPVAL, GPDH, GPDL, GPHM, GPLM regs
#define C6X_GPIO_DATA_OFF				0	
#define C6X_GPIO_DATA_ON				1

#define C6X_GPIO_DELTA_OFF				0
#define C6X_GPIO_DELTA_ON				1

#define C6X_GPIO_MASK_OFF				0
#define C6X_GPIO_MASK_ON				1

// GPEN, GPVAL, GPDI, GPDH/GPDL, GPHM/GPLM registers set/get macros
// These macros operate with individual bits only and use universal shifted left 
// bit defs above (v) and require bit # (i) in order to set/get register value. 
// Returned value is shifted right to cpmly with above defs.
// Bit # (i) must be within 0..15 range. Bit value (v) must be 0..1.

#define C6X_SET_GPIO_ENABLE(i,v)  		*C6X_GPEN = ( ((*C6X_GPEN) & ~(1<<(i&0xf))) | ((v&1) << (i&0xf)) )
#define C6X_GET_GPIO_ENABLE(i)  		 ( ((*C6X_GPEN) & (1<<(i&0xf))) >> (i&0xf) ) 

#define C6X_SET_GPIO_DIR(i,v)	  		*C6X_GPDIR = ( ((*C6X_GPDIR) & ~(1<<(i&0xf))) | ((v&1) << (i&0xf)) )
#define C6X_GET_GPIO_DIR(i)  			 ( ((*C6X_GPDIR) & (1<<(i&0xf))) >> (i&0xf) ) 

#define C6X_SET_GPIO_DATA(i,v)	  		*C6X_GPVAL = ( ((*C6X_GPVAL) & ~(1<<(i&0xf))) | ((v&1) << (i&0xf)) )
#define C6X_GET_GPIO_DATA(i)  			 ( ((*C6X_GPVAL) & (1<<(i&0xf))) >> (i&0xf) ) 

#define C6X_CLEAR_GPIO_DH(i)	  		*C6X_GPDH = (1 << (i&0xf))
#define C6X_GET_GPIO_DH(i)  			 ( ((*C6X_GPDH) & (1<<(i&0xf))) >> (i&0xf) ) 

#define C6X_CLEAR_GPIO_DL(i)	  		*C6X_GPDL = (1 << (i&0xf))
#define C6X_GET_GPIO_DL(i)  			 ( ((*C6X_GPDL) & (1<<(i&0xf))) >> (i&0xf) ) 

#define C6X_SET_GPIO_HM(i,v)	  		*C6X_GPHM = ( ((*C6X_GPHM) & ~(1<<(i&0xf))) | ((v&1) << (i&0xf)) )
#define C6X_GET_GPIO_HM(i)  			 ( ((*C6X_GPHM) & (1<<(i&0xf))) >> (i&0xf) ) 

#define C6X_SET_GPIO_LM(i,v)	  		*C6X_GPLM = ( ((*C6X_GPLM) & ~(1<<(i&0xf))) | ((v&1) << (i&0xf)) )
#define C6X_GET_GPIO_LM(i)  			 ( ((*C6X_GPLM) & (1<<(i&0xf))) >> (i&0xf) ) 


// macros to set/read all GPIO control registers
// These macros write/read full register contents only with built-in mask applied.
#define C6X_SET_GPIO_ENABLE_RG(v) 	 	*C6X_GPEN = (v & 0xffff)
#define C6X_GET_GPIO_ENABLE_RG() 		((*C6X_GPEN) & 0xffff)

#define C6X_SET_GPIO_DIR_RG(v)	 	 	*C6X_GPDIR = (v & 0xffff)
#define C6X_GET_GPIO_DIR_RG()  			((*C6X_GPDIR) & 0xffff)

#define C6X_SET_GPIO_DATA_RG(v)	 	 	*C6X_GPVAL = (v & 0xffff)
#define C6X_GET_GPIO_DATA_RG() 			((*C6X_GPVAL) & 0xffff)

#define C6X_SET_GPIO_DH_RG(v)		 	*C6X_GPDH = (v & 0xffff)
#define C6X_GET_GPIO_DH_RG()  			((*C6X_GPDH) & 0xffff)

#define C6X_SET_GPIO_DL_RG(v)	 	 	*C6X_GPDL = (v & 0xffff)
#define C6X_GET_GPIO_DL_RG()  			((*C6X_GPDL) & 0xffff)

#define C6X_SET_GPIO_HM_RG(v)	 	 	*C6X_GPHM = (v & 0xffff)
#define C6X_GET_GPIO_HM_RG()  			((*C6X_GPHM) & 0xffff)

#define C6X_SET_GPIO_LM_RG(v)	 	 	*C6X_GPLM = (v & 0xffff)
#define C6X_GET_GPIO_LM_RG()  			((*C6X_GPLM) & 0xffff)

#define C6X_SET_GPIO_GC_RG(v)	 	 	*C6X_GPGC = (v & 0x37)
#define C6X_GET_GPIO_GC_RG()  			((*C6X_GPGC) & 0x37)

#define C6X_SET_GPIO_POL_RG(v)	 		*C6X_GPPOL = (v & 0xf1)
#define C6X_GET_GPIO_POL_RG()  			((*C6X_GPPOL) & 0xf1)

#endif
//=====================================================================



#ifdef	__C620X_C670X_DSP_DEF__
//===================== C620x/C670x DSP memory areas ==================
// C620x/C670x DSP area defs, which are defined for all C620x/C670x defs

//------------------ C6x memory areas -----------------------------

#define C6X_MAP0_ONCHIP_PRAM_SADDR		0x01400000
#define C6201_MAP0_ONCHIP_PRAM_SADDR	0x01400000
#define C6201_MAP0_ONCHIP_PRAM_EADDR	0x0140ffff
#define C6701_MAP0_ONCHIP_PRAM_EADDR	0x0140ffff
#define C6202_MAP0_ONCHIP_PRAM_EADDR	0x0143ffff
#define C6203_MAP0_ONCHIP_PRAM_EADDR	0x0145ffff

#define C6X_MAP1_ONCHIP_PRAM_SADDR		0x00000000
#define C6201_MAP1_ONCHIP_PRAM_SADDR	0x00000000
#define C6201_MAP1_ONCHIP_PRAM_EADDR	0x0000ffff
#define C6701_MAP1_ONCHIP_PRAM_EADDR	0x0000ffff
#define C6202_MAP1_ONCHIP_PRAM_EADDR	0x0003ffff
#define C6203_MAP1_ONCHIP_PRAM_EADDR	0x0005ffff

#define C6201_ONCHIP_PRAM_LEN			0x00010000
#define C6701_ONCHIP_PRAM_LEN			0x00010000
#define C6202_ONCHIP_PRAM_LEN			0x00040000
#define C6203_ONCHIP_PRAM_LEN			0x00060000

#define C6X_ONCHIP_DRAM_SADDR			0x80000000
#define C6201_ONCHIP_DRAM_SADDR			0x80000000
#define C6201_ONCHIP_DRAM_EADDR			0x8000ffff
#define C6701_ONCHIP_DRAM_EADDR			0x8000ffff
#define C6202_ONCHIP_DRAM_EADDR			0x8001ffff
#define C6203_ONCHIP_DRAM_EADDR			0x8007ffff

#define C6201_ONCHIP_DRAM_LEN			0x00010000
#define C6701_ONCHIP_DRAM_LEN			0x00010000
#define C6202_ONCHIP_DRAM_LEN			0x00020000
#define C6203_ONCHIP_DRAM_LEN			0x00080000

#define C6X_ONCHIP_RG_AREA_SADDR		0x01800000
#define C6X_ONCHIP_RG_AREA_EADDR		0x019fffff
#define C6X_ONCHIP_RG_AREA_LEN			0x00200000

#define C6X_MAP0_CE0_AREA_SADDR			0x00000000
#define C6X_MAP0_CE0_AREA_EADDR			0x00ffffff
#define C6X_MAP0_CE0_AREA_LEN			0x01000000
#define C6X_MAP1_CE0_AREA_SADDR			0x00400000
#define C6X_MAP1_CE0_AREA_EADDR			0x013fffff
#define C6X_MAP1_CE0_AREA_LEN			0x01000000

#define C6X_MAP0_CE1_AREA_SADDR			0x01000000
#define C6X_MAP0_CE1_AREA_EADDR			0x013fffff
#define C6X_MAP0_CE1_AREA_LEN			0x00400000
#define C6X_MAP1_CE1_AREA_SADDR			0x01400000
#define C6X_MAP1_CE1_AREA_EADDR			0x017fffff
#define C6X_MAP1_CE1_AREA_LEN			0x00400000

#define C6X_CE2_AREA_SADDR				0x02000000
#define C6X_CE2_AREA_EADDR				0x02ffffff
#define C6X_CE2_AREA_LEN				0x01000000

#define C6X_CE3_AREA_SADDR				0x03000000
#define C6X_CE3_AREA_EADDR				0x03ffffff
#define C6X_CE3_AREA_LEN				0x01000000
//-----------------------------------------------------------------

#endif
//=====================================================================




#ifdef	__C6X_SELECT_C64XX_DSP__
//===================== C64xx DSP memory areas ========================

//------------------ C6x memory areas -----------------------------

#define C64XX_ONCHIP_RAM_SADDR			0x00000000
#define C64XX_ONCHIP_RAM_EADDR			0x000fffff
#define C64XX_ONCHIP_RAM_LEN			0x00100000

#define C6X_ONCHIP_RAM_SADDR			C64XX_ONCHIP_RAM_SADDR
#define C6X_ONCHIP_RAM_EADDR			C64XX_ONCHIP_RAM_EADDR
#define C6X_ONCHIP_RAM_LEN				C64XX_ONCHIP_RAM_LEN


#define C6X_ONCHIP_L2RAM_SADDR			C64XX_ONCHIP_RAM_SADDR
#define C6X_ONCHIP_L2RAM_EADDR			C64XX_ONCHIP_RAM_EADDR
#define C6X_ONCHIP_L2RAM_LEN			C64XX_ONCHIP_RAM_LEN

#define C6X_ONCHIP_PRAM_SADDR			C64XX_ONCHIP_RAM_SADDR
#define C6X_ONCHIP_PRAM_EADDR			C64XX_ONCHIP_RAM_EADDR
#define C6X_ONCHIP_PRAM_LEN				C64XX_ONCHIP_RAM_LEN

#define C6X_ONCHIP_DRAM_SADDR			C64XX_ONCHIP_RAM_SADDR
#define C6X_ONCHIP_DRAM_EADDR			C64XX_ONCHIP_RAM_EADDR
#define C6X_ONCHIP_DRAM_LEN				C64XX_ONCHIP_RAM_LEN


#define C64XX_EMIFA_CE0_AREA_SADDR		0x80000000
#define C64XX_EMIFA_CE0_AREA_EADDR		0x8fffffff
#define C64XX_EMIFA_CE0_AREA_LEN		0x10000000

#define C64XX_EMIFA_CE1_AREA_SADDR		0x90000000
#define C64XX_EMIFA_CE1_AREA_EADDR		0x9fffffff
#define C64XX_EMIFA_CE1_AREA_LEN		0x10000000

#define C64XX_EMIFA_CE2_AREA_SADDR		0xa0000000
#define C64XX_EMIFA_CE2_AREA_EADDR		0xafffffff
#define C64XX_EMIFA_CE2_AREA_LEN		0x10000000

#define C64XX_EMIFA_CE3_AREA_SADDR		0xb0000000
#define C64XX_EMIFA_CE3_AREA_EADDR		0xbfffffff
#define C64XX_EMIFA_CE3_AREA_LEN		0x10000000


#define C6X_EMIFA_CE0_AREA_SADDR		C64XX_EMIFA_CE0_AREA_SADDR
#define C6X_EMIFA_CE0_AREA_EADDR		C64XX_EMIFA_CE0_AREA_EADDR
#define C6X_EMIFA_CE0_AREA_LEN			C64XX_EMIFA_CE0_AREA_LEN

#define C6X_EMIFA_CE1_AREA_SADDR		C64XX_EMIFA_CE1_AREA_SADDR
#define C6X_EMIFA_CE1_AREA_EADDR		C64XX_EMIFA_CE1_AREA_EADDR
#define C6X_EMIFA_CE1_AREA_LEN			C64XX_EMIFA_CE1_AREA_LEN

#define C6X_EMIFA_CE2_AREA_SADDR		C64XX_EMIFA_CE2_AREA_SADDR
#define C6X_EMIFA_CE2_AREA_EADDR		C64XX_EMIFA_CE2_AREA_EADDR
#define C6X_EMIFA_CE2_AREA_LEN			C64XX_EMIFA_CE2_AREA_LEN

#define C6X_EMIFA_CE3_AREA_SADDR		C64XX_EMIFA_CE3_AREA_SADDR
#define C6X_EMIFA_CE3_AREA_EADDR		C64XX_EMIFA_CE3_AREA_EADDR
#define C6X_EMIFA_CE3_AREA_LEN			C64XX_EMIFA_CE3_AREA_LEN

#define C6X_CE0_AREA_SADDR				C64XX_EMIFA_CE0_AREA_SADDR
#define C6X_CE0_AREA_EADDR				C64XX_EMIFA_CE0_AREA_EADDR
#define C6X_CE0_AREA_LEN				C64XX_EMIFA_CE0_AREA_LEN

#define C6X_CE1_AREA_SADDR				C64XX_EMIFA_CE1_AREA_SADDR
#define C6X_CE1_AREA_EADDR				C64XX_EMIFA_CE1_AREA_EADDR
#define C6X_CE1_AREA_LEN				C64XX_EMIFA_CE1_AREA_LEN

#define C6X_CE2_AREA_SADDR				C64XX_EMIFA_CE2_AREA_SADDR
#define C6X_CE2_AREA_EADDR				C64XX_EMIFA_CE2_AREA_EADDR
#define C6X_CE2_AREA_LEN				C64XX_EMIFA_CE2_AREA_LEN

#define C6X_CE3_AREA_SADDR				C64XX_EMIFA_CE3_AREA_SADDR
#define C6X_CE3_AREA_EADDR				C64XX_EMIFA_CE3_AREA_EADDR
#define C6X_CE3_AREA_LEN				C64XX_EMIFA_CE3_AREA_LEN


#define C64XX_EMIFB_CE0_AREA_SADDR		0x60000000
#define C64XX_EMIFB_CE0_AREA_EADDR		0x63ffffff
#define C64XX_EMIFB_CE0_AREA_LEN		0x04000000

#define C64XX_EMIFB_CE1_AREA_SADDR		0x64000000
#define C64XX_EMIFB_CE1_AREA_EADDR		0x67ffffff
#define C64XX_EMIFB_CE1_AREA_LEN		0x04000000

#define C64XX_EMIFB_CE2_AREA_SADDR		0x68000000
#define C64XX_EMIFB_CE2_AREA_EADDR		0x6Bffffff
#define C64XX_EMIFB_CE2_AREA_LEN		0x04000000

#define C64XX_EMIFB_CE3_AREA_SADDR		0x6C000000
#define C64XX_EMIFB_CE3_AREA_EADDR		0x6fffffff
#define C64XX_EMIFB_CE3_AREA_LEN		0x04000000

#define C6X_EMIFB_CE0_AREA_SADDR		C64XX_EMIFB_CE0_AREA_SADDR
#define C6X_EMIFB_CE0_AREA_EADDR		C64XX_EMIFB_CE0_AREA_EADDR
#define C6X_EMIFB_CE0_AREA_LEN			C64XX_EMIFB_CE0_AREA_LEN

#define C6X_EMIFB_CE1_AREA_SADDR		C64XX_EMIFB_CE1_AREA_SADDR
#define C6X_EMIFB_CE1_AREA_EADDR		C64XX_EMIFB_CE1_AREA_EADDR
#define C6X_EMIFB_CE1_AREA_LEN			C64XX_EMIFB_CE1_AREA_LEN

#define C6X_EMIFB_CE2_AREA_SADDR		C64XX_EMIFB_CE2_AREA_SADDR
#define C6X_EMIFB_CE2_AREA_EADDR		C64XX_EMIFB_CE2_AREA_EADDR
#define C6X_EMIFB_CE2_AREA_LEN			C64XX_EMIFB_CE2_AREA_LEN

#define C6X_EMIFB_CE3_AREA_SADDR		C64XX_EMIFB_CE3_AREA_SADDR
#define C6X_EMIFB_CE3_AREA_EADDR		C64XX_EMIFB_CE3_AREA_EADDR
#define C6X_EMIFB_CE3_AREA_LEN			C64XX_EMIFB_CE3_AREA_LEN


#define C64XX_MCBSP0_EDMA_DATA_AREA_SADDR	0x30000000
#define C64XX_MCBSP0_EDMA_DATA_AREA_EADDR	0x33ffffff
#define C64XX_MCBSP0_EDMA_DATA_AREA_LEN		0x04000000

#define C64XX_MCBSP1_EDMA_DATA_AREA_SADDR	0x34000000
#define C64XX_MCBSP1_EDMA_DATA_AREA_EADDR	0x37ffffff
#define C64XX_MCBSP1_EDMA_DATA_AREA_LEN		0x04000000

#define C64XX_MCBSP2_EDMA_DATA_AREA_SADDR	0x38000000
#define C64XX_MCBSP2_EDMA_DATA_AREA_EADDR	0x3bffffff
#define C64XX_MCBSP2_EDMA_DATA_AREA_LEN		0x04000000


#define C6X_MCBSP0_EDMA_DATA_AREA_SADDR		C64XX_MCBSP0_EDMA_DATA_AREA_SADDR
#define C6X_MCBSP0_EDMA_DATA_AREA_EADDR		C64XX_MCBSP0_EDMA_DATA_AREA_EADDR
#define C6X_MCBSP0_EDMA_DATA_AREA_LEN		C64XX_MCBSP0_EDMA_DATA_AREA_LEN

#define C6X_MCBSP1_EDMA_DATA_AREA_SADDR		C64XX_MCBSP1_EDMA_DATA_AREA_SADDR
#define C6X_MCBSP1_EDMA_DATA_AREA_EADDR		C64XX_MCBSP1_EDMA_DATA_AREA_EADDR
#define C6X_MCBSP1_EDMA_DATA_AREA_LEN		C64XX_MCBSP1_EDMA_DATA_AREA_LEN

#define C6X_MCBSP2_EDMA_DATA_AREA_SADDR		C64XX_MCBSP2_EDMA_DATA_AREA_SADDR
#define C6X_MCBSP2_EDMA_DATA_AREA_EADDR		C64XX_MCBSP2_EDMA_DATA_AREA_EADDR
#define C6X_MCBSP2_EDMA_DATA_AREA_LEN		C64XX_MCBSP2_EDMA_DATA_AREA_LEN


#define C64XX_ONCHIP_RG_AREA_SADDR		0x01800000	// peripheral regs
#define C64XX_ONCHIP_RG_AREA_EADDR		0x01c3ffff
#define C64XX_ONCHIP_RG_AREA_LEN		0x00440000

#define C64XX_ONCHIP_RG_AREA2_SADDR		0x02000000	// QDMA reg area
#define C64XX_ONCHIP_RG_AREA2_EADDR		0x02000033
#define C64XX_ONCHIP_RG_AREA2_LEN		0x00000034


#define C6X_ONCHIP_RG_AREA_SADDR		C64XX_ONCHIP_RG_AREA_SADDR
#define C6X_ONCHIP_RG_AREA_EADDR		C64XX_ONCHIP_RG_AREA_EADDR
#define C6X_ONCHIP_RG_AREA_LEN			C64XX_ONCHIP_RG_AREA_LEN	

#define C6X_ONCHIP_RG_AREA2_SADDR		C64XX_ONCHIP_RG_AREA2_SADDR
#define C6X_ONCHIP_RG_AREA2_EADDR		C64XX_ONCHIP_RG_AREA2_EADDR
#define C6X_ONCHIP_RG_AREA2_LEN			C64XX_ONCHIP_RG_AREA2_LEN


//-----------------------------------------------------------------

#endif
//=====================================================================




#endif
