/***************************************************************************

  TORNADO-P6x API Host Control Library. (Host PCI Environment) v.2.1
  (C) MicroLAB Systems Ltd, 1999-2002

  revision history:
     v.1.0 (July-1999):  initial entry for TORNADO-P62/P67 support
     v.2.0 (April-2000): added support for TORNADO-P6202/P6203, functions revise
     v.2.1 (June-2002): added support for TORNADO-P6414/P6415/P6416 f/w rev.1A
						(these DOS utilities provide limited support for TP64xx boards,
						 that denotes that only 32kByte sub-area of on-board DPRAM is
						 supported, which is allocated to UMB)
     v.2.4 (Jan-2004):   added support for TORNADO-P64xx board f/w rev.1A-1 and rev.1A-2
****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <conio.h>
#include <fcntl.h>
#include <types.h>
#include <stat.h>
#include <io.h>
#include <stdlib.h>
#include <errno.h>
#include <dos.h>
#include <bios.h>
#include <malloc.h>


#include "tp6x_h.h"
#include "tp6x_h.var"
#include "_pci.h"

//-- declarations for 32-bit port INPUT/OUTPUT functions (file _IOP32.ASM)
//   These functions are provided for TP6X_read_dword() and TP6X_write_dword()
//   fuctions in case PCI area is allocated to I/O and request is performed to
//   32-bit I/O port data. Microsoft C Compiler does not provide 32-bit INP/OUTP functions.
extern void	_inport32(unsigned port,unsigned long *data);
extern void	_outport32(unsigned port,unsigned long data);


//------------- local functions ------------------------------
static int _tp6x_set_err(int e)	// local, used for exit fm API fucntions to set error code
{
	if(e<0) _tp6x_err_flag=1;
   	_tp6x_last_err=e;
   	return(e);
}


//------------------ TP6X Utility functions ---------------------


int TP6X_last_err()		// returns last function error
{
	return(_tp6x_last_err);
}


int TP6X_err_flag()		// returns trigger error flag
{
	return(_tp6x_err_flag);
}


int TP6X_clr_err_flag()		// clears trigger error flag
{
	_tp6x_err_flag=0;
}


int TP6X_err_message(int err, char *s)	// interpretes error message
{  
	char s1[100];
 	
    if(s==NULL) return(TP6X_OK);

    strcpy(s,"TP6X>> ");

    switch(err)
	{
		case TP6X_OK:
			strcat(s,"OK");
			break;

		case TP6X_PARAM_ERR:
			strcat(s,"invalid function parameter");
			break;

		case TP6X_HW_ERR:
			strcat(s,"hardware error");
			break;

		case TP6X_NOT_OPEN_ERR:
			strcat(s,"API is not open");
			break;

		case TP6X_NO_BOARDS_FOUND_ERR:
			strcat(s,"no TORNADO-P6x boards found");
			break;

		case TP6X_INV_PCI_AREA_INFO_ERR:
			strcat(s,"invalid or ambigious PCI configiration info");
			break;

		case TP6X_INCOMPAT_PCI_AREA_INFO_ERR:
			strcat(s,"incompatible TORNADO-P6x PCI configiration info");
			break;

		case TP6X_INV_BOARD_INFO_ERR:
			strcat(s,"invalid TORNADO-P6x on-board resources info");
			break;

		case TP6X_BOARD_NOT_FOUND_ERR:
			strcat(s,"requested TORNADO-P6x board not found");
			break;

		case TP6X_INV_CMND_ERR:
			strcat(s,"function is not allowed");
			break;

		case TP6X_INV_BADDR_AREA_ERR:
			strcat(s,"access to invalid PCI pass-thru area");
			break;

		case TP6X_ACCESS_BADDR_AREA_ERR:
			strcat(s,"unable to access data of PCI pass-thru area");
			break;

		case TP6X_ACCESS_INV_BADDR_MEM_ERR:
			strcat(s,"access to PCI pass-thru area with invalid memory type");
			break;

		case TP6X_MEM_ALLOC_ERR:
			strcat(s,"not enough PC memory");
			break;

		default:
			//-- check for possible PCI-BIOS i/f error
			if(_PCI_err_message(err,s1)==_PCI_NO_ERR)
				strcpy(s,s1);
			else	
				strcat(s,"invalid error code");
			break;
    }
	return(TP6X_OK);
}


int TP6X_last_err_message(char *s)	// interpretes last error message
{  
	return(TP6X_err_message(_tp6x_last_err,s));
}

//---------------------------------------------------------------------

//------------------ API Open/Close -----------------------------------

	//--- local function used for step-by-step return
	//    of VENDOR_ID and DEV_ID for all available TP6X boards
	//    (used by TP6X_open())

static void _tp6x_board_list(int i, unsigned *vendor_id, unsigned *dev_id)
{
	switch(i)
	{
		case 0:
	    	*vendor_id=TP62_VENDOR_ID;
	       	*dev_id=TP62_DEV_ID;
	       	break;

		case 1:
	    	*vendor_id=TP67_VENDOR_ID;
	       	*dev_id=TP67_DEV_ID;
	       	break;

	    case 2:
	       	*vendor_id=TP6202_VENDOR_ID;
	       	*dev_id=TP6202_DEV_ID;
	       	break;

	    case 3:
	    	*vendor_id=TP6203_VENDOR_ID;
	       	*dev_id=TP6203_DEV_ID;
	       	break;

	    case 4:
	    	*vendor_id=TP6414_VENDOR_ID;
	       	*dev_id=TP6414_DEV_ID;
	       	break;

	    case 5:
	    	*vendor_id=TP6415_VENDOR_ID;
	       	*dev_id=TP6415_DEV_ID;
	       	break;

	    case 6:
	    	*vendor_id=TP6416_VENDOR_ID;
	       	*dev_id=TP6416_DEV_ID;
	       	break;

	    default:
			// internal error, return invlid PCI vendor/device ID codes
	       	*vendor_id=0xffff;
	       	*dev_id=0xffff;
	       	break;
	  }
}


static int _tp6x_validate_dev_id(int dev_id)
{
	if(   dev_id==TP62_DEV_ID
       || dev_id==TP67_DEV_ID
       || dev_id==TP6202_DEV_ID
       || dev_id==TP6203_DEV_ID
       || dev_id==TP6414_DEV_ID
       || dev_id==TP6415_DEV_ID
       || dev_id==TP6416_DEV_ID
      )
		return(TP6X_OK);
    else
		return(TP6X_PARAM_ERR);
}


int TP6X_open()
     // opens PCI BIOS, find and setup all TP6X boards
{
	int r,i,j,i1,j1,n;
    unsigned vendor_id, dev_id;
    _TP6X_BOARD_DD	*_BD,*_BD1;
    BYTE b;

	// check for TP6x API has been open already
 	if(_TP6X_OPEN) return(TP6X_OK);

	// clear all vars
	_tp6x_open_flag=_OFF;
	_tp6x_boards = 0;
	_tp6x_board_sel	= TP6X_INVALID;
	_TP6X_SD=NULL;
	_BD=NULL;

	//--- allocate memory for SYSTEM DESCRIPTOR
	r=TP6X_MEM_ALLOC_ERR;
	if((_TP6X_SD=(_TP6X_SYS_DD *)halloc(_TP6X_SD_LEN,1))==NULL)
		return(_tp6x_set_err(r));

  	//--- clear SD
	_BD=_TP6X_BD_PTR(0);
	for(i=0;i<TP6X_MAX_BOARDS;_BD++,i++)
	{ 
		_BD->id=TP6X_INVALID;
	    _BD->dev_id=TP6X_INVALID;
	    _BD->index=TP6X_INVALID;
	    _BD->rev=TP6X_INVALID;
	    _BD->fw_rev=TP6X_INVALID;
	    _BD->bus=TP6X_INVALID;
	    _BD->df=TP6X_INVALID;
	    _BD->irq=TP6X_INVALID;
	    _BD->baddr[0]=0L;
	    _BD->baddr[1]=0L;
	    _BD->baddr[2]=0L;
	    _BD->baddr[3]=0L;
	    _BD->baddr[4]=0L;
	    _BD->dpram_len=0L;
	    _BD->sram_len=0L;
	    _BD->dram_len=0L;
	}

	//--- check for PCI BIOS
	if((r=_PCI_open(NULL))!=_PCI_NO_ERR) return(_tp6x_set_err(r));

	//--- check for all TP6X boards installed
	//    Note: total number of detected boards should be <TP6X_MAX_BOARDS
	_BD=_TP6X_BD_PTR(0);
    for(_tp6x_boards=0,n=0;n<TP6X_BOARD_TYPES; n++)
	{
    	_tp6x_board_list(n,&vendor_id,&dev_id);
        for(i=0;i<TP6X_MAX_BOARDS;i++)	// find TP6x boards
	    {
        	if(_PCI_find_device(vendor_id,dev_id,i,&(_BD->bus),&(_BD->df))==_PCI_NO_ERR)
			{	
				//-- board found, register it
		  		_BD->id=_tp6x_boards;
		  		_BD->dev_id=dev_id;
		  		_BD->index=i;
                _PCI_read_cnf_byte(_BD->bus,_BD->df,_PCI_REV_ID_CNF_RG,&(_BD->rev));
                _PCI_read_cnf_dword(_BD->bus,_BD->df,_PCI_BADDR0_CNF_RG,&(_BD->baddr[0]));
                _PCI_read_cnf_dword(_BD->bus,_BD->df,_PCI_BADDR1_CNF_RG,&(_BD->baddr[1]));
                _PCI_read_cnf_dword(_BD->bus,_BD->df,_PCI_BADDR2_CNF_RG,&(_BD->baddr[2]));
                _PCI_read_cnf_dword(_BD->bus,_BD->df,_PCI_BADDR3_CNF_RG,&(_BD->baddr[3]));
                _PCI_read_cnf_dword(_BD->bus,_BD->df,_PCI_BADDR4_CNF_RG,&(_BD->baddr[4]));
                _PCI_read_cnf_byte(_BD->bus,_BD->df,_PCI_INT_LINE_CNF_RG,&(_BD->irq));

				_tp6x_boards++;
		  		_BD++;
			  	//--- if already MAX boards is detected, then exit
			  	//    and skip other undetected TP6X boards
		  		if(_tp6x_boards>=TP6X_MAX_BOARDS) goto end_of_reg_loop;
			}
			else
			{ 
				//--- nomore this type boards found.
				//    Cancel this loop and proceed with next board type.
				break;  
			}
		}
	}
	
end_of_reg_loop: 
		;

	//-- all TP6X boards is detected
	//   now perform PCI assigned params validation

	_BD=_TP6X_BD_PTR(0);
	for(i=0;i<_tp6x_boards;i++, _BD++)	// src board sel loop
	{ 
		for(j=0;j<=TP6X_BADDR_AREAS;j++)	// src BADDR# sel loop
	    {
	    	//--- validate TP6X specific PCI config params
	      	//    1. check for area is allocated as MEM-32BIT or MEM-RSRV
	      	//    2. check for DPM area is allocated to <1MB
	      	//    3. ensure that HIF area is allocated
	      	// ?? 4. check for reserved area is not allocated

			if( 	TP6X_BADDR_AREA_IS_MEM_RSRV (_BD->baddr[j])
		   		||  TP6X_BADDR_AREA_IS_MEM_32BIT(_BD->baddr[j])   
                ||  (		j==TP6X_DPM_AREA
                    	&& 	(     	TP6X_BADDR_AREA_IS_INVALID(_BD->baddr[j])
	      	      	      		|| 	!TP6X_BADDR_AREA_IS_MEM_1M(_BD->baddr[j])
			    			)
		        	)
		   		||	(		j==TP6X_HIF_AREA
                    	&& 	(    TP6X_BADDR_AREA_IS_INVALID(_BD->baddr[j])
			    			)
					)
/*		   		||  (    	j==TP6X_RSRV_AREA
                     	&&  TP6X_BADDR_AREA_IS_VALID(_BD->baddr[j])
					)
*/
			  )
			{
            	r=TP6X_INCOMPAT_PCI_AREA_INFO_ERR;
	      	  	goto exit_err;
			}


			//-- validate baddr values for all boards
		  	//   (check for unique BADDR[i] value for all boards ;
	  	  	//    == may appear in case not-enough UMB area)

			if(TP6X_BADDR_AREA_IS_VALID(_BD->baddr[j]))
			{
		  		_BD1=_TP6X_BD_PTR(0);
                for(i1=0;i1<_tp6x_boards;i1++, _BD1++) // dest board sel
		   		{ 
					for(j1=0;j1<=TP6X_BADDR_AREAS;j1++) // dest BADDR# sel
		       		{ 
						if(  	TP6X_BADDR_AREA_IS_VALID(_BD1->baddr[j1])
		       	    		&& 	j1!=j 
							&& 	i1!=i
			    			&& 	_BD1->baddr[j1]==_BD->baddr[j]
			   			  )
			   			{ 
							//-- error: matched PCI assigned addresses
			     			//          clear vars and dealloc memory
							r=TP6X_INV_PCI_AREA_INFO_ERR;
							goto exit_err;
			   			}
		       		}
		   		}
		 	}

		}
	}	//--- no validation errors, proceed with next board

	  
	if(_tp6x_boards>0)	// now check the result and init API
	{
    	_tp6x_open_flag=_ON;

		//-- now set DPRAM/SRAM/SDRAM len for each board
	   	_BD=_TP6X_BD_PTR(0);
  	   	for(_tp6x_board_sel=0;_tp6x_board_sel<_tp6x_boards;_BD++,_tp6x_board_sel++)
	    {
			//-- set DPRAM configuration
            if((r=TP6X_nvram_read(TP6X_NVRAM_DPRAM_LEN_ID_ADDR,&b))!=TP6X_OK)
		   		goto exit_err;

			switch((unsigned)b)
			{
		  		case TP6X_NVRAM_DPRAM_LEN_32K_ID:
		       		if(		_BD->dev_id==TP62_DEV_ID 
						|| 	_BD->dev_id==TP67_DEV_ID 
						|| 	_BD->dev_id==TP6202_DEV_ID 
						|| 	_BD->dev_id==TP6203_DEV_ID
					  ) // this option is valid for TP62xx/TP67 boards only
			 		{ 
			       		_BD->dpram_len=0x20000L;
		          		break;
			 		}
			 		else			// return error for TP64xx boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}

		  		case TP6X_NVRAM_DPRAM_LEN_64K_ID:
		       		if(		_BD->dev_id==TP62_DEV_ID 
						|| 	_BD->dev_id==TP67_DEV_ID 
						|| 	_BD->dev_id==TP6202_DEV_ID 
						|| 	_BD->dev_id==TP6203_DEV_ID
					  ) // this option is valid for TP62xx/TP67 boards only
			 		{ 
		       			_BD->dpram_len=0x40000L;
		          		break;
			 		}
			 		else			// return error for TP64xx boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		case TP6X_NVRAM_DPRAM_LEN_128K_ID:
		       		if(		_BD->dev_id==TP6414_DEV_ID 
						|| 	_BD->dev_id==TP6415_DEV_ID 
						|| 	_BD->dev_id==TP6416_DEV_ID 
					  ) // this option is valid for TP64xx boards only
			 		{
		       			_BD->dpram_len=0x80000L;
		          		break;
			 		}
			 		else			// return error for TP62xx/TP67 boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		case TP6X_NVRAM_DPRAM_LEN_256K_ID:
		       		if(		_BD->dev_id==TP6414_DEV_ID
						|| 	_BD->dev_id==TP6415_DEV_ID
						|| 	_BD->dev_id==TP6416_DEV_ID
					  ) // this option is valid for TP64xx boards only
			 		{
		       			_BD->dpram_len=0x100000L;
		          		break;
			 		}
			 		else			// return error for TP62xx/TP67 boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		case TP6X_NVRAM_DPRAM_LEN_512K_ID:
		       		if(		_BD->dev_id==TP6414_DEV_ID
						|| 	_BD->dev_id==TP6415_DEV_ID
						|| 	_BD->dev_id==TP6416_DEV_ID
					  ) // this option is valid for TP64xx boards only
			 		{
		       			_BD->dpram_len=0x200000L;
		          		break;
			 		}
			 		else			// return error for TP62xx/TP67 boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		default:
		       		r=TP6X_INV_BOARD_INFO_ERR;
		       		goto exit_err;
			}

			//-- set SRAM configuration
            if((r=TP6X_nvram_read(TP6X_NVRAM_SRAM_LEN_ID_ADDR,&b))!=TP6X_OK)
		   		goto exit_err;

	       	switch((unsigned)b)
			{
		  		case TP6X_NVRAM_SRAM_LEN_0K_ID:
		       		_BD->sram_len=0x0L;
		       		break;

		  		case TP6X_NVRAM_SRAM_LEN_16K_ID:
		       		if(		_BD->dev_id==TP62_DEV_ID
						|| 	_BD->dev_id==TP67_DEV_ID
						|| 	_BD->dev_id==TP6202_DEV_ID
						|| 	_BD->dev_id==TP6203_DEV_ID
					  ) // this option is valid for TP62xx/TP67 boards only
			 		{
		       			_BD->sram_len=0x10000L;
		          		break;
			 		}
			 		else			// return error for TP64xx boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}

		  		case TP6X_NVRAM_SRAM_LEN_32K_ID:
		       		if(		_BD->dev_id==TP62_DEV_ID 
						|| 	_BD->dev_id==TP67_DEV_ID
						|| 	_BD->dev_id==TP6202_DEV_ID 
						|| 	_BD->dev_id==TP6203_DEV_ID
					  ) // this option is valid for TP62xx/TP67 boards only
			 		{ 
		       			_BD->sram_len=0x20000L;
		          		break;
			 		}
			 		else			// return error for TP64xx boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}

		  		case TP6X_NVRAM_SRAM_LEN_64K_ID:
		       		if(		_BD->dev_id==TP62_DEV_ID 
						|| 	_BD->dev_id==TP67_DEV_ID 
						|| 	_BD->dev_id==TP6202_DEV_ID 
						|| 	_BD->dev_id==TP6203_DEV_ID
					  ) // this option is valid for TP62xx/TP67 boards only
			 		{ 
		       			_BD->sram_len=0x40000L;
		          		break;
			 		}
			 		else			// return error for TP64xx boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}

		  		case TP6X_NVRAM_SRAM_LEN_128K_ID:
		       		if(		_BD->dev_id==TP62_DEV_ID
						|| 	_BD->dev_id==TP67_DEV_ID
						|| 	_BD->dev_id==TP6202_DEV_ID 
						|| 	_BD->dev_id==TP6203_DEV_ID
					  ) // this option is valid for TP62xx/TP67 boards only
			 		{ 
		       			_BD->sram_len=0x80000L;
		          		break;
			 		}
			 		else			// return error for TP64xx boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		case TP6X_NVRAM_SRAM_LEN_256K_ID:
		       		if(		_BD->dev_id==TP62_DEV_ID 
						|| 	_BD->dev_id==TP67_DEV_ID
						|| 	_BD->dev_id==TP6202_DEV_ID 
						|| 	_BD->dev_id==TP6203_DEV_ID
					  ) // this option is valid for TP62xx/TP67 boards only
			 		{ 
		       			_BD->sram_len=0x100000L;
		          		break;
			 		}
			 		else			// return error for TP64xx boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		case TP6X_NVRAM_SRAM_LEN_512K_ID:
		       		if(		_BD->dev_id==TP62_DEV_ID 
						|| 	_BD->dev_id==TP67_DEV_ID 
						|| 	_BD->dev_id==TP6202_DEV_ID 
						|| 	_BD->dev_id==TP6203_DEV_ID
					  ) // this option is valid for TP62xx/TP67 boards only
			 		{ 
		       			_BD->sram_len=0x200000L;
		          		break;
			 		}
			 		else			// return error for TP64xx boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		case TP6X_NVRAM_SRAM_LEN_1M_ID:
		       		if(		_BD->dev_id==TP62_DEV_ID 
						|| 	_BD->dev_id==TP67_DEV_ID 
						|| 	_BD->dev_id==TP6202_DEV_ID 
						|| 	_BD->dev_id==TP6203_DEV_ID
					  ) // this option is valid for TP62xx/TP67 boards only
			 		{
		       			_BD->sram_len=0x400000L;
		          		break;
			 		}
			 		else			// return error for TP64xx boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		case TP6X_NVRAM_SRAM_LEN_128KX64_ID:
		       		if(		_BD->dev_id==TP6414_DEV_ID 
						|| 	_BD->dev_id==TP6415_DEV_ID 
						|| 	_BD->dev_id==TP6416_DEV_ID 
					  ) // this option is valid for TP64xx boards only
			 		{ 
		       			_BD->sram_len=0x100000L;
		          		break;
			 		}
			 		else			// return error for TP62xx/TP67 boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		case TP6X_NVRAM_SRAM_LEN_256KX64_ID:
		       		if(		_BD->dev_id==TP6414_DEV_ID 
						|| 	_BD->dev_id==TP6415_DEV_ID 
						|| 	_BD->dev_id==TP6416_DEV_ID 
					  ) // this option is valid for TP64xx boards only
			 		{ 
		       			_BD->sram_len=0x200000L;
		          		break;
			 		}
			 		else			// return error for TP62xx/TP67 boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		case TP6X_NVRAM_SRAM_LEN_512KX64_ID:
		       		if(		_BD->dev_id==TP6414_DEV_ID 
						|| 	_BD->dev_id==TP6415_DEV_ID 
						|| 	_BD->dev_id==TP6416_DEV_ID 
					  ) // this option is valid for TP64xx boards only
			 		{ 
		       			_BD->sram_len=0x400000L;
		          		break;
			 		}
			 		else			// return error for TP62xx/TP67 boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		default:
		       		r=TP6X_INV_BOARD_INFO_ERR;
		       		goto exit_err;
			}

			//-- set SDRAM configuration
            if((r=TP6X_nvram_read(TP6X_NVRAM_SDRAM_LEN_ID_ADDR,&b))!=TP6X_OK)
		   		goto exit_err;

	       	switch((unsigned)b)
			{
		  		case TP6X_NVRAM_SDRAM_LEN_0M_ID:
		       		_BD->dram_len=0x0L;
		       		break;

		  		case TP6X_NVRAM_SDRAM_LEN_1M_ID:
		       		if(		_BD->dev_id==TP62_DEV_ID 
						|| 	_BD->dev_id==TP67_DEV_ID 
						|| 	_BD->dev_id==TP6202_DEV_ID 
						|| 	_BD->dev_id==TP6203_DEV_ID
					  ) // this option is valid for TP62xx/TP67 boards only
			 		{ 
		       			_BD->dram_len=0x400000L;
		          		break;
			 		}
			 		else			// return error for TP64xx boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		case TP6X_NVRAM_SDRAM_LEN_4M_ID:
		       		if(		_BD->dev_id==TP62_DEV_ID 
						|| 	_BD->dev_id==TP67_DEV_ID 
						|| 	_BD->dev_id==TP6202_DEV_ID 
						|| 	_BD->dev_id==TP6203_DEV_ID
					  ) // this option is valid for TP62xx/TP67 boards only
			 		{ 
		       			_BD->dram_len=0x1000000L;
		          		break;
			 		}
			 		else			// return error for TP64xx boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		case TP6X_NVRAM_SDRAM_LEN_8M_ID:
		       		if(		_BD->dev_id==TP6202_DEV_ID
						|| 	_BD->dev_id==TP6203_DEV_ID
					  ) // this option is valid for TP6202/TP6203 boards only
			 		{ 
						_BD->dram_len=0x2000000L;
		          		break;
			 		}
			 		else			// return error for TP62/TP67 boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		case TP6X_NVRAM_SDRAM_LEN_4MX64_ID:
		       		if(		_BD->dev_id==TP6414_DEV_ID 
						|| 	_BD->dev_id==TP6415_DEV_ID 
						|| 	_BD->dev_id==TP6416_DEV_ID 
					  ) // this option is valid for TP64xx boards only
			 		{ 
		       			_BD->dram_len=0x2000000L;
		          		break;
			 		}
			 		else			// return error for TP62xx/TP67 boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		case TP6X_NVRAM_SDRAM_LEN_16MX64_ID:
		       		if(		_BD->dev_id==TP6414_DEV_ID
						|| 	_BD->dev_id==TP6415_DEV_ID 
						|| 	_BD->dev_id==TP6416_DEV_ID 
					  ) // this option is valid for TP64xx boards only
			 		{ 
		       			_BD->dram_len=0x8000000L;
		          		break;
			 		}
			 		else			// return error for TP62xx/TP67 boards
		       	 	{
						r=TP6X_INV_BOARD_INFO_ERR;
		       			goto exit_err;
			 		}


		  		default:
		       		r=TP6X_INV_BOARD_INFO_ERR;
		       		goto exit_err;
			}

			//-- read and validate f/w revision code
            if((r=TP6X_nvram_read(TP6X_NVRAM_FW_REV_ADDR,&b))!=TP6X_OK)
		   		goto exit_err;

			// validate f/w rev value
	       	if(   	b==0
		  		|| 	(    	!(_BD->dev_id==TP62_DEV_ID || _BD->dev_id==TP67_DEV_ID)
		       			&& 	b==0xff
		     		)
		  		|| 	(b>=0x80 && b<0xff)
		  		|| 	(    	(_BD->dev_id==TP62_DEV_ID || _BD->dev_id==TP67_DEV_ID)
		       			&& 	b==1
		      		)
		 	  )
		    {
				r=TP6X_INV_BOARD_INFO_ERR;
		       	goto exit_err;
		    }

			// for TP62/TP67 board convert 0xFF to rev.1 code
	       	if(   	(_BD->dev_id==TP62_DEV_ID || _BD->dev_id==TP67_DEV_ID)
		  		&& 	b==0xff
		 	  )
		    	b=0x01;

			// save f/w rev value
	       	_BD->fw_rev=b;

		}

		_tp6x_board_sel=0;   // set default board as #0
	   	r=TP6X_OK;		// exit with OK

	}
	else
	{
		r=TP6X_NO_BOARDS_FOUND_ERR;	// set error if no boards found

exit_err:
		// EXIT ON ERROR fm other prg points
		_tp6x_open_flag=_OFF;
        _tp6x_boards = 0;
	   	_tp6x_board_sel= TP6X_INVALID;
        hfree(_TP6X_SD);
	   	_TP6X_SD=NULL;
	}

	return(_tp6x_set_err(r));	// exit
}


int TP6X_close()
     // close T6X API
{
	// check for API is not open or has been already closed
	if(!_TP6X_OPEN) return(TP6X_OK);

	_tp6x_open_flag=_OFF;	// clear all vars
	_tp6x_boards = 0;
	_tp6x_board_sel	= TP6X_INVALID;
	hfree(_TP6X_SD);		// deallocate memory for SD
	_TP6X_SD=NULL;
    return(_tp6x_set_err(TP6X_OK));
}


int TP6X_boards()
     // return number of detected TP6X boards
{
	if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

	_tp6x_set_err(TP6X_OK);
    return(_tp6x_boards);
}

int TP6X_board_id(unsigned dev_id, unsigned index)
     // return BOARD_ID for board with DEV_ID & INDEX
{ 	int i;
    _TP6X_BOARD_DD *_BD;


 	if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

	//--- validate params
    if(   _tp6x_validate_dev_id(dev_id)!=TP6X_OK
	   || index>=_tp6x_boards
	   || index>=TP6X_MAX_BOARDS
	  )
		return(_tp6x_set_err(TP6X_PARAM_ERR));

	//--- scan SD for matching DEV_ID and INDEX
    _BD=_TP6X_BD_PTR(0);
	for(i=0;i<TP6X_MAX_BOARDS;i++,_BD++)
	{
		if(    _BD->id >=0
            && _BD->dev_id ==dev_id
	       	&& _BD->index ==index
	      )
	    {		//-- scan match, exit OK
	    	_tp6x_set_err(TP6X_OK);
        	return(i);
	    }
	}

	//-- board not found, exit with error
    return(_tp6x_set_err(TP6X_BOARD_NOT_FOUND_ERR));
}

int TP6X_sel_board(unsigned dev_id, unsigned index)
     // select current board using DEV_ID/INDEX
{
	int i;

	if((i=TP6X_board_id(dev_id,index))<0)
		return(_tp6x_set_err(i));

	_tp6x_board_sel=i;
    return(_tp6x_set_err(TP6X_OK));
}

int TP6X_sel_board_id(int id)
     // select current board using BOARD_ID
     // returns CURRENT BOARD ID
     // in case ID==TP6X_CURRENT_BOARD, then ID of current board is returned
{
	_TP6X_BOARD_DD *_BD;

 	if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

	//--- validate params
    if(   	id!=TP6X_CURRENT_BOARD
	   	&& 	(   	id<0
	       		|| 	id>=_tp6x_boards
	       		|| 	id>=TP6X_MAX_BOARDS
	      	)
	  )
		return(_tp6x_set_err(TP6X_PARAM_ERR));

	if(id!=TP6X_CURRENT_BOARD) _tp6x_board_sel=id;

	_tp6x_set_err(TP6X_OK);
	return(_tp6x_board_sel);
}


int TP6X_board_info(int id, unsigned *dev_id, unsigned *index, BYTE *rev,
		    		BYTE *irq,
		    		unsigned long *baddr0,unsigned long *baddr1,
		    		unsigned long *baddr2,unsigned long *baddr3,
		    		unsigned long *baddr4
		   		   )
     // returns main board info using BOARD_ID
     // returns ID or requested board
     // in case ID==TP6X_CURRENT_BOARD, then current board info is returned
{
	if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

	//--- validate params
    if(   		id!=TP6X_CURRENT_BOARD
	   	&& 	(   	id<0
	       		|| 	id>=_tp6x_boards
	       		|| 	id>=TP6X_MAX_BOARDS
	      	)
	  )
		return(_tp6x_set_err(TP6X_PARAM_ERR));

	if(id==TP6X_CURRENT_BOARD) id=_tp6x_board_sel;

	if(dev_id!=NULL) *dev_id=_TP6X_SD->BD[id].dev_id;
	if(index !=NULL) *index =_TP6X_SD->BD[id].index;
	if(rev   !=NULL) *rev   =_TP6X_SD->BD[id].rev;
	if(irq   !=NULL) *irq   =_TP6X_SD->BD[id].irq;
	if(baddr0!=NULL) *baddr0=_TP6X_SD->BD[id].baddr[0];
	if(baddr1!=NULL) *baddr1=_TP6X_SD->BD[id].baddr[1];
	if(baddr2!=NULL) *baddr2=_TP6X_SD->BD[id].baddr[2];
	if(baddr3!=NULL) *baddr3=_TP6X_SD->BD[id].baddr[3];
	if(baddr4!=NULL) *baddr4=_TP6X_SD->BD[id].baddr[4];

	_tp6x_set_err(TP6X_OK);
	return(id);	// returns ID of requested board
}


int TP6X_dev_name(char *s)
	// returns device name for the current board
{
	_TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    if(s!=NULL)
	{ 
		switch (_BD->dev_id)
		{
			case TP62_DEV_ID:
		  		strcpy(s,"TORNADO-P62");
				break;

		  	case TP67_DEV_ID:
				strcpy(s,"TORNADO-P67");
				break;

		  	case TP6202_DEV_ID:
		  		strcpy(s,"TORNADO-P6202");
				break;

		  	case TP6203_DEV_ID:
		  		strcpy(s,"TORNADO-P6203");
				break;

		  	case TP6414_DEV_ID:
		  		strcpy(s,"TORNADO-P6414");
				break;

		  	case TP6415_DEV_ID:
		  		strcpy(s,"TORNADO-P6415");
				break;

		  	case TP6416_DEV_ID:
		  		strcpy(s,"TORNADO-P6416");
				break;

		  	default:
				strcpy(s,"TORNADO-P6X");
				break;
		}
	}
    return(_tp6x_set_err(TP6X_OK));
}


unsigned TP6X_dev_id()
     // returns DEV ID for current board
     // In case error occurs, then ERR_FLAG is set and returned value is TP6X_INV_DEV_ID=0xFFFF.
{
	_TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) 
	{ 
		_tp6x_set_err(TP6X_NOT_OPEN_ERR);
	  	return(TP6X_INV_DEV_ID);
	}

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    _tp6x_set_err(TP6X_OK);
    return(_BD->dev_id);    
}


int TP6X_dev_fw_rev()
     // returns DEV F/W-REVISION for current board
     // In case error occurs, then ERR_FLAG is set and returned value is <0.
{
    _TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    _tp6x_set_err(TP6X_OK);
    return(((unsigned)_BD->fw_rev)&0xff);    
}



int TP6X_area_alloc_name(unsigned area,char *s)
	// returns name for PCI pass-thru area type for the current board
{
	_TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));
    if( area>=TP6X_BADDR_AREAS) return(_tp6x_set_err(TP6X_PARAM_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    if(s!=NULL)
	{ 
		if(TP6X_BADDR_AREA_IS_INVALID(_BD->baddr[area])) 	
		  	strcpy(s,"not allocated");

		if(TP6X_BADDR_AREA_IS_MEM_1M(_BD->baddr[area]))
			strcpy(s,"memory <1MB");

        if(TP6X_BADDR_AREA_IS_MEM_32BIT(_BD->baddr[area])) 	
			strcpy(s,"memory 4GB");

		if(TP6X_BADDR_AREA_IS_IO(_BD->baddr[area])) 	
			strcpy(s,"I/O");

		if(TP6X_BADDR_AREA_IS_MEM_RSRV(_BD->baddr[area])) 	
			strcpy(s,"invalid");
	}
    return(_tp6x_set_err(TP6X_OK));
}


long TP6X_dpram_len()
	// returns amount of installed DPRAM
{
    _TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    _tp6x_set_err(TP6X_OK);
    return(_BD->dpram_len);
}


long TP6X_sram_len()
	// returns amount of installed SRAM (SBSRAM)
{
    _TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    _tp6x_set_err(TP6X_OK);
    return(_BD->sram_len);
}

long TP6X_dram_len()
	// returns amount of installed SDRAM
{
    _TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    _tp6x_set_err(TP6X_OK);
    return(_BD->dram_len);
}


unsigned long TP6X_dpram_area_dsp_baddr()
	// TP62xx/TP67 boards:
	//		returns DSP base address of DPRAM area
	//
	// TP64XX boards:
	//		returns DSP base address of DPRAM 8Kx32 sub-area, which is accessed by PCI-bus
	//
	// This function is actually useful for TP64xx boards only and can be used by host 
	// PC s/w to pass base address of communication DPRAM sub-area to DSP s/w (via HPI).
{
    _TP6X_BOARD_DD *_BD;
	unsigned long addr;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
   	_tp6x_set_err(TP6X_OK);

	if(		_BD->dev_id==TP6414_DEV_ID 
		|| 	_BD->dev_id==TP6415_DEV_ID 
		|| 	_BD->dev_id==TP6416_DEV_ID 
	  ) 
    	return((_BD->baddr[3] & ~0x3) | TP64XX_DSP_DPRAM_SADDR);
	else			
		return(TP6X_DSP_DPRAM_SADDR);
}


//----------------------------------------------------------------


//-------------- nvRAM Access Control ----------------------------
// Utility functions for Write/Read of bytes to/fm AMCC
// serial nvRAM (EPROM).



#define _tp6x_wait_pcic_nvram_ready() {while((TP6X_read_byte(TP6X_PCIC_AREA,_AMCC_MCSR_NVCMD_RG) & 0x80)!=0);}

int	TP6X_nvram_write(unsigned addr,BYTE b)
{ long l,l1;

	if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));
    if(addr>=_AMCC_NVRAM_LEN_MAX) return(_tp6x_set_err(TP6X_PARAM_ERR));

	//--- wait until AMCC controller is ready
	_tp6x_wait_pcic_nvram_ready();

	//--- load low-byte of address
    TP6X_write_byte(TP6X_PCIC_AREA,_AMCC_MCSR_NVCMD_RG,0x80);
    TP6X_write_byte(TP6X_PCIC_AREA,_AMCC_MCSR_NVDATAP_RG,(addr&0xff));

	//--- load high-byte of address
    TP6X_write_byte(TP6X_PCIC_AREA,_AMCC_MCSR_NVCMD_RG,0xa0);
    TP6X_write_byte(TP6X_PCIC_AREA,_AMCC_MCSR_NVDATAP_RG,((addr>>8)&0xff));
	_tp6x_wait_pcic_nvram_ready();

	//--- load data and send write command and wait for ready
    TP6X_write_byte(TP6X_PCIC_AREA,_AMCC_MCSR_NVDATAP_RG,b);
    TP6X_write_byte(TP6X_PCIC_AREA,_AMCC_MCSR_NVCMD_RG,0xc0);
	_tp6x_wait_pcic_nvram_ready();
	
	// form delay about 20ms (required)
	_bios_timeofday(0,&l);
	do
	{
		_bios_timeofday(0,&l1);
	} 
		while(l1<(l+1));

    return(_tp6x_set_err(TP6X_OK));
}



int	TP6X_nvram_read(unsigned addr,BYTE *b)
{
    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));
    if(addr>=_AMCC_NVRAM_LEN_MAX) return(_tp6x_set_err(TP6X_PARAM_ERR));

    if(b!=NULL)
	{
    	//--- wait until AMCC controller is ready
	  	_tp6x_wait_pcic_nvram_ready();
		
		//--- load low-byte of address
        TP6X_write_byte(TP6X_PCIC_AREA,_AMCC_MCSR_NVCMD_RG,0x80);
        TP6X_write_byte(TP6X_PCIC_AREA,_AMCC_MCSR_NVDATAP_RG,(addr&0xff));

		//--- load high-byte of address
		TP6X_write_byte(TP6X_PCIC_AREA,_AMCC_MCSR_NVCMD_RG,0xa0);
        TP6X_write_byte(TP6X_PCIC_AREA,_AMCC_MCSR_NVDATAP_RG,((addr>>8)&0xff));

		//--- send read command and wait for ready
		TP6X_write_byte(TP6X_PCIC_AREA,_AMCC_MCSR_NVCMD_RG,0xe0);
	  	_tp6x_wait_pcic_nvram_ready();

		//--- read nvRAM byte and return
    	*b=TP6X_read_byte(TP6X_PCIC_AREA,_AMCC_MCSR_NVDATAP_RG);
	}

    return(_tp6x_set_err(TP6X_OK));
}


//=============== Data access functions ==========================
/*
	Data read/write functions provide AREA INVARIANT (independent)
	access to data in the pass-through areas for the current selected
	board.

	Supplied functions provide BYTE/WORD/DWORD data formats.
*/

static int _tp6x_validate_baddr_area(int area)
	// local function for validation of BADDR PT area
{
    _TP6X_BOARD_DD *_BD;

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);

	//--- validate params
    if( area<0 || area>=TP6X_BADDR_AREAS)
		return(TP6X_PARAM_ERR);


	if(TP6X_BADDR_AREA_IS_INVALID(_BD->baddr[area]))
		return(TP6X_INV_BADDR_AREA_ERR);

	//-- decode area type
	if(TP6X_BADDR_AREA_IS_MEM(_BD->baddr[area]))
	{
		//--- check for 32-bit memory allocation type
		//    return error in this case
	    if(TP6X_BADDR_AREA_IS_MEM_32BIT(_BD->baddr[area]))	
			return(TP6X_ACCESS_BADDR_AREA_ERR);

		//--- check for reserved memory types
		if(TP6X_BADDR_AREA_IS_MEM_RSRV(_BD->baddr[area]))
			return(TP6X_ACCESS_INV_BADDR_MEM_ERR);
	}
	
	return(TP6X_OK);
}



BYTE TP6X_read_byte(int area, unsigned long addr)
     // returns BYTE value fm #AREA at ADDR address
     // error is returned via TP6X_LAST_ERR
{ 
	BYTE d, *mem;
    int r;
    unsigned io_baddr;
    _TP6X_BOARD_DD *_BD;

	d=0;
 	if(r=TP6X_NOT_OPEN_ERR, !_TP6X_OPEN) goto exit;
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);

 	if((r=_tp6x_validate_baddr_area(area))!=TP6X_OK) goto exit;

	if(TP6X_BADDR_AREA_IS_MEM(_BD->baddr[area]))
	{
		//--- read data byte fm memory
		mem=TP6X_BADDR_MEM_1MB_PTR(_BD->baddr[area]);
	    d=mem[addr];
	}
	else	
	{
		//--- read I/O port
	    io_baddr=TP6X_BADDR_IO_ADDR(_BD->baddr[area]);
    	d=inp(io_baddr+((unsigned)addr));
	}

	r=TP6X_OK;
			
exit:
	_tp6x_set_err(r);
    return(d);	// returns data byte

}

unsigned TP6X_read_word(int area, unsigned long addr)
     // returns W16 value fm #AREA at ADDR address
     // error is returned via TP6X_LAST_ERR
{ 
	unsigned d, *mem;
    int r;
    unsigned io_baddr;
    _TP6X_BOARD_DD *_BD;

	d=0;
 	if(r=TP6X_NOT_OPEN_ERR, !_TP6X_OPEN) goto exit;
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);

 	if((r=_tp6x_validate_baddr_area(area))!=TP6X_OK) goto exit;

	addr &=~1L;	//-- adjust to the W16 boundary
	if(TP6X_BADDR_AREA_IS_MEM(_BD->baddr[area]))
	{
		//--- read data byte fm memory
		mem=TP6X_BADDR_MEM_1MB_PTR(_BD->baddr[area]);
	    d=mem[addr/2];
    }
	else
	{
		//--- read I/O port
		io_baddr=TP6X_BADDR_IO_ADDR(_BD->baddr[area]);
        d=inpw(io_baddr+((unsigned)addr));
	}

	r=TP6X_OK;
			
exit:
	_tp6x_set_err(r);
    return(d);	// returns data
}


unsigned long TP6X_read_dword(int area, unsigned long addr)
     // returns W32 value fm #AREA at ADDR address
     // error is returned via TP6X_LAST_ERR
{ 
	unsigned long d, *mem;
    int r;
    unsigned io_baddr;
    _TP6X_BOARD_DD *_BD;

	d=0;
 	if(r=TP6X_NOT_OPEN_ERR, !_TP6X_OPEN) goto exit;
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);

 	if((r=_tp6x_validate_baddr_area(area))!=TP6X_OK) goto exit;

	addr &=~3L;	//-- adjust to the W32 boundary
	if(TP6X_BADDR_AREA_IS_MEM(_BD->baddr[area]))
	{
		//--- read data byte fm memory
		mem=TP6X_BADDR_MEM_1MB_PTR(_BD->baddr[area]);
	    d=mem[addr/4];
    }
	else
	{  
		//--- read I/O port
		io_baddr=TP6X_BADDR_IO_ADDR(_BD->baddr[area]);
		_inport32(io_baddr+((unsigned)addr), &d);
	}

	r=TP6X_OK;
			
exit:
	_tp6x_set_err(r);
    return(d);	// returns data
}



int TP6X_write_byte(int area, unsigned long addr,unsigned d)
     // writes BYTE value to #AREA at ADDR address
{ 
	BYTE *mem;
    int r;
    unsigned io_baddr;
    _TP6X_BOARD_DD *_BD;

 	if(r=TP6X_NOT_OPEN_ERR, !_TP6X_OPEN) goto exit;
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);

 	if((r=_tp6x_validate_baddr_area(area))!=TP6X_OK) goto exit;

	if(TP6X_BADDR_AREA_IS_MEM(_BD->baddr[area]))
	{
		//--- write to memory
		mem=TP6X_BADDR_MEM_1MB_PTR(_BD->baddr[area]);
	    mem[addr]=(BYTE) (d&0xff);
	}
	else	
	{
		//--- write to I/O port
		io_baddr=TP6X_BADDR_IO_ADDR(_BD->baddr[area]);
		outp(io_baddr+((unsigned)addr),(BYTE) (d&0xff));
	}

	r=TP6X_OK;
			
exit:
	return(_tp6x_set_err(r));
}


int TP6X_write_word(int area, unsigned long addr, unsigned d)
     // writes W16 value to #AREA at ADDR address
{ 
	unsigned *mem;
    int r;
    unsigned io_baddr;
    _TP6X_BOARD_DD *_BD;

 	if(r=TP6X_NOT_OPEN_ERR, !_TP6X_OPEN) goto exit;
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);

 	if((r=_tp6x_validate_baddr_area(area))!=TP6X_OK) goto exit;

	addr &=~1L;	//-- adjust to the W16 boundary
	if(TP6X_BADDR_AREA_IS_MEM(_BD->baddr[area]))
	{
		//--- write to memory
		mem=TP6X_BADDR_MEM_1MB_PTR(_BD->baddr[area]);
	    mem[addr/2]=d;
	}
	else	
	{
		//--- write to I/O port	
		io_baddr=TP6X_BADDR_IO_ADDR(_BD->baddr[area]);
        outpw(io_baddr+((unsigned)addr),d);
	}

	r=TP6X_OK;
			
exit:
	return(_tp6x_set_err(r));
}


int TP6X_write_dword(int area, unsigned long addr, unsigned long d)
     // writes W32 value to #AREA at ADDR address
{ 
	unsigned long *mem;
    int r;
    unsigned io_baddr;
    _TP6X_BOARD_DD *_BD;

 	if(r=TP6X_NOT_OPEN_ERR, !_TP6X_OPEN) goto exit;
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);

 	if((r=_tp6x_validate_baddr_area(area))!=TP6X_OK) goto exit;

	addr &=~3L;	//-- adjust to the W32 boundary
	if(TP6X_BADDR_AREA_IS_MEM(_BD->baddr[area]))
	{
		//--- write to memory
		mem=TP6X_BADDR_MEM_1MB_PTR(_BD->baddr[area]);
	    mem[addr/4]=d;
	}
	else	
	{
		//--- write to I/O port
		io_baddr=TP6X_BADDR_IO_ADDR(_BD->baddr[area]);
	    _outport32(io_baddr+((unsigned)addr), d);
	}

	r=TP6X_OK;
			
exit:
	return(_tp6x_set_err(r));
}


int TP6X_write_dword16(int area, unsigned long addr, unsigned long d)
     // writes W32 value to #AREA at ADDR address using two sequential 16-bit accesses
     // This is required to fix the bug of AMCC rev.E chip when accessing PCIC OMBX registers
     // in I/O mapped area and to fix the bug with old TP62/TP67 rev.2B when accessing HPI port
     // registers using 32-bit cycle.
{ 
	unsigned long d1,d2;
    int r;
    unsigned io_baddr,*mem ;
    _TP6X_BOARD_DD *_BD;

 	if(r=TP6X_NOT_OPEN_ERR, !_TP6X_OPEN) goto exit;
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);

 	if((r=_tp6x_validate_baddr_area(area))!=TP6X_OK) goto exit;

	addr &=~3L;	//-- adjust to the W32 boundary
	d1= d & 0xffffL;
	d2=(d>>16) & 0xffffL;
	if(TP6X_BADDR_AREA_IS_MEM(_BD->baddr[area]))
	{
		//--- write to memory
		mem=(unsigned*)TP6X_BADDR_MEM_1MB_PTR(_BD->baddr[area]);
	    mem[addr/2]=(unsigned)d1;
	    mem[(addr/2)+1]=(unsigned)d2;
	}
	else	
	{
		//--- write to I/O port	
		io_baddr=TP6X_BADDR_IO_ADDR(_BD->baddr[area]);
		outpw(io_baddr+((unsigned)addr),(unsigned)d1);
        outpw(io_baddr+((unsigned)(addr+2)),(unsigned)d2);
	}

	r=TP6X_OK;
			
exit:
	return(_tp6x_set_err(r));
}


unsigned long TP6X_read_dword16(int area, unsigned long addr)
     // retuens W32 value fm #AREA at ADDR address using two sequential 16-bit accesses.
     // This is required to fix the bug of AMCC rev.E chip when accessing PCIC OMBX registers
     // in I/O mapped area and to fix the bug with old TP62/TP67 rev.2B when accessing HPI port
     // registers using 32-bit cycle.
     // error is returned via TP6X_LAST_ERR
{ 
	unsigned long d, d1,d2;
    int r;
    unsigned io_baddr,*mem;
    _TP6X_BOARD_DD *_BD;

	d=0;
 	if(r=TP6X_NOT_OPEN_ERR, !_TP6X_OPEN) goto exit;
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);

 	if((r=_tp6x_validate_baddr_area(area))!=TP6X_OK) goto exit;

	addr &=~3L;	//-- adjust to the W32 boundary
	if(TP6X_BADDR_AREA_IS_MEM(_BD->baddr[area]))
	{
		//--- read data byte fm memory
	    mem=(unsigned*)TP6X_BADDR_MEM_1MB_PTR(_BD->baddr[area]);
	    d1=(unsigned long)(mem[addr/2]);
	    d2=(unsigned long)(mem[(addr/2)+1]);
	}
	else
	{  
		//--- read from I/O port	
		io_baddr=TP6X_BADDR_IO_ADDR(_BD->baddr[area]);
		d1=(unsigned long) inpw(io_baddr+((unsigned)addr));
        d2=(unsigned long) inpw(io_baddr+((unsigned)(addr+2)));
	}

	d=((d2<<16) &0xffff0000L) | (d1 &0xffffL);
	r=TP6X_OK;
			
exit:
	_tp6x_set_err(r);
    return(d);	// returns data
}


//================================================================

//==================== SMP-0/1 access ============================
// SMP access functions have limited functionality for TP64xx boards,
// since TP64xx boards do not provide SMP registers and provides
// host access to 8Kx32 sub-area of on-board DPRAM only.


int TP6X_smp_ptr(unsigned long addr,unsigned area,unsigned smp,
		 		 unsigned *page,void **p)
	// TP62xx/TP67: convert ADDR/AREA/SMP# to *PAGE SMP_RG value & PC RAM **P pointer.
    // TP64xx: convert ADDR to PC RAM **P pointer, ignore AREA, SMP, PAGE parameters.
	//		   For TP64xx, ADDR must be within 8Kx32 range (0..0x7FFF) only.
	 

{  
	int r;
    unsigned dpm_mode;
    _TP6X_BOARD_DD *_BD;


	if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);

	if(		_BD->dev_id==TP62_DEV_ID 
		|| 	_BD->dev_id==TP67_DEV_ID 
		|| 	_BD->dev_id==TP6202_DEV_ID 
		|| 	_BD->dev_id==TP6203_DEV_ID 
	  ) 
	{ 
		// check for TP62xx/TP67 boards
    	if(smp>(((dpm_mode=TP6X_SMP_MODE_DATA)==TP6X_SMP_MODE_SPA) ? 0:1))
			return(_tp6x_set_err(TP6X_PARAM_ERR));

		switch(area)
    	{
    		case TP6X_SMP_DPRAM_AREA:
	  			if(addr>=TP6X_DPM_DPRAM_LEN)
					return(_tp6x_set_err(TP6X_PARAM_ERR));
	  			break;

			case TP6X_SMP_DPSEM_AREA:
	  			if(addr>=TP6X_DPM_DPSEM_LEN )
					return(_tp6x_set_err(TP6X_PARAM_ERR));
	  			addr *=4L;	// set dword address for DPSEM#
	  			break;

        	default:
				return(_tp6x_set_err(TP6X_PARAM_ERR));
		}

		if(page!=NULL)
	    	*page=((unsigned)((long)(addr/((dpm_mode==TP6X_SMP_MODE_SPA)? TP6X_SMP_SPA_SMP_LEN:TP6X_SMP_DPA_SMP_LEN)))
    	           | area | (smp==0 ? dpm_mode:0)
	    	  	  )   ;
    	if(p!=NULL)
     		*p= __MK_FP__(_BD->baddr[TP6X_DPM_AREA] >>4,
            	          (unsigned)( (long)(addr%((dpm_mode==TP6X_SMP_MODE_SPA)? TP6X_SMP_SPA_SMP_LEN:TP6X_SMP_DPA_SMP_LEN))
                	                )
		       			  + ((smp==0)? TP6X_SMP_DPA_SMP0_OFFSET:TP6X_SMP_DPA_SMP1_OFFSET)
		     			 );
    	return(_tp6x_set_err(TP6X_OK));

	}
	else			
	{ 
		// check for TP64xx boards
		if(addr>=TP64XX_DPM_DPRAM_LEN_MAX)
			return(_tp6x_set_err(TP6X_PARAM_ERR));

    	if(p!=NULL)
     		*p= __MK_FP__(_BD->baddr[TP6X_DPM_AREA] >>4,(unsigned) addr);

    	return(_tp6x_set_err(TP6X_OK));
	}
}


int TP6X_set_smp_ptr(unsigned long addr,unsigned area,unsigned smp,void **p)
     // TP62xx/TP67: Set SMP RG to the specified DPM address.
     // 			 Set PC SMP pointer to the *P.
     // TP64xx: Set PC SMP pointer to the *P. Ignore AREA and SMP param.

{  
	unsigned page;
    int r;
    _TP6X_BOARD_DD *_BD;

	if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);

	if((r=TP6X_smp_ptr(addr,area,smp,&page,p))!=TP6X_OK)
		return(_tp6x_set_err(r));

	// set SMP register for TP62xx/TP67    
	if(		_BD->dev_id==TP62_DEV_ID 
		|| 	_BD->dev_id==TP67_DEV_ID 
		|| 	_BD->dev_id==TP6202_DEV_ID 
		|| 	_BD->dev_id==TP6203_DEV_ID 
	  ) 
	{ 
		if(smp==0) 
			TP6X_SET_SMP0_RG(page);
    	else	
			TP6X_SET_SMP1_RG(page);
	}

    return(_tp6x_set_err(TP6X_OK));
}


int TP6X_set_dpram(unsigned long addr,unsigned long d)
     // write DWORD to DPRAM area (used for occasional access to DPRAM)
{  
	unsigned page,smp_old;
    unsigned long *p;
    int r;
    _TP6X_BOARD_DD *_BD;

	if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);

	if((r=TP6X_smp_ptr(addr,TP6X_SMP_DPRAM_AREA,TP6X_SMP_SMP0_SEL,&page,&p))!=TP6X_OK)
		return(_tp6x_set_err(r));

	// set & restore SMP register for TP62xx/TP67    
	if(		_BD->dev_id==TP62_DEV_ID 
		|| 	_BD->dev_id==TP67_DEV_ID 
		|| 	_BD->dev_id==TP6202_DEV_ID 
		|| 	_BD->dev_id==TP6203_DEV_ID 
	  ) 
	{ 
		smp_old = TP6X_SMP0_RG_DATA;
    	TP6X_SET_SMP0_RG(page);
    	*p=d;
    	TP6X_SET_SMP0_RG(smp_old);
	}
	else
    	*p=d;

    return(_tp6x_set_err(TP6X_OK));
}
     

unsigned long TP6X_dpram_data(unsigned long addr)
     // read DWORD fm DPRAM area (used for occasional access to DPRAM)
{  
	unsigned page,smp_old;
    unsigned long *p,rd;
    int r;
    _TP6X_BOARD_DD *_BD;

	if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);


	if((r=TP6X_smp_ptr(addr,TP6X_SMP_DPRAM_AREA,TP6X_SMP_SMP0_SEL,&page,&p))!=TP6X_OK)
	{ 
		_tp6x_set_err(r);
        return(0);
	}

	// set & restore SMP register for TP62xx/TP67    
	if(		_BD->dev_id==TP62_DEV_ID 
		|| 	_BD->dev_id==TP67_DEV_ID 
		|| 	_BD->dev_id==TP6202_DEV_ID 
		|| 	_BD->dev_id==TP6203_DEV_ID 
	  ) 
	{ 
		smp_old = TP6X_SMP0_RG_DATA;
    	TP6X_SET_SMP0_RG(page);
    	_tp6x_set_err(TP6X_OK);
    	rd=*p;
    	TP6X_SET_SMP0_RG(smp_old);
	}
	else
    	rd=*p;

    return(rd);
}


int TP6X_set_dpsem(unsigned addr,int d)
    // write Data to DPSEM #A (used for occasional access to DPSEM)
	// This fucntion is valid for TP62xx/TP67 only.
{  
	unsigned page, smp_old;
    unsigned *p;
    int r;
    _TP6X_BOARD_DD *_BD;

	if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);


	// return error for TP64xx
	if(		_BD->dev_id==TP6414_DEV_ID 
		|| 	_BD->dev_id==TP6415_DEV_ID 
		|| 	_BD->dev_id==TP6416_DEV_ID 
	  ) 
		return(_tp6x_set_err(TP6X_INV_CMND_ERR));

	if(d<0 || d>1)  return(_tp6x_set_err(TP6X_PARAM_ERR));
    if((r=TP6X_smp_ptr(addr,TP6X_SMP_DPSEM_AREA,TP6X_SMP_SMP0_SEL,&page,&p))!=TP6X_OK)
		return(_tp6x_set_err(r));
    smp_old = TP6X_SMP0_RG_DATA;
    TP6X_SET_SMP0_RG(page);
    *p=d;
    TP6X_SET_SMP0_RG(smp_old);
    return(_tp6x_set_err(TP6X_OK));
}
     
int TP6X_dpsem_data(unsigned addr)
     // read DPSEM (used for occasional access to DPSEM)
	 // This function is valid for TP62xx/TP67 only.
{  
	unsigned page, smp_old;
    unsigned *p;
    int r;
    _TP6X_BOARD_DD *_BD;

	if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));
	_BD=_TP6X_BD_PTR(_tp6x_board_sel);


	// return error for TP64xx
	if(		_BD->dev_id==TP6414_DEV_ID 
		|| 	_BD->dev_id==TP6415_DEV_ID 
		|| 	_BD->dev_id==TP6416_DEV_ID 
	  ) 
		return(_tp6x_set_err(TP6X_INV_CMND_ERR));

	if((r=TP6X_smp_ptr(addr,TP6X_SMP_DPSEM_AREA,TP6X_SMP_SMP0_SEL,&page,&p))!=TP6X_OK)
		return(_tp6x_set_err(r));
    smp_old = TP6X_SMP0_RG_DATA;
    TP6X_SET_SMP0_RG(page);
    _tp6x_set_err(TP6X_OK);
    r= (*p) & TP6X_DPM_DPSEM_BITMASK;
    TP6X_SET_SMP0_RG(smp_old);
    return(r);
}



//========================================================================



//==================== Universal API HPI port access functions ===========
// These functions are universal for all TP6x boards and are provided for 
// compatibility with TP62/TP67 API v.1 macros.
// These functions recognize TP62/TP67 f/w revision code and automatically 
// splits 32-bit HPI access cycle for TP62/TP67 rev.1 to two 16-bit HPI 
// access cycles as a workaround for on-board bug.
// HPI access cycles for TP6202/TP6203 and TP64xx boards have 32-bit format.


unsigned long 	TP6X_hpia_data()
// returns HPIA register data. For TP6202/TP6203 boards returned is the address data only
// with the AINC/DSPINT data being masked out.
{  
	_TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    _tp6x_set_err(TP6X_OK);

    if(_BD->dev_id==TP6202_DEV_ID || _BD->dev_id==TP6203_DEV_ID) 
		// for TP6202/TP6203 boards, return address field of HPIA register
		return(TP6X_read_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPIA_RG) & ~0x3L);

	// for TP64xx boards return HPIA register	
   	if(		_BD->dev_id==TP6414_DEV_ID 
		|| 	_BD->dev_id==TP6415_DEV_ID
		|| 	_BD->dev_id==TP6416_DEV_ID
	  ) 
		return(TP6X_read_dword(TP6X_HIF_AREA,TP64XX_HIF_HPIA_RG));

	// finally, this is TP62/TP67 board.
	// recognize f/w rev.1 in order to use two 16-bit access cycles 
	// instead of one 32-bit access cycle for TP62/TP67 f/w rev.2
	if(_BD->fw_rev==1)
   		return(TP6X_read_dword16(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPIA_RG));
	else
   		return(TP6X_read_dword(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPIA_RG));
}


int TP6X_set_hpia(unsigned long d)
// Sets HPIA register data. For TP6202/TP6203 boards set the address field only
// with the AINC/DSPINT data being left unchanged. 
{  
	_TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);


	//	For TP6202/TP6203 boards set the address field only	with the 
	//  AINC/DSPINT bits left unchanged.	
    if(_BD->dev_id==TP6202_DEV_ID || _BD->dev_id==TP6203_DEV_ID) 
		TP6X_write_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPIA_RG,
	 					 (TP6X_read_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPIA_RG) & 0x2L)  | (d&~0x3L)
						);
	else
	{
		// for TP64xx boards just and set HPIA register	
   		if(		_BD->dev_id==TP6414_DEV_ID 
			|| 	_BD->dev_id==TP6415_DEV_ID
			|| 	_BD->dev_id==TP6416_DEV_ID
	  	  ) 
			TP6X_write_dword(TP6X_HIF_AREA,TP64XX_HIF_HPIA_RG,d);

     	else
        { 	
			// finally, this is TP62/TP67 board.
			// recognize f/w rev.1 in order to use two 16-bit access cycles 
			// instead of one 32-bit access cycle for TP62/TP67 f/w rev.2
	  		if(_BD->fw_rev==1)	
	     		TP6X_write_dword16(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPIA_RG,d);
	   		else
	     		TP6X_write_dword(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPIA_RG,d);
		}
	}

    return(_tp6x_set_err(TP6X_OK));
}


unsigned long TP6X_hpid_data()
// returns HPID register data. 
// For TP6202/TP6203 boards this function just reads HPID register, whereas
// the address AINC feature is controlled via AINC bit of HPIA register, which
// should have been set previously.
{  
	_TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    _tp6x_set_err(TP6X_OK);

	// for TP6202/TP6203/TP64xx boards, just read from HPID register, however
	// note different HPID register address
    if(		_BD->dev_id==TP6202_DEV_ID 
		|| 	_BD->dev_id==TP6203_DEV_ID
	  )
		return(TP6X_read_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPID_RG));

	if( 	_BD->dev_id==TP6414_DEV_ID
		|| 	_BD->dev_id==TP6415_DEV_ID
		|| 	_BD->dev_id==TP6416_DEV_ID
  	  ) 
		return(TP6X_read_dword(TP6X_HIF_AREA,TP64XX_HIF_HPID_RG));

	// finally, this is TP62/TP67 board.
	// recognize f/w rev.1 in order to use two 16-bit access cycles 
	// instead of one 32-bit access cycle for TP62/TP67 f/w rev.2
  	if(_BD->fw_rev==1)
    	return(TP6X_read_dword16(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPID_RG));
   	else
    	return(TP6X_read_dword(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPID_RG));
 }



int TP6X_set_hpid(unsigned long d)
// Sets HPID register data. 
// For TP6202/TP6203 boards this function just writes to HPID register, whereas
// the address AINC feature is controlled via AINC bit of HPIA register, which
// should have been set previously.
{  
	_TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    _tp6x_set_err(TP6X_OK);

	// for TP6202/TP6203/TP64xx boards, just write to HPID register, however
	// note different HPID register address
    if(		_BD->dev_id==TP6202_DEV_ID 
		|| 	_BD->dev_id==TP6203_DEV_ID
	  )
		TP6X_write_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPID_RG,d);
	else
	{
		if( 	_BD->dev_id==TP6414_DEV_ID
			|| 	_BD->dev_id==TP6415_DEV_ID
			|| 	_BD->dev_id==TP6416_DEV_ID
	  	  ) 
			TP6X_write_dword(TP6X_HIF_AREA,TP64XX_HIF_HPID_RG,d);
		else
    	{ 	
			// finally, this is TP62/TP67 board.
			// recognize f/w rev.1 in order to use two 16-bit access cycles 
			// instead of one 32-bit access cycle for TP62/TP67 f/w rev.2
			if(_BD->fw_rev==1)	
	    		TP6X_write_dword16(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPID_RG,d);
	   		else
	    		TP6X_write_dword(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPID_RG,d);
		}
	}

    return(_tp6x_set_err(TP6X_OK));
}



unsigned long TP6X_hpid_ainc_data()
// returns HPID register data and increments address.
// For TP62/TP67 and TP64xx, this is access to 32-bit HPID_AINC register.
// For TP6202/TP6203 with 32-bit HPI port, this is access to HPID with 
// further check for the AINC bit of HPIA register and adjusting HPIA 
// contents if required.
{  
	_TP6X_BOARD_DD *_BD;
    unsigned long d,a;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    _tp6x_set_err(TP6X_OK);

	// read HPID register and adjust HPIA contents upon the state of AINC bit
    if(_BD->dev_id==TP6202_DEV_ID || _BD->dev_id==TP6203_DEV_ID) 
	{ 
	  	if(TP6X_HPI32_AINC_DATA!=0)
			// if AINC bit is set, just return HPID data
			return(TP6X_read_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPID_RG));	
	    else
		{ 
			// if AINC bit is not set, then set it, read HPID data, and reset AINC bit
			TP6X_SET_HPI32_AINC;
			d=TP6X_read_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPID_RG);
			TP6X_CLR_HPI32_AINC;
			return(d);
	    }
	}

	// for TP64xx boards just read HPID-AINC register	
	if(		_BD->dev_id==TP6414_DEV_ID 
		|| 	_BD->dev_id==TP6415_DEV_ID
		|| 	_BD->dev_id==TP6416_DEV_ID
  	  ) 
		return(TP6X_read_dword(TP6X_HIF_AREA,TP64XX_HIF_HPID_AINC_RG));

	// finally, this is TP62/TP67 board.
	// recognize f/w rev.1 in order to use two 16-bit access cycles 
	// instead of one 32-bit access cycle for TP62/TP67 f/w rev.2
	if(_BD->fw_rev==1)
    	return(TP6X_read_dword16(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPID_AINC_RG));
	else
		return(TP6X_read_dword(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPID_AINC_RG));
}



int TP6X_set_hpid_ainc(unsigned long d)
// Sets HPID register data with address auto-increment feature.
// For TP62/TP67 boards and TP64xx boards, this is access to 32-bit HPID_AINC register.
// For TP6202/TP6203 with 32-bit HPI port, this is access to HPID with further check for
// the AINC bit of HPIA register and adjusting HPIA contents if required.
{  
	_TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    _tp6x_set_err(TP6X_OK);

	// write to HPID register and adjust HPIA contents upon the state of AINC bit
    if(_BD->dev_id==TP6202_DEV_ID || _BD->dev_id==TP6203_DEV_ID) 
	{ 
		if(TP6X_HPI32_AINC_DATA!=0)
			// if AINC bit is set, just write to HPID data
			TP6X_write_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPID_RG,d);
	    else
	      { 
			// if AINC bit is not set, then set it, write HPID data, and reset AINC bit
			TP6X_SET_HPI32_AINC;
			TP6X_write_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPID_RG,d);
			TP6X_CLR_HPI32_AINC;
	      }
	}
	else
	{ 	
		// for TP64xx boards just write to HPID-AINC register	
		if(		_BD->dev_id==TP6414_DEV_ID 
			|| 	_BD->dev_id==TP6415_DEV_ID
			|| 	_BD->dev_id==TP6416_DEV_ID
  	  	  ) 
			TP6X_write_dword(TP6X_HIF_AREA,TP64XX_HIF_HPID_AINC_RG,d);
		else
		{
			// finally, this is TP62/TP67 board.
			// recognize f/w rev.1 in order to use two 16-bit access cycles 
			// instead of one 32-bit access cycle for TP62/TP67 f/w rev.2
	  		if(_BD->fw_rev==1)	
	     		TP6X_write_dword16(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPID_AINC_RG,d);
	   		else
	     		TP6X_write_dword(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPID_AINC_RG,d);
		}
	}

    return(_tp6x_set_err(TP6X_OK));
}



int TP6X_hpi_dspint_data()
// recognizes TP6x board and returns state of DSPINT bit
{ 
	unsigned long d;
    _TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    _tp6x_set_err(TP6X_OK);

	// for TP6202/TP6203 boards, read HPIA register and return AINC bit
    if(_BD->dev_id==TP6202_DEV_ID || _BD->dev_id==TP6203_DEV_ID) 
		return((int)TP6X_HPI32_DSPINT_DATA);

	// for TP64xx boards return DSPINT bit of HPIC register	
	if(		_BD->dev_id==TP6414_DEV_ID 
		|| 	_BD->dev_id==TP6415_DEV_ID
		|| 	_BD->dev_id==TP6416_DEV_ID
  	  ) 
		return((int)TP64XX_HPIC_DSPINT_DATA);

	// for TP62/TP67 boards, return DSPINT bit of HPIC register
    return((int)TP6X_HPI16_DSPINT_DATA);
}


int TP6X_set_hpi_dspint()
// Uniform function for setting DSPINT to '1'. For TP62/TP67/TP64xx boards, 
// DSPINT is set C6x DSP HPIC register, whereas for TP6202/TP6203 boards, 
// DSPINT is set via HPIA register.
{  
	_TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);
    _tp6x_set_err(TP6X_OK);

    if(_BD->dev_id==TP6202_DEV_ID || _BD->dev_id==TP6203_DEV_ID) 
		TP6X_SET_HPI32_DSPINT;
    else
	{
		if(		_BD->dev_id==TP6414_DEV_ID 
			|| 	_BD->dev_id==TP6415_DEV_ID
			|| 	_BD->dev_id==TP6416_DEV_ID
  	  	) 
			TP64XX_SET_HPIC_DSPINT;
		else
			TP6X_SET_HPI16_DSPINT;
	}

   	return(_tp6x_set_err(TP6X_OK));
}



int TP6X_hpi_init()
// Init HPI and clear HPI error. 
// For TP62/TP67 with 16-bit HPI port, set HWOB and HINT bits of HPIC and clear HPIA rg.
// For TP64xx boards with 32-bit HPI port, set HINT bit of HPIC and clear HPIA registers.
// For 32-bit HPI at TP6202/TP6203 disable AINC (actually set AINC bit) and clear address 
// field HPIA rg.
// !!! TP6203 only with TMX silicon !!!:
// -------------------------------------
//       C6203 TMX rev.1 and later has the bug, which does not
//       allow to read HPIA data prior HPID data is accessed. As the fix,
//       first write TP6X_DSP_ONCHIP_DRAM_SADDR value to HPIA register,
//       perform read of HPID and then write 0x00000002 to HPIA again, and clear
//		 HPI error. Actually, this can be done only once after power on.
//       DSP reset signal does not effect this silicon bug.
{  
	_TP6X_BOARD_DD *_BD;
    unsigned long xl;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);

	// perform some specific actions for TP6202/TP6203 boards
    if(_BD->dev_id==TP6202_DEV_ID || _BD->dev_id==TP6203_DEV_ID)
	{
	 	TP6X_write_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPIA_RG,TP6X_DSP_ONCHIP_DRAM_SADDR+0x2L);
	  	xl=TP6X_read_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPID_RG);
	  	TP6X_write_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPIA_RG,0x2L);
	}
    else
	{
		if(		_BD->dev_id==TP6414_DEV_ID
			|| 	_BD->dev_id==TP6415_DEV_ID
			|| 	_BD->dev_id==TP6416_DEV_ID
  	  	)
		{
			// for TP64xx boards, set HINT bit of HPIC register in order to clear
			// HINT irq, and clear HPIA register
			TP64XX_CLR_HPIC_HINT;
	     	TP6X_write_dword(TP6X_HIF_AREA,TP64XX_HIF_HPIA_RG,0L);
		}
		else
		{
			// finally, this is TP62/TP67 board.
			// set HWOB and HINT bits of HPIC 
	  		TP6X_CLR_HPI16_HINT;

			// recognize f/w rev.1 in order to use two 16-bit access cycles 
			// instead of one 32-bit access cycle for TP62/TP67 f/w rev.2
	  		if(_BD->fw_rev==1)	
	     		TP6X_write_dword16(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPIA_RG,0L);
	   		else
	     		TP6X_write_dword(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPIA_RG,0L);
		}
	}

	// clear HPI error if it is set
    TP6X_CLR_HPI_ERR;

    return(_tp6x_set_err(TP6X_OK));
}


unsigned TP6X_hpic_data()
// returns HPIC register contents as 16-bit LSW, since LSW==MSW for HPIC.
// This function is valid for TP62/TP67/TP64xx boards only.
{  
	_TP6X_BOARD_DD *_BD;
	unsigned long ul;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));
    _BD=_TP6X_BD_PTR(_tp6x_board_sel);

	// return error for TP6202/TP6203 boards without HPIC register
    if(_BD->dev_id==TP6202_DEV_ID || _BD->dev_id==TP6203_DEV_ID) 
		return(_tp6x_set_err(TP6X_INV_CMND_ERR));

    _tp6x_set_err(TP6X_OK);

	if(		_BD->dev_id==TP6414_DEV_ID 
		|| 	_BD->dev_id==TP6415_DEV_ID
		|| 	_BD->dev_id==TP6416_DEV_ID
  	  ) 
	{
		// for TP64xx boards, read 32-bit HPIC contents and return 16-bit LSW only
		ul= TP6X_read_dword(TP6X_HIF_AREA,TP64XX_HIF_HPIC_RG) & 0xffffL;
		return( (unsigned) ul);
	}

	// finally, this is TP62/TP67 board.
	// just read 16-bit LSW of HPIC and return it.
    return(TP6X_read_word(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPIC_RG));
}



int TP6X_set_hpic(unsigned d)
// Duplicate provided 16-bit input data to LSW/MSW of 32-bit word and write 
// combined 32-bit word to 32-bit HPIC register.
// This function is valid for TP62/TP67/TP64xx boards only. 
{  
	_TP6X_BOARD_DD *_BD;

    if(!_TP6X_OPEN) return(_tp6x_set_err(TP6X_NOT_OPEN_ERR));

    _BD=_TP6X_BD_PTR(_tp6x_board_sel);

	// return error for TP6202/TP6203 boards without HPIC register
    if(_BD->dev_id==TP6202_DEV_ID || _BD->dev_id==TP6203_DEV_ID) 
		return(_tp6x_set_err(TP6X_INV_CMND_ERR));

    _tp6x_set_err(TP6X_OK);

	if(		_BD->dev_id==TP6414_DEV_ID 
		|| 	_BD->dev_id==TP6415_DEV_ID
		|| 	_BD->dev_id==TP6416_DEV_ID
  	  ) 
	{
		// for TP64xx boards, clear unused HWOB bit and write 32-bit data to HPIC 
		// LSW/MSW are duplicated
		TP6X_write_dword(TP6X_HIF_AREA,TP64XX_HIF_HPIC_RG, 
						 ((((unsigned long)d)&0x16L)<<16) | (((unsigned long)d)&0x16L));
	}
	else
    { 	
		// for TP62/TP67 boards, write to HPIC LSW only and set HWOB bit
    	TP6X_write_word(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPIC_RG,(d&0x16)|1);
	}
    return(_tp6x_set_err(TP6X_OK));
}

//========================================================================
