/***************************************************************************

  TORNADO-P6X Host Control Library. v.2.3
  (C) MicroLAB Systems Ltd, 1999-2004

  Notes:
	1. This file include definitions and function declarations only
	   and should be used together with TP6X_H.C file.
	2. TP6X DOS Utilities provide limited support for on-board DPRAM of
	   TP64XX boards. Since PC is unable to allocate full 128K/256K/512Kx32
	   TP64XX on-board DPRAM into UMB area, TP64XX must be configured
	   via TP_DP1M.EXE utility to allocate DPRAM area as 8Kx32 sub-area only.
	   Allocation of this PC allocated 8Kx32 DPRAM sub-area inside DSP
	   address space can vary and is defined by base address of this
	   sub-area inside PC UMB area.

****************************************************************************/

//--- header declaration
#ifndef	__TP6X_DEF__		
#define __TP6X_DEF__		1

#define	_ON					1
#define	_OFF				0

#ifndef BYTE
typedef unsigned char BYTE ; 
#endif

//-------------- function declarations ------------------------
int 			TP6X_open();
int				TP6X_close();
int				TP6X_boards();
int				TP6X_board_id(unsigned,unsigned);
int 			TP6X_sel_board(unsigned,unsigned);
int 			TP6X_sel_board_id(int);
int 			TP6X_board_info(int,unsigned*,unsigned*,BYTE*,BYTE*,
							unsigned long*,unsigned long*,unsigned long*,
                    		unsigned long*,unsigned long*);
int 			TP6X_dev_name(char *s);
unsigned 		TP6X_dev_id();
int 			TP6X_dev_fw_rev();
int 			TP6X_area_alloc_name(unsigned area,char *s);
long 			TP6X_dpram_len();
long 			TP6X_sram_len();
long 			TP6X_dram_len();
unsigned long 	TP6X_dpram_area_dsp_baddr();

int				TP6X_nvram_write(unsigned,BYTE);
int				TP6X_nvram_read(unsigned,BYTE *);

BYTE 			TP6X_read_byte (int, unsigned long);
unsigned 		TP6X_read_word (int, unsigned long);
unsigned long	TP6X_read_dword(int, unsigned long);
unsigned long	TP6X_read_dword16(int, unsigned long);
int 			TP6X_write_byte (int, unsigned long,unsigned);
int	 			TP6X_write_word (int, unsigned long,unsigned);
int				TP6X_write_dword(int, unsigned long,unsigned long);
int				TP6X_write_dword16(int, unsigned long,unsigned long);

int				TP6X_smp_ptr(unsigned long addr,unsigned area,unsigned smp,unsigned *page,void **p);
int				TP6X_set_smp_ptr(unsigned long addr,unsigned area,unsigned smp,void **p);
int				TP6X_set_dpram(unsigned long addr,unsigned long d);
unsigned long 	TP6X_dpram_data(unsigned long addr);
int 			TP6X_set_dpsem(unsigned addr,int d);
int 			TP6X_dpsem_data(unsigned addr);


int				TP6X_last_err();
int				TP6X_err_flag();
int				TP6X_clr_err_flag();
int 			TP6X_err_message(int,char *);
int 			TP6X_last_err_message(char *);

//-------------------------------------------------------------


//------------------- board info ------------------------------
#define TP62_DEV_ID					0x7510
#define TP67_DEV_ID					0x7511
#define TP6202_DEV_ID				0x7512
#define TP6203_DEV_ID				0x7513
#define TP6414_DEV_ID				0x7530
#define TP6415_DEV_ID				0x7531
#define TP6416_DEV_ID				0x7532
#define TP6X_INV_DEV_ID				0xFFFF	// returned by TP6X_dev_id() in case API is not open or no TP6x boards detected

#define TP62_VENDOR_ID				0x10e8	// TP62: AMCC as vendor so far
#define TP67_VENDOR_ID				0x10e8	// TP67: AMCC as vendor so far
#define TP6202_VENDOR_ID			0x10e8	// TP6202: AMCC as vendor so far
#define TP6203_VENDOR_ID			0x10e8	// TP6203: AMCC as vendor so far
#define TP6414_VENDOR_ID			0x10e8	// TP6414: AMCC as vendor so far
#define TP6415_VENDOR_ID			0x10e8	// TP6415: AMCC as vendor so far
#define TP6416_VENDOR_ID			0x10e8	// TP6416: AMCC as vendor so far

#define	TP6X_BOARD_TYPES			7		// current total number of valid TP6X class boards
#define	TP6X_BADDR_AREAS			5		// valid areas = BADDR0..4

#define	TP6X_MAX_BOARDS				32		// max number of installed boards
#define	TP6X_CURRENT_BOARD			-1		// current board selector for API functions


	//-- macros for current board ID
#define	TP62_BOARD					(TP6X_dev_id()==TP62_DEV_ID)
#define	TP67_BOARD					(TP6X_dev_id()==TP67_DEV_ID)
#define	TP62_TP67_BOARD				(TP62_BOARD || TP67_BOARD)
#define	TP6202_BOARD				(TP6X_dev_id()==TP6202_DEV_ID)
#define	TP6203_BOARD				(TP6X_dev_id()==TP6203_DEV_ID)
#define	TP6202_TP6203_BOARD			(TP6202_BOARD || TP6203_BOARD)
#define	TP62XX_TP67_BOARD			(TP62_TP67_BOARD || TP6202_TP6203_BOARD)
#define	TP6414_BOARD				(TP6X_dev_id()==TP6414_DEV_ID)
#define	TP6415_BOARD				(TP6X_dev_id()==TP6415_DEV_ID)
#define	TP6416_BOARD				(TP6X_dev_id()==TP6416_DEV_ID)
#define	TP64XX_BOARD				(TP6414_BOARD || TP6415_BOARD || TP6416_BOARD)

	//-- macros for board f/w revision ID
#define	TP62_BOARD_FW_REV1			(TP62_BOARD && TP6X_dev_fw_rev()==1)
#define	TP62_BOARD_FW_REV2			(TP62_BOARD && TP6X_dev_fw_rev()==2)
#define	TP67_BOARD_FW_REV1			(TP67_BOARD && TP6X_dev_fw_rev()==1)
#define	TP67_BOARD_FW_REV2			(TP67_BOARD && TP6X_dev_fw_rev()==2)
#define	TP62_TP67_BOARD_FW_REV1		((TP67_BOARD || TP62_BOARD) && TP6X_dev_fw_rev()==1)
#define	TP62_TP67_BOARD_FW_REV2		((TP67_BOARD || TP62_BOARD) && TP6X_dev_fw_rev()==2)

#define	TP6202_BOARD_FW_REV1		(TP6202_BOARD && TP6X_dev_fw_rev()==1)
#define	TP6203_BOARD_FW_REV1		(TP6203_BOARD && TP6X_dev_fw_rev()==1)
#define	TP6202_TP6203_BOARD_FW_REV1	((TP6202_BOARD || TP6203_BOARD) && TP6X_dev_fw_rev()==1)

#define	TP6414_BOARD_FW_REV1A		(TP6414_BOARD && TP6X_dev_fw_rev()==1)
#define	TP6415_BOARD_FW_REV1A		(TP6415_BOARD && TP6X_dev_fw_rev()==1)
#define	TP6416_BOARD_FW_REV1A		(TP6416_BOARD && TP6X_dev_fw_rev()==1)
#define	TP64XX_BOARD_FW_REV1A		(TP64XX_BOARD && TP6X_dev_fw_rev()==1)

#define	TP6414_BOARD_FW_REV1		TP6414_BOARD_FW_REV1A
#define	TP6415_BOARD_FW_REV1		TP6415_BOARD_FW_REV1A
#define	TP6416_BOARD_FW_REV1		TP6416_BOARD_FW_REV1A
#define	TP64XX_BOARD_FW_REV1		TP64XX_BOARD_FW_REV1A
			// added TP64xx f/w rev.1A-1 and rev.1A-2, which differ in the DSP EMIF settings
#define	TP6414_BOARD_FW_REV1A1		(TP6414_BOARD && TP6X_dev_fw_rev()==2)
#define	TP6415_BOARD_FW_REV1A1		(TP6415_BOARD && TP6X_dev_fw_rev()==2)
#define	TP6416_BOARD_FW_REV1A1		(TP6416_BOARD && TP6X_dev_fw_rev()==2)
#define	TP64XX_BOARD_FW_REV1A1		(TP64XX_BOARD && TP6X_dev_fw_rev()==2)

#define	TP6414_BOARD_FW_REV1A2		(TP6414_BOARD && TP6X_dev_fw_rev()==3)
#define	TP6415_BOARD_FW_REV1A2		(TP6415_BOARD && TP6X_dev_fw_rev()==3)
#define	TP6416_BOARD_FW_REV1A2		(TP6416_BOARD && TP6X_dev_fw_rev()==3)
#define	TP64XX_BOARD_FW_REV1A2		(TP64XX_BOARD && TP6X_dev_fw_rev()==3)

#define	TP6414_BOARD_FW_REV1B		(TP6414_BOARD && TP6X_dev_fw_rev()==4)
#define	TP6415_BOARD_FW_REV1B		(TP6415_BOARD && TP6X_dev_fw_rev()==4)
#define	TP6416_BOARD_FW_REV1B		(TP6416_BOARD && TP6X_dev_fw_rev()==4)
#define	TP64XX_BOARD_FW_REV1B		(TP64XX_BOARD && TP6X_dev_fw_rev()==4)


//=============== Host I/F rg definitions ==================

	//--- TP6X IO/MEM Pass-thru (BADDR) areas definition
#define	TP6X_PCIC_AREA				0
#define	TP6X_ECC_AREA				1
#define	TP6X_DPM_AREA				3
#define	TP6X_HIF_AREA				4
#define	TP6X_RSRV_AREA				2


	//--- PCIC = {AMCC S5933}: Op-Registers -----
	//    (assigned to TP6X_BADDR_AREA_PCI_RG area)
#define _AMCC_OMB1_RG				0x00
#define _AMCC_OMB2_RG				0x04
#define _AMCC_OMB3_RG				0x08
#define _AMCC_OMB4_RG				0x0c
#define _AMCC_IMB1_RG				0x10
#define _AMCC_IMB2_RG				0x14
#define _AMCC_IMB3_RG				0x18
#define _AMCC_IMB4_RG				0x1c
#define _AMCC_FIFO_RG				0x20
#define _AMCC_MWAR_RG				0x24
#define _AMCC_MWTC_RG				0x28
#define _AMCC_MRAR_RG				0x2c
#define _AMCC_MRTC_RG				0x30
#define _AMCC_MBEF_RG				0x34
#define _AMCC_INTCSR_RG				0x38
#define _AMCC_MCSR_RG				0x3c
#define _AMCC_MCSR_NVDATAP_RG		(_AMCC_MCSR_RG + 2) /* Data in byte 2 */
#define _AMCC_MCSR_NVCMD_RG 		(_AMCC_MCSR_RG + 3) /* Command in byte 3 */

#define _AMCC_NVRAM_LEN_MAX			0x800		// maximum length of NVRAM (bytes)

	//--- DPRAM/DPSEM area (32-bit)
	//    Note, that TP64xx provide limited DPRAM support via TP6x DOS Utilities, since
	//	  only 32kByte non-pageble sub-area of TP64xx on-board DPRAM is allocated into UMB.
	//	  For full DPRAM support use TORNADO Software Development Kit (TSDK) for Windows.
#define	TP6X_DPM_DPRAM_LEN			TP6X_dpram_len()
#define	TP6X_DPM_DPRAM_LEN_MAX		0x40000L	// TP62xx/TP67 only
#define	TP6X_DPM_DPSEM_LEN			0x8 		// TP62xx/TP67 only: DPSEM length (8 semaphores)

#define	TP64XX_DPM_DPRAM_LEN_MAX	0x8000L		// TP64xx only

#define	TP6X_DPM_DPRAM_HM_RQ		(TP6X_dpram_len()-4L)	// TP62xx/TP67 only
#define	TP6X_DPM_DPRAM_MH_RQ		(TP6X_dpram_len()-8L)	// TP62xx/TP67 only

#define	TP6X_DPM_DPSEM_BITMASK		0x1 	// TP62xx/TP67 only: DPSEM data bitmask
#define	TP6X_DPM_DPSEM_OFF			0x1 	// TP62xx/TP67 only: DPSEM OFF data
#define	TP6X_DPM_DPSEM_ON			0x0 	// TP62xx/TP67 only: DPSEM ON data


	//--- ECC area (16-bit)
#define	TP6X_ECC_ECC				0x0		// TBC sub-area base address
#define	TP6X_ECC_AUX_EMU_SEL_RG		0x40	// r/w: DSP JTAG path selector (ECC/XDS)
#define	TP6X_ECC_AUX_STAT_RG		0x44	// r: ECC timeout, IRQ
#define	TP6X_ECC_AUX_ECC_RESET		0x4c	// AUX sub-area (byte): ECC reset
#define	TP6X_ECC_AUX_CLR_ECC_ERR	0x4d	// w/byte-0: clear ECC timeout (data ignored)

		//-- EMU selector defs (TP6X_ECC_EMU_SEL_RG)
#define	TP6X_EMU_SEL_BITMASK		1
#define	TP6X_EMU_SEL_XEMU			0
#define	TP6X_EMU_SEL_ECC			1

		//-- ECC STATUS RG
#define	TP6X_ECC_ERR_BITMASK		1	// ECC access Timeout error
#define	TP6X_ECC_IRQ_BITMASK		2	// ECC IRQ


	//--- HIF area (16-bit access)
			//--- general RGs (8-bit)
#define	TP6X_HIF_SMP_RG				0x0		// TP62xx/TP67 only: r/w: SMP-0/1 registers
#define	TP6X_HIF_SMP0_RG			0x0		// TP62xx/TP67 only
#define	TP6X_HIF_SMP1_RG			0x4		// TP62xx/TP67 only
#define	TP6X_HIF_CNTR_RG			0x8		// r/w: DSP control RG (reset)
#define	TP6X_HIF_IM_RG				0x10	// r/w: Host IMR
#define	TP6X_HIF_IS_RG				0x14	// r/w: Host ISR
#define	TP6X_HIF_HM_RQ0_RG			0x18	// r/w: Host-to-DSP RQ-0/1 (4-bit doorbell rgs)
#define	TP6X_HIF_HM_RQ1_RG			0x1c	
#define	TP6X_HIF_CLR_DPRAM_ERR		0x20	// TP62xx/TP67 only: w/BYTE: clear DPRAM timeout
#define	TP6X_HIF_CLR_HPI_ERR		0x21	// w/BYTE: clear HPI timeout
#define	TP6X_HIF_CLR_MH_RQ			0x22	// w/BYTE: clear DSP-to-HOST rq
#define	TP6X_HIF_DSP_BMODE_RG		0x2C	// TP64xx only: r/w: DSP bootmode control for host mode

			//--- C6x HPI defs
			//    These old defs for HPI regs are for TP62/TP67/TP64xx only and are
			//    keeped for compatibility API with rev.1. Not recommended for new designs,
#define	TP6X_HIF_HPIC_RG					0x30	// r/w: C6x/HPI: HPIC
#define	TP6X_HIF_HPIC_RG_LSW				0x30	// r/w: C6x/HPI: HPIC/LSW
#define	TP6X_HIF_HPIC_RG_MSW				0x32	// r/w: C6x/HPI: HPIC/MSW
#define	TP6X_HIF_HPIA_RG					0x34	// r/w: C6x/HPI: HPIA
#define	TP6X_HIF_HPIA_RG_LSW				0x34	// r/w: C6x/HPI: HPIA/LSW
#define	TP6X_HIF_HPIA_RG_MSW				0x36	// r/w: C6x/HPI: HPIA/MSW
#define	TP6X_HIF_HPID_AINC_RG				0x38	// r/w: C6x/HPI: HPID+AINC
#define	TP6X_HIF_HPID_AINC_RG_LSW			0x38	// r/w: C6x/HPI: HPID+AINC/LSW
#define	TP6X_HIF_HPID_AINC_RG_MSW			0x3a	// r/w: C6x/HPI: HPID+AINC/MSW
#define	TP6X_HIF_HPID_RG					0x3c	// r/w: C6x/HPI: HPID
#define	TP6X_HIF_HPID_RG_LSW				0x3c	// r/w: C6x/HPI: HPID/LSW
#define	TP6X_HIF_HPID_RG_MSW				0x3e	// r/w: C6x/HPI: HPID/MSW

			//    recommended new defs for HPI regs for TP62/TP67
#define	TP6X_HIF_HPI16_HPIC_RG				0x30	// r/w: C6201/C6701 HPIC
#define	TP6X_HIF_HPI16_HPIC_RG_LSW			0x30	// r/w: C6201/C6701 HPIC/LSW
#define	TP6X_HIF_HPI16_HPIC_RG_MSW			0x32	// r/w: C6201/C6701 HPIC/MSW
#define	TP6X_HIF_HPI16_HPIA_RG				0x34	// r/w: C6201/C6701 HPIA
#define	TP6X_HIF_HPI16_HPIA_RG_LSW			0x34	// r/w: C6201/C6701 HPIA/LSW
#define	TP6X_HIF_HPI16_HPIA_RG_MSW			0x36	// r/w: C6201/C6701 HPIA/MSW
#define	TP6X_HIF_HPI16_HPID_AINC_RG			0x38	// r/w: C6201/C6701 HPID+AINC
#define	TP6X_HIF_HPI16_HPID_AINC_RG_LSW		0x38	// r/w: C6201/C6701 HPID+AINC/LSW
#define	TP6X_HIF_HPI16_HPID_AINC_RG_MSW		0x3a	// r/w: C6201/C6701 HPID+AINC/MSW
#define	TP6X_HIF_HPI16_HPID_RG				0x3c	// r/w: C6201/C6701 HPID
#define	TP6X_HIF_HPI16_HPID_RG_LSW			0x3c	// r/w: C6201/C6701 HPID/LSW
#define	TP6X_HIF_HPI16_HPID_RG_MSW			0x3e	// r/w: C6201/C6701 HPID/MSW


			//--- TP6202/TP6203 32-bit HPI port
#define	TP6X_HIF_HPI32_HPID_RG				0x30	// r/w: C6202/C6203 HPID reg
#define	TP6X_HIF_HPI32_HPIA_RG				0x34	// r/w: C6202/C6203 HPIA reg

			//--- TP64xx 32-bit HPI port
#define	TP64XX_HIF_HPIC_RG					0x30	// r/w: 32-bit HPIC reg
#define	TP64XX_HIF_HPIA_RG					0x34	// r/w: 32-bit HPIA reg
#define	TP64XX_HIF_HPID_AINC_RG				0x38	// r/w: 32-bit HPID+AINC reg
#define	TP64XX_HIF_HPID_RG					0x3C	// r/w: 32-bit HPID reg


			//--- HIF area defs
				//-- DSP CTRL register
#define	TP6X_MRESET_GO					1	// Host MRESET control
#define	TP6X_MRESET_RESET				0	
#define	TP6X_MRESET_BITMASK				1

#define	TP6X_MRESET_MODE_BITMASK		2	// mask for MRESET mode
#define	TP6X_MRESET_MODE_HOST			0	// host mode for MRESET
#define	TP6X_MRESET_MODE_SA				2	// stand-alone mode for MRESET
											// (x-reset, reset button, uP supervisor)

#define	TP6X_DSP_AMEN_BITMASK			0x10	// mask for DSP PCI-bus mastering enable
#define	TP6X_DSP_AMEN_OFF				0		// AMEN disabled
#define	TP6X_DSP_AMEN_ON				0x10	// AMEN enabled

#define	TP6X_DSP_PD_BITMASK				0x40	// DSP PD (power down) state (TP62xx/TP67 only)
#define	TP6X_DSP_PD_OFF					0		// DSP is noth in the PD state
#define	TP6X_DSP_PD_ON					0x40	// DSP is in the PD state
#define	TP6X_DSP_PD						TP6X_DSP_PD_ON

#define	TP6X_DSP_MRESET_BITMASK			0x80	// mask for returned MRESET value (effected by MRESET or X-MRESET/uP_supervisor)
#define	TP6X_DSP_MRESET_GO				0x80	// MRESET=MGO
#define	TP6X_DSP_MRESET_RESET			0x0		// MRESET=RESET


				//-- 4-bit Host-to-DSP rq door bell rgs
#define	TP6X_HM_RQ_BITMASK				0xf

				//-- SMP-0/1 rg defs (TP62xx/TP67 only)
#define	TP6X_SMP_SMP0_SEL				0		//-- SMP0/1 IDs
#define	TP6X_SMP_SMP1_SEL				1

#define	TP6X_SMP_MODE_BITMASK			0x40	// DPM mode (bit #6 @SMP0)
#define	TP6X_SMP_MODE_SPA				0x0		// DPM single-page-access (SPA) mode (SMP-0 only)
#define	TP6X_SMP_MODE_DPA				0x40	// DPM dual-page-access (DPA) mode (SMP-0/1)

#define	TP6X_SMP_AREA_BITMASK			0x80	//-- DPRAM/DPSEM defs (SMP-0/1)
#define	TP6X_SMP_DPRAM_AREA				0   	// DPRAM/DPSEM sel: bit#7 of SMP0
#define	TP6X_SMP_DPSEM_AREA				0x80
#define	TP6X_SMP_SPA_PAGE_BITMASK		0x7		// SPA mode: DPRAM page # (SMP-0)
#define	TP6X_SMP_DPA_PAGE_BITMASK		0xf		// DPA mode: DPRAM page # (SMP-0/1)

#define	TP6X_SMP_DPA_SMP0_OFFSET		0x0000	// DPA mode: SMP0 offset
#define	TP6X_SMP_DPA_SMP1_OFFSET		0x4000	// DPA mode: SMP1 offset
#define	TP6X_SMP_SMP_LEN				0x8000	// summary DPM page length
#define	TP6X_SMP_SPA_SMP_LEN			0x8000	// SPA mode page length
#define	TP6X_SMP_DPA_SMP_LEN			0x4000	// DPA mode page length

#define	TP64XX_SMP_SMP_LEN				0x8000	// SMP length for TP64xx boards

		//-- IMR/ISR bits
#define	TP6X_IM_DPRAM_IE_BITMASK		0x1		// TP62xx/TP67 and TP64xx with 256Kx32 DPRAM only: DPRAM IE/IRQ
#define	TP6X_IM_DPRAM_IE_OFF			0
#define	TP6X_IM_DPRAM_IE_ON				0x1
#define	TP6X_IM_DPRAM_IE				TP6X_IM_DPRAM_IE_ON

#define	TP6X_IS_DPRAM_IRQ_BITMASK		0x1		// TP62xx/TP67 and TP64xx with 256Kx32 DPRAM only: DPRAM IE/IRQ
#define	TP6X_IS_DPRAM_IRQ_OFF			0
#define	TP6X_IS_DPRAM_IRQ_ON			0x1
#define	TP6X_IS_DPRAM_IRQ				TP6X_IS_DPRAM_IRQ_ON

#define	TP6X_IM_MH_RQ_IE_BITMASK		0x4		// DSP-to-Host RQ IE/IRQ
#define	TP6X_IM_MH_RQ_IE_OFF			0
#define	TP6X_IM_MH_RQ_IE_ON				0x4
#define	TP6X_IM_MH_RQ_IE				TP6X_IM_MH_RQ_IE_ON

#define	TP6X_IS_MH_RQ_IRQ_BITMASK		0x4
#define	TP6X_IS_MH_RQ_IRQ_OFF			0
#define	TP6X_IS_MH_RQ_IRQ_ON			0x4
#define	TP6X_IS_MH_RQ_IRQ				TP6X_IS_MH_RQ_IRQ_ON

#define	TP6X_IM_DPRAM_ERR_IE_BITMASK	0x10	// TP62xx/TP67 only:  DPRAM access Timeout IE/IRQ
#define	TP6X_IM_DPRAM_ERR_IE_OFF		0
#define	TP6X_IM_DPRAM_ERR_IE_ON			0x10
#define	TP6X_IM_DPRAM_ERR_IE			TP6X_IM_DPRAM_ERR_IE_ON

#define	TP6X_IS_DPRAM_ERR_IRQ_BITMASK	0x10	// TP62xx/TP67 only
#define	TP6X_IS_DPRAM_ERR_IRQ_OFF		0
#define	TP6X_IS_DPRAM_ERR_IRQ_ON		0x10
#define	TP6X_IS_DPRAM_ERR_IRQ			TP6X_IS_DPRAM_ERR_IRQ_ON

#define	TP6X_IM_HPI_ERR_IE_BITMASK		0x20	// HPI access Timeout IE/IRQ
#define	TP6X_IM_HPI_ERR_IE_OFF			0
#define	TP6X_IM_HPI_ERR_IE_ON			0x20
#define	TP6X_IM_HPI_ERR_IE				TP6X_IM_HPI_ERR_IE_ON

#define	TP6X_IS_HPI_ERR_IRQ_BITMASK		0x20
#define	TP6X_IS_HPI_ERR_IRQ_OFF			0
#define	TP6X_IS_HPI_ERR_IRQ_ON			0x20
#define	TP6X_IS_HPI_ERR_IRQ				TP6X_IS_HPI_ERR_IRQ_ON

#define	TP6X_IM_HPI_HINT_IE_BITMASK		0x2		// TP62/TP67/TP64xx only: HPI HINT
#define	TP6X_IM_HPI_HINT_IE_OFF			0
#define	TP6X_IM_HPI_HINT_IE_ON			0x2
#define	TP6X_IM_HPI_HINT_IE				TP6X_IM_HPI_HINT_IE_ON

#define	TP6X_IS_HPI_HINT_IRQ_BITMASK	0x2     // TP62/TP67/TP64xx only
#define	TP6X_IS_HPI_HINT_IRQ_OFF		0
#define	TP6X_IS_HPI_HINT_IRQ_ON			0x2
#define	TP6X_IS_HPI_HINT_IRQ			TP6X_IS_HPI_HINT_IRQ_ON

#define	TP6X_IM_HPI16_HINT_IE			TP6X_IM_HPI_HINT_IE		// old API TP62/TP67 defs for HPI HINT IE/IRQ
#define	TP6X_IS_HPI16_HINT_IRQ			TP6X_IS_HPI_HINT_IRQ


				//-- DSP Bootmode control register (TP64xx only)
				//   This register can be written during DSP reset for host DSP mode only.
#define	TP6X_DSP_BMODE_BITMASK			3
#define	TP6X_DSP_BMODE_NONE				0
#define	TP6X_DSP_BMODE_HPI				1
#define	TP6X_DSP_BMODE_FLASH			2




//===================== MACROS for data access ==============================
		
		//--- PCIC access
#define	TP6X_SET_PCIC_OMBX(n,d)			TP6X_write_dword16(TP6X_PCIC_AREA,_AMCC_OMB1_RG+((n%4)*4),d)
#define	TP6X_PCIC_OMBX_DATA(n)			TP6X_read_dword16(TP6X_PCIC_AREA,_AMCC_OMB1_RG+((n%4)*4))
#define	TP6X_PCIC_IMBX_DATA(n)			TP6X_read_dword(TP6X_PCIC_AREA,_AMCC_IMB1_RG+((n%4)*4))
#define	TP6X_PCIC_MBEF_DATA				TP6X_read_dword(TP6X_PCIC_AREA,_AMCC_MBEF_RG)
#define	TP6X_SET_PCIC_INTCSR(d)			TP6X_write_dword(TP6X_PCIC_AREA,_AMCC_INTCSR_RG,d)
#define	TP6X_PCIC_INTCSR_DATA			TP6X_read_dword(TP6X_PCIC_AREA,_AMCC_INTCSR_RG)
#define	TP6X_SET_PCIC_FIFO(d)			TP6X_write_dword(TP6X_PCIC_AREA,_AMCC_FIFO_RG,d)
#define	TP6X_PCIC_FIFO_DATA				TP6X_read_dword(TP6X_PCIC_AREA,_AMCC_FIFO_RG)
#define	TP6X_SET_PCIC_MCSR(d)			TP6X_write_dword(TP6X_PCIC_AREA,_AMCC_MCSR_RG,d)
#define	TP6X_PCIC_MCSR_DATA				TP6X_read_dword(TP6X_PCIC_AREA,_AMCC_MCSR_RG)
#define	TP6X_PCIC_MWAR_DATA				TP6X_read_dword(TP6X_PCIC_AREA,_AMCC_MWAR_RG)
#define	TP6X_PCIC_MWTC_DATA				TP6X_read_dword(TP6X_PCIC_AREA,_AMCC_MWTC_RG)
#define	TP6X_PCIC_MRAR_DATA				TP6X_read_dword(TP6X_PCIC_AREA,_AMCC_MRAR_RG)
#define	TP6X_PCIC_MRTC_DATA				TP6X_read_dword(TP6X_PCIC_AREA,_AMCC_MRTC_RG)

		//--- DPRAM/DPSEM areas
#define TP6X_SET_DPRAM(a,d)				TP6X_set_dpram(a,d)
#define TP6X_DPRAM_DATA(a)				TP6X_dpram_data(a)
			// HM_RQ/MH_RQ via DPRAM are supported for TP62xx/TP67 only
#define TP6X_SET_DPRAM_HM_RQ(d)			TP6X_set_dpram(TP6X_DPM_DPRAM_HM_RQ,d)
#define TP6X_DPRAM_HM_RQ_DATA			TP6X_dpram_data(TP6X_DPM_DPRAM_HM_RQ)
#define TP6X_SET_DPRAM_MH_RQ(d)			TP6X_set_dpram(TP6X_DPM_DPRAM_MH_RQ,d)
#define TP6X_DPRAM_MH_RQ_DATA			TP6X_dpram_data(TP6X_DPM_DPRAM_MH_RQ)
			// DPSEM are supported for TP62xx/TP67 only
#define TP6X_SET_DPSEM(a,d)				TP6X_set_dpsem(a,d)
#define TP6X_DPSEM_DATA(a)				TP6X_dpsem_data(a)


		//--- ECC area
			//--- EMU SEL rg
#define TP6X_EMU_SEL_RG_DATA			(TP6X_read_byte(TP6X_ECC_AREA,TP6X_ECC_AUX_EMU_SEL_RG) & TP6X_EMU_SEL_BITMASK)
#define TP6X_SET_EMU_SEL_RG(d)			TP6X_write_byte(TP6X_ECC_AREA,TP6X_ECC_AUX_EMU_SEL_RG,d & TP6X_EMU_SEL_BITMASK)
#define TP6X_SET_EMU_SEL_XEMU			TP6X_write_byte(TP6X_ECC_AREA,TP6X_ECC_AUX_EMU_SEL_RG,TP6X_EMU_SEL_XEMU)
#define TP6X_SET_EMU_SEL_ECC			TP6X_write_byte(TP6X_ECC_AREA,TP6X_ECC_AUX_EMU_SEL_RG,TP6X_EMU_SEL_ECC)

			//--- ECC STATUS rg
#define TP6X_ECC_STAT_RG_DATA			(TP6X_read_byte(TP6X_ECC_AREA,TP6X_ECC_AUX_STAT_RG) & (TP6X_ECC_ERR_BITMASK | TP6X_ECC_IRQ_BITMASK))
#define TP6X_ECC_ERR_DATA				(TP6X_read_byte(TP6X_ECC_AREA,TP6X_ECC_AUX_STAT_RG) & TP6X_ECC_ERR_BITMASK)
#define TP6X_ECC_IRQ_DATA				(TP6X_read_byte(TP6X_ECC_AREA,TP6X_ECC_AUX_STAT_RG) & TP6X_ECC_IRQ_BITMASK)

			//--- strobes for ECC reset and clear ECC TMOUT error
#define TP6X_ECC_RESET					TP6X_write_byte(TP6X_ECC_AREA,TP6X_ECC_AUX_ECC_RESET,0)
#define TP6X_CLR_ECC_ERR				TP6X_write_byte(TP6X_ECC_AREA,TP6X_ECC_AUX_CLR_ECC_ERR,0)
#define TP6X_ECC_RESET_CLR_ECC_ERR		TP6X_write_word(TP6X_ECC_AREA,TP6X_ECC_AUX_ECC_RESET,0)

		//--- HIF area
			//--- CNTR rg
#define TP6X_CNTR_RG_DATA				TP6X_read_byte(TP6X_HIF_AREA,TP6X_HIF_CNTR_RG)
#define TP6X_SET_CNTR_RG(d)				TP6X_write_byte(TP6X_HIF_AREA,TP6X_HIF_CNTR_RG,d)

#define TP6X_MRESET_DATA				(TP6X_CNTR_RG_DATA & TP6X_MRESET_BITMASK)
#define TP6X_SET_MRESET					TP6X_SET_CNTR_RG(TP6X_CNTR_RG_DATA & ~TP6X_MRESET_BITMASK)
#define TP6X_SET_MGO					TP6X_SET_CNTR_RG(TP6X_CNTR_RG_DATA |  TP6X_MRESET_BITMASK)

#define TP6X_MRESET_MODE_DATA			(TP6X_CNTR_RG_DATA & TP6X_MRESET_MODE_BITMASK)
#define TP6X_SET_MRESET_MODE_HOST		TP6X_SET_CNTR_RG(TP6X_CNTR_RG_DATA & ~TP6X_MRESET_MODE_BITMASK)
#define TP6X_SET_MRESET_MODE_SA			TP6X_SET_CNTR_RG(TP6X_CNTR_RG_DATA |  TP6X_MRESET_MODE_BITMASK)

#define TP6X_DSP_AMEN_DATA				(TP6X_CNTR_RG_DATA & TP6X_DSP_AMEN_BITMASK)
#define TP6X_SET_DSP_AMEN_OFF			TP6X_SET_CNTR_RG(TP6X_CNTR_RG_DATA & ~TP6X_DSP_AMEN_BITMASK)
#define TP6X_SET_DSP_AMEN_ON			TP6X_SET_CNTR_RG(TP6X_CNTR_RG_DATA |  TP6X_DSP_AMEN_BITMASK)

#define TP6X_DSP_MRESET_DATA			(TP6X_CNTR_RG_DATA & TP6X_DSP_MRESET_BITMASK)
#define TP6X_DSP_PD_DATA				(TP6X_CNTR_RG_DATA & TP6X_DSP_PD_BITMASK)

			//--- HM_RQ rgs
#define TP6X_HM_RQ0_RG_DATA				(TP6X_read_byte(TP6X_HIF_AREA,TP6X_HIF_HM_RQ0_RG) & TP6X_HM_RQ_BITMASK)
#define TP6X_SET_HM_RQ0_RG(d)			TP6X_write_byte(TP6X_HIF_AREA,TP6X_HIF_HM_RQ0_RG,d & TP6X_HM_RQ_BITMASK)
#define TP6X_HM_RQ1_RG_DATA				(TP6X_read_byte(TP6X_HIF_AREA,TP6X_HIF_HM_RQ1_RG) & TP6X_HM_RQ_BITMASK)
#define TP6X_SET_HM_RQ1_RG(d)			TP6X_write_byte(TP6X_HIF_AREA,TP6X_HIF_HM_RQ1_RG,d & TP6X_HM_RQ_BITMASK)

			//--- output strobes
#define TP6X_CLR_DPRAM_ERR				TP6X_write_byte(TP6X_HIF_AREA,TP6X_HIF_CLR_DPRAM_ERR,0)
#define TP6X_CLR_HPI_ERR				TP6X_write_byte(TP6X_HIF_AREA,TP6X_HIF_CLR_HPI_ERR,0)
#define TP6X_CLR_MH_RQ					TP6X_write_byte(TP6X_HIF_AREA,TP6X_HIF_CLR_MH_RQ,0)

			//--- IMR rg
#define TP6X_IM_RG_DATA					TP6X_read_byte(TP6X_HIF_AREA,TP6X_HIF_IM_RG)
#define TP6X_SET_IM_RG(d)				TP6X_write_byte(TP6X_HIF_AREA,TP6X_HIF_IM_RG,d)

#define TP6X_DPRAM_IE_DATA				(TP6X_IM_RG_DATA & TP6X_IM_DPRAM_IE)
#define TP6X_SET_DPRAM_IE				TP6X_SET_IM_RG(TP6X_IM_RG_DATA | TP6X_IM_DPRAM_IE)
#define TP6X_CLR_DPRAM_IE				TP6X_SET_IM_RG(TP6X_IM_RG_DATA & ~TP6X_IM_DPRAM_IE)
#define TP6X_HPI_HINT_IE_DATA			(TP6X_IM_RG_DATA & TP6X_IM_HPI_HINT_IE)
#define TP6X_SET_HPI_HINT_IE			TP6X_SET_IM_RG(TP6X_IM_RG_DATA | TP6X_IM_HPI_HINT_IE)
#define TP6X_CLR_HPI_HINT_IE			TP6X_SET_IM_RG(TP6X_IM_RG_DATA & ~TP6X_IM_HPI_HINT_IE)
#define TP6X_MH_RQ_IE_DATA				(TP6X_IM_RG_DATA & TP6X_IM_MH_RQ_IE)
#define TP6X_SET_MH_RQ_IE				TP6X_SET_IM_RG(TP6X_IM_RG_DATA | TP6X_IM_MH_RQ_IE)
#define TP6X_CLR_MH_RQ_IE				TP6X_SET_IM_RG(TP6X_IM_RG_DATA & ~TP6X_IM_MH_RQ_IE)
#define TP6X_DPRAM_ERR_IE_DATA			(TP6X_IM_RG_DATA & TP6X_IM_DPRAM_ERR_IE)
#define TP6X_SET_DPRAM_ERR_IE			TP6X_SET_IM_RG(TP6X_IM_RG_DATA | TP6X_IM_DPRAM_ERR_IE)
#define TP6X_CLR_DPRAM_ERR_IE			TP6X_SET_IM_RG(TP6X_IM_RG_DATA & ~TP6X_IM_DPRAM_ERR_IE)
#define TP6X_HPI_ERR_IE_DATA			(TP6X_IM_RG_DATA & TP6X_IM_HPI_ERR_IE)
#define TP6X_SET_HPI_ERR_IE				TP6X_SET_IM_RG(TP6X_IM_RG_DATA | TP6X_IM_HPI_ERR_IE)
#define TP6X_CLR_HPI_ERR_IE				TP6X_SET_IM_RG(TP6X_IM_RG_DATA & ~TP6X_IM_HPI_ERR_IE)

			//--- ISR rg
#define TP6X_IS_RG_DATA					TP6X_read_byte(TP6X_HIF_AREA,TP6X_HIF_IS_RG)
#define TP6X_DPRAM_IRQ_DATA				(TP6X_IS_RG_DATA & TP6X_IS_DPRAM_IRQ)
#define TP6X_HPI_HINT_DATA				(TP6X_IS_RG_DATA & TP6X_IS_HPI_HINT_IRQ)
#define TP6X_MH_RQ_DATA					(TP6X_IS_RG_DATA & TP6X_IS_MH_RQ_IRQ)
#define TP6X_DPRAM_ERR_DATA				(TP6X_IS_RG_DATA & TP6X_IS_DPRAM_ERR_IRQ)
#define TP6X_HPI_ERR_DATA				(TP6X_IS_RG_DATA & TP6X_IS_HPI_ERR_IRQ)

			//--- DSP bootmode rg (TP64xx boards only)
#define TP6X_DSP_BMODE_RG_DATA			TP6X_read_byte(TP6X_HIF_AREA,TP6X_HIF_DSP_BMODE_RG)
#define TP6X_SET_DSP_BMODE_RG(d)		TP6X_write_byte(TP6X_HIF_AREA,TP6X_HIF_DSP_BMODE_RG,d)

#define TP6X_SET_DSP_BMODE_NONE			TP6X_write_byte(TP6X_HIF_AREA,TP6X_HIF_DSP_BMODE_RG,TP6X_DSP_BMODE_NONE)
#define TP6X_SET_DSP_BMODE_HPI			TP6X_write_byte(TP6X_HIF_AREA,TP6X_HIF_DSP_BMODE_RG,TP6X_DSP_BMODE_HPI)
#define TP6X_SET_DSP_BMODE_FLASH		TP6X_write_byte(TP6X_HIF_AREA,TP6X_HIF_DSP_BMODE_RG,TP6X_DSP_BMODE_FLASH)

#define TP6X_DSP_BMODE_DATA				TP6X_DSP_BMODE_RG_DATA
#define TP6X_DSP_BMODE_IS_NONE			(TP6X_DSP_BMODE_RG_DATA==TP6X_DSP_BMODE_NONE)
#define TP6X_DSP_BMODE_IS_HPI			(TP6X_DSP_BMODE_RG_DATA==TP6X_DSP_BMODE_HPI)
#define TP6X_DSP_BMODE_IS_FLASH			(TP6X_DSP_BMODE_RG_DATA==TP6X_DSP_BMODE_FLASH)


			//-- SMP-0/1 rgs
#define TP6X_SMP0_RG_DATA				TP6X_read_byte(TP6X_HIF_AREA,TP6X_HIF_SMP0_RG)
#define TP6X_SET_SMP0_RG(d)				TP6X_write_byte(TP6X_HIF_AREA,TP6X_HIF_SMP0_RG,d)
#define TP6X_SMP1_RG_DATA				TP6X_read_byte(TP6X_HIF_AREA,TP6X_HIF_SMP1_RG)
#define TP6X_SET_SMP1_RG(d)				TP6X_write_byte(TP6X_HIF_AREA,TP6X_HIF_SMP1_RG,d)

#define TP6X_SMP_MODE_DATA				(TP6X_SMP0_RG_DATA & TP6X_SMP_MODE_BITMASK)
#define TP6X_SET_SMP_MODE_SPA			TP6X_SET_SMP0_RG(TP6X_SMP0_RG_DATA & ~TP6X_SMP_MODE_BITMASK)
#define TP6X_SET_SMP_MODE_DPA			TP6X_SET_SMP0_RG(TP6X_SMP0_RG_DATA | TP6X_SMP_MODE_DPA)

#define TP6X_SMP0_AREA_DATA				(TP6X_SMP0_RG_DATA & TP6X_SMP_AREA_BITMASK)
#define TP6X_SET_SMP0_DPRAM_AREA		TP6X_SET_SMP0_RG(TP6X_SMP0_RG_DATA & ~TP6X_SMP_AREA_BITMASK)
#define TP6X_SET_SMP0_DPSEM_AREA		TP6X_SET_SMP0_RG(TP6X_SMP0_RG_DATA | TP6X_SMP_DPSEM_AREA)
#define TP6X_SMP1_AREA_DATA				(TP6X_SMP1_RG_DATA & TP6X_SMP_AREA_BITMASK)
#define TP6X_SET_SMP1_DPRAM_AREA		TP6X_SET_SMP1_RG(TP6X_SMP1_RG_DATA & ~TP6X_SMP_AREA_BITMASK)
#define TP6X_SET_SMP1_DPSEM_AREA		TP6X_SET_SMP1_RG(TP6X_SMP1_RG_DATA | TP6X_SMP_DPSEM_AREA)

#define TP6X_SMP0_PAGE_DATA				(TP6X_SMP0_RG_DATA & TP6X_SMP_DPA_PAGE_BITMASK)
#define TP6X_SMP1_PAGE_DATA				(TP6X_SMP1_RG_DATA & TP6X_SMP_DPA_PAGE_BITMASK)
#define TP6X_SET_SMP0_PAGE(d)			TP6X_SET_SMP0_RG((TP6X_SMP0_RG_DATA & ~TP6X_SMP_DPA_PAGE_BITMASK) | (d & TP6X_SMP_DPA_PAGE_BITMASK))
#define TP6X_SET_SMP1_PAGE(d)			TP6X_SET_SMP1_RG((TP6X_SMP1_RG_DATA & ~TP6X_SMP_DPA_PAGE_BITMASK) | (d & TP6X_SMP_DPA_PAGE_BITMASK))

//--------------------- macros for HPI access ----------------------------
	//-- old universal API rev.1 C6201/C6701/C64xx HPI port defs
#define	C6X_HPIC_HWOB_BITMASK				0x1
#define	C6X_HPIC_DSPINT_BITMASK				0x2
#define	C6X_HPIC_HINT_BITMASK				0x4
#define	C6X_HPIC_HRDY_BITMASK				0x8
#define	C6X_HPIC_FETCH_BITMASK				0x10

#define	C6X_HPIC_HWOB_OFF					0
#define	C6X_HPIC_HWOB_ON					0x1

#define	C6X_HPIC_DSPINT_OFF					0x2
#define	C6X_HPIC_DSPINT_ON					0x2

#define	C6X_HPIC_HINT_OFF					0
#define	C6X_HPIC_HINT_ON					0x4

#define	C6X_HPIC_HRDY_OFF					0
#define	C6X_HPIC_HRDY_ON					0x8

#define	C6X_HPIC_FETCH_OFF					0
#define	C6X_HPIC_FETCH_ON					0x10


	//-- C6201/C6701 16-bit HPI port defs and C64xx 16-bit HPI defs
#define	C6X_HPI16_HPIC_HWOB_BITMASK			0x1
#define	C6X_HPI16_HPIC_DSPINT_BITMASK		0x2
#define	C6X_HPI16_HPIC_HINT_BITMASK			0x4
#define	C6X_HPI16_HPIC_HRDY_BITMASK			0x8
#define	C6X_HPI16_HPIC_FETCH_BITMASK		0x10

	//-- C6202/C6203 HPI port defs
#define	C6X_HPI32_HPIA_DSPINT_BITMASK		0x1L
#define	C6X_HPI32_HPIA_AINC_BITMASK			0x2L
#define	C6X_HPI32_HPIA_AINC_INC				0L
#define	C6X_HPI32_HPIA_AINC_NO_INC			2L

	//-- C64xx 32-bit HPI port defs
	//   Note, that HWOB bit is not used.
#define	C64XX_HPIC_DSPINT_BITMASK			0x2
#define	C64XX_HPIC_HINT_BITMASK				0x4
#define	C64XX_HPIC_HRDY_BITMASK				0x8
#define	C64XX_HPIC_FETCH_BITMASK			0x10

	//-- C6x HPI access address range
#define	C6X_HPIA_MIN						0x0L
#define	C6X_HPIA_MAX						0xffffffffL


	//-- uniform TP6x HPI port access functions/macros, which are valid for all TP6x boards
	//   (function/macros names are compatible with old TP6x host API rev.1).
unsigned long	TP6X_hpia_data();
int 			TP6X_set_hpia(unsigned long);
unsigned long 	TP6X_hpid_data();
int 			TP6X_set_hpid(unsigned long);
unsigned long	TP6X_hpid_ainc_data();
int 			TP6X_set_hpid_ainc(unsigned long);
int 	      	TP6X_hpi_dspint_data();
int 	      	TP6X_set_hpi_dspint();
unsigned 		TP6X_hpic_data();			// returns 16-bit LSW only, since MSW==LSW for HPIC
int 			TP6X_set_hpic(unsigned);	// parameter must be 16-bit LSW only, since MSW==LSW for HPIC
int 	      	TP6X_hpi_init();

		//-- universal HPI registers access
		//   Note, that for TP6202/TP6203 access to HPIA reg returns/alters ADDRESS field 
		//	 only (DSPINT and AINC are not processed).
#define TP6X_HPIA_DATA				TP6X_hpia_data()
#define TP6X_SET_HPIA(d)			TP6X_set_hpia(d)

		//-- HPI data reg access
#define TP6X_HPID_DATA				TP6X_hpid_data()
#define TP6X_SET_HPID(d)			TP6X_set_hpid(d)

		//-- HPI data reg access with AINC feature 
		//   (For TP6202/TP6203, the AINC bit in HPIA is not altered)
#define TP6X_HPID_AINC_DATA			TP6X_hpid_ainc_data()
#define TP6X_SET_HPID_AINC(d)		TP6X_set_hpid_ainc(d)

		//-- HPI DSPINT bit
#define TP6X_HPI_DSPINT_DATA		TP6X_hpi_dspint_data()
#define TP6X_SET_HPI_DSPINT			TP6X_set_hpi_dspint()

		//-- HPIC register access
#define TP6X_HPIC_DATA				TP6X_hpic_data()
#define TP6X_SET_HPIC(d)			TP6X_set_hpic(d)

		//-- HPI init
#define TP6X_HPI_INIT				TP6X_hpi_init()


	//-- TP62/TP67 16-bit HPI port access macros
#define TP6X_HPI16_HPIC_DATA		TP6X_hpic_data()
#define TP6X_SET_HPI16_HPIC(d)		TP6X_set_hpic(d)
#define TP6X_SET_HPI16_HPIC_LSW(d)	TP6X_write_word(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPIC_RG_LSW,((unsigned)d)&0x17)
#define TP6X_SET_HPI16_HPIC_MSW(d)	TP6X_write_word(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPIC_RG_MSW,((unsigned)d)&0x17)
#define TP6X_HPI16_HPIC_DATA_LSW	(TP6X_read_word(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPIC_RG_LSW) &0x1f)
#define TP6X_HPI16_HPIC_DATA_MSW	(TP6X_read_word(TP6X_HIF_AREA,TP6X_HIF_HPI16_HPIC_RG_MSW) & 0x1f)

#define TP6X_HPI16_DSPINT_DATA		((TP6X_HPI16_HPIC_DATA>>1) & 0x1)
#define TP6X_HPI16_HINT_DATA		((TP6X_HPI16_HPIC_DATA>>2) & 0x1)
#define TP6X_HPI16_HRDY_DATA		((TP6X_HPI16_HPIC_DATA>>3) & 0x1)
#define TP6X_HPI16_FETCH_DATA		((TP6X_HPI16_HPIC_DATA>>4) & 0x1)	// always returns '0'
#define TP6X_HPI16_HWOB_DATA		((TP6X_HPI16_HPIC_DATA)    & 0x1)
#define TP6X_SET_HPI16_HWOB			TP6X_SET_HPI16_HPIC(C6X_HPIC_HWOB_ON)
#define TP6X_SET_HPI16_DSPINT		TP6X_SET_HPI16_HPIC(C6X_HPIC_DSPINT_ON)
#define TP6X_CLR_HPI16_HINT			TP6X_SET_HPI16_HPIC(C6X_HPIC_HINT_ON)
#define TP6X_SET_HPI16_FETCH		TP6X_SET_HPI16_HPIC(C6X_HPIC_FETCH_ON)

#define TP6X_HPI16_HPIA_DATA		TP6X_hpia_data()
#define TP6X_SET_HPI16_HPIA(d)		TP6X_set_hpia(d)

#define TP6X_HPI16_HPID_DATA		TP6X_hpid_data()
#define TP6X_SET_HPI16_HPID(d)		TP6X_set_hpid(d)

#define TP6X_HPI16_HPID_AINC_DATA	TP6X_hpid_ainc_data()
#define TP6X_SET_HPI16_HPID_AINC(d)	TP6X_set_hpid_ainc(d)

#define TP6X_HPI16_HPI_INIT			TP6X_hpi_init()


	//-- TP6202/TP6203 32-bit HPI port access macros
#define TP6X_HPI32_HPID_DATA		TP6X_hpid_data()
#define TP6X_SET_HPI32_HPID(d)		TP6X_set_hpid(d)

		//-- full HPIA reg access with set/return of DSPINT/AINC bits
#define TP6X_HPI32_HPIA_DATA		TP6X_read_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPIA_RG)
#define TP6X_SET_HPI32_HPIA(d)		TP6X_write_dword(TP6X_HIF_AREA,TP6X_HIF_HPI32_HPIA_RG,(unsigned long)d)

		//-- HPIA reg access to ADDRESS field only
#define TP6X_HPI32_HPIA_ADDR_DATA	TP6X_hpia_data()
#define TP6X_SET_HPI32_HPIA_ADDR(d)	TP6X_set_hpia(d)

		//-- DSPINT/AINC bits set/return macros
#define TP6X_HPI32_DSPINT_DATA		(TP6X_HPI32_HPIA_DATA & C6X_HPI32_HPIA_DSPINT_BITMASK)
#define TP6X_SET_HPI32_DSPINT		TP6X_SET_HPI32_HPIA(TP6X_HPI32_HPIA_DATA | 0x1L)
#define TP6X_HPI32_AINC_DATA		(~(TP6X_HPI32_HPIA_DATA>>1) & 0x1L)					// inversed AINC is returned
#define TP6X_SET_HPI32_AINC			TP6X_SET_HPI32_HPIA(TP6X_HPI32_HPIA_DATA & ~0x3L)	// inversed AINC is set/cleared
#define TP6X_CLR_HPI32_AINC			TP6X_SET_HPI32_HPIA((TP6X_HPI32_HPIA_DATA & ~0x3L) | 0x2L)

#define TP6X_HPI32_HPI_INIT			TP6X_hpi_init()


	//-- TP64xx 32-bit HPI port access macros
	//   Note, that HPIC HWOB bit for TP64xx is ignored.
#define TP64XX_HPIC_DATA			TP6X_hpic_data()
#define TP64XX_SET_HPIC(d)			TP6X_set_hpic(d)

#define TP64XX_HPIC_DSPINT_DATA		((TP64XX_HPIC_DATA>>1) & 0x1)
#define TP64XX_HPIC_HINT_DATA		((TP64XX_HPIC_DATA>>2) & 0x1)
#define TP64XX_HPIC_HRDY_DATA		((TP64XX_HPIC_DATA>>3) & 0x1)
#define TP64XX_HPIC_FETCH_DATA		((TP64XX_HPIC_DATA>>4) & 0x1)	// always returns '0'
#define TP64XX_SET_HPIC_DSPINT		TP64XX_SET_HPIC(C6X_HPIC_DSPINT_ON)
#define TP64XX_CLR_HPIC_HINT		TP64XX_SET_HPIC(C6X_HPIC_HINT_ON)
#define TP64XX_SET_HPIC_FETCH		TP64XX_SET_HPIC(C6X_HPIC_FETCH_ON)

#define TP64XX_HPIA_DATA			TP6X_hpia_data()
#define TP64XX_SET_HPIA(d)			TP6X_set_hpia(d)

#define TP64XX_HPID_DATA			TP6X_hpid_data()
#define TP64XX_SET_HPID(d)			TP6X_set_hpid(d)

#define TP64XX_HPID_AINC_DATA		TP6X_hpid_ainc_data()
#define TP64XX_SET_HPID_AINC(d)		TP6X_set_hpid_ainc(d)

#define TP64XX_HPI_INIT				TP6X_hpi_init()



	//-- universal API rev.1 HPI access macros: valid for TP62/TP67/TP64xx boards only
#define TP6X_SET_HPIC_LSW(d)		TP6X_SET_HPI16_HPIC_LSW(d)	// TP62/TP67 only
#define TP6X_SET_HPIC_MSW(d)		TP6X_SET_HPI16_HPIC_MSW(d)	// TP62/TP67 only
#define TP6X_HPIC_DATA_LSW			TP6X_HPI16_HPIC_DATA_LSW	// TP62/TP67 only
#define TP6X_HPIC_DATA_MSW			TP6X_HPI16_HPIC_DATA_MSW	// TP62/TP67 only

#define TP6X_HPIC_HINT_DATA			TP6X_HPI16_HINT_DATA
#define TP6X_HPIC_HRDY_DATA			TP6X_HPI16_HRDY_DATA
#define TP6X_HPIC_FETCH_DATA		TP6X_HPI16_FETCH_DATA
#define TP6X_HPIC_HWOB_DATA			TP6X_HPI16_HWOB_DATA
#define TP6X_HPIC_DSPINT_DATA		TP6X_HPI_DSPINT_DATA
#define TP6X_SET_HPIC_HWOB			TP6X_SET_HPI16_HWOB
#define TP6X_CLR_HPIC_HINT			TP6X_CLR_HPI16_HINT
#define TP6X_SET_HPIC_FETCH			TP6X_SET_HPI16_FETCH
#define TP6X_SET_HPIC_DSPINT		TP6X_SET_HPI_DSPINT


//------------------- error codes -------------------------
#define	TP6X_OK									0		
#define	TP6X_PARAM_ERR							-1
#define	TP6X_MEM_ALLOC_ERR						-2
#define	TP6X_HW_ERR								-5
#define	TP6X_NOT_OPEN_ERR						-10
#define	TP6X_NO_BOARDS_FOUND_ERR				-11
#define	TP6X_INV_PCI_AREA_INFO_ERR				-12
#define	TP6X_INCOMPAT_PCI_AREA_INFO_ERR			-13
#define	TP6X_INV_BOARD_INFO_ERR					-14
#define	TP6X_BOARD_NOT_FOUND_ERR				-20
#define TP6X_INV_CMND_ERR						-21
#define TP6X_INV_BADDR_AREA_ERR					-22
#define TP6X_ACCESS_BADDR_AREA_ERR				-23
#define TP6X_ACCESS_INV_BADDR_MEM_ERR			-24



//================ TP6x DSP Environment Areas ===============

	//~~~~~~~~~ TP62xx/TP67 DSP memory areas
		//----------- SBSRAM area @CE-2
#define	TP6X_DSP_SBSRAM_SADDR					0x02000000
#define	TP6X_DSP_SBSRAM_EADDR_MAX				0x023fffff
#define	TP6X_DSP_SBSRAM_LEN_MAX					0x00400000

		//----------- SDRAM area @CE-3
#define	TP6X_DSP_SDRAM_SADDR					0x03000000
#define	TP6X_DSP_SDRAM_EADDR_MAX				0x03ffffff
#define	TP6X_DSP_SDRAM_LEN_MAX					0x01000000

		//----------- optional SDRAM area @CE-2 (TP6202/TP6203 only with 8M SDRAM installed)
#define	TP6X_DSP_SDRAM2_SADDR					0x02000000
#define	TP6X_DSP_SDRAM2_EADDR_MAX				0x02ffffff
#define	TP6X_DSP_SDRAM2_LEN_MAX					0x01000000

		//----------- FLASH/EPROM area @CE-1
#define	TP6X_DSP_FLASH_SADDR					0x01000000	
#define	TP6X_DSP_FLASH_EADDR					0x013fffff
#define	TP6X_DSP_FLASH_LEN						0x00400000

		//----------- DPRAM/DPSEM/PCIC/IOX/PIOX/SX-A/SX-B @CE-0
#define TP6X_DSP_DPRAM_SADDR					0x00000000
#define TP6X_DSP_DPRAM_EADDR_MAX				0x0003ffff
#define TP6X_DSP_DPRAM_LEN_MAX					0x00040000

#define TP6X_DSP_DPSEM_SADDR					0x00080000	
#define TP6X_DSP_DPSEM_EADDR					0x0008001f
#define TP6X_DSP_DPSEM_LEN						0x00000020

#define TP6X_DSP_PIOX_SADDR						0x00200000
#define TP6X_DSP_PIOX_EADDR						0x003fffff
#define TP6X_DSP_PIOX_LEN						0x00200000

#define TP6X_DSP_SXA_SADDR						0x00100000
#define TP6X_DSP_SXA_EADDR						0x0010003f
#define TP6X_DSP_SXA_LEN						0x00000040

#define TP6X_DSP_SXB_SADDR						0x00140000
#define TP6X_DSP_SXB_EADDR						0x0014003f
#define TP6X_DSP_SXB_LEN						0x00000040

#define TP6X_DSP_PCIC_SADDR						0x000c0000
#define TP6X_DSP_PCIC_EADDR						0x000c003f	
#define TP6X_DSP_PCIC_LEN						0x00000040	

#define TP6X_DSP_IOX_SADDR						0x000c0040
#define TP6X_DSP_IOX_EADDR						0x000c007f	
#define TP6X_DSP_IOX_LEN						0x00000040	

		//---------- C6x DSP on-chip memory areas
#define TP6X_DSP_ONCHIP_PRAM_SADDR				0x01400000L  // C6x on-chip P-RAM
#define TP6X_DSP_ONCHIP_PRAM_EADDR				0x0140ffffL  // C6x P-RAM end address (minimum general for C6x, provided for compatibility with rev.1 API)
#define TP62_DSP_ONCHIP_PRAM_EADDR				0x0140ffffL  // C6201 P-RAM end address 
#define TP67_DSP_ONCHIP_PRAM_EADDR				0x0140ffffL  // C6701 P-RAM end address 
#define TP6202_DSP_ONCHIP_PRAM_EADDR			0x0143ffffL  // C6202 P-RAM end address 
#define TP6203_DSP_ONCHIP_PRAM_EADDR			0x0145ffffL  // C6203 P-RAM end address 
#define TP6X_DSP_ONCHIP_DRAM_SADDR				0x80000000L  // C6x on-chip D-RAM start address
#define TP6X_DSP_ONCHIP_DRAM_EADDR				0x8000ffffL  // C6x D-RAM end address (minimum general for C6x, provided for compatibility with rev.1 API)
#define TP62_DSP_ONCHIP_DRAM_EADDR				0x8000ffffL  // C6201 D-RAM end address 
#define TP67_DSP_ONCHIP_DRAM_EADDR				0x8000ffffL  // C6701 D-RAM end address 
#define TP6202_DSP_ONCHIP_DRAM_EADDR			0x8001ffffL  // C6202 D-RAM end address 
#define TP6203_DSP_ONCHIP_DRAM_EADDR			0x8007ffffL  // C6203 D-RAM end address 
#define TP6X_DSP_ONCHIP_RG_SADDR				0x01800000L  // C6x DSP on-chip registers
#define TP6X_DSP_ONCHIP_RG_EADDR				0x01FfffffL



	//~~~~~~~~~ TP64xx DSP memory areas
		//----------- SBSRAM area
#define	TP64XX_DSP_SBSRAM_SADDR					0x80000000
#define	TP64XX_DSP_SBSRAM_EADDR_MAX				0x807fffff
#define	TP64XX_DSP_SBSRAM_LEN_MAX				0x00800000

		//----------- SDRAM area
#define	TP64XX_DSP_SDRAM_SADDR					0xA0000000
#define	TP64XX_DSP_SDRAM_EADDR_MAX				0xA7ffffff
#define	TP64XX_DSP_SDRAM_LEN_MAX				0x08000000

		//----------- FLASH/EPROM area
#define	TP64XX_DSP_FLASH_SADDR					0x64000000
#define	TP64XX_DSP_FLASH_EADDR					0x641fffff
#define	TP64XX_DSP_FLASH_LEN					0x00200000

		//----------- DPRAM area
#define TP64XX_DSP_DPRAM_SADDR					0x90000000
#define TP64XX_DSP_DPRAM_EADDR_MAX				0x901fffff
#define TP64XX_DSP_DPRAM_LEN_MAX				0x00200000

		//----------- IOX/PIOX/SIOX area
#define TP64XX_DSP_PIOX_SADDR					0x6C000000
#define TP64XX_DSP_PIOX_EADDR					0x6C1fffff
#define TP64XX_DSP_PIOX_LEN						0x00200000

#define TP64XX_DSP_SXA_SADDR					0x60100000
#define TP64XX_DSP_SXA_EADDR					0x6010007f
#define TP64XX_DSP_SXA_LEN						0x00000040

#define TP64XX_DSP_SXB_SADDR					0x60180000
#define TP64XX_DSP_SXB_EADDR					0x6018007f
#define TP64XX_DSP_SXB_LEN						0x00000040

#define TP64XX_DSP_PCIC_SADDR					0x60000000
#define TP64XX_DSP_PCIC_EADDR					0x6000003f
#define TP64XX_DSP_PCIC_LEN						0x00000040

#define TP64XX_DSP_IOX_SADDR					0x60000040
#define TP64XX_DSP_IOX_EADDR					0x6000007f
#define TP64XX_DSP_IOX_LEN						0x00000040

		//---------- C6x DSP on-chip memory areas
#define TP64XX_DSP_ONCHIP_RAM_SADDR				0x00000000L  // C64xx on-chip P-RAM
#define TP64XX_DSP_ONCHIP_RAM_EADDR				0x000fffffL  // C64xx P-RAM end address (minimum general for C6x, provided for compatibility with rev.1 API)
#define TP64XX_DSP_ONCHIP_RG_SADDR				0x01800000L  // C64xx DSP on-chip registers
#define TP64XX_DSP_ONCHIP_RG_EADDR				0x01FfffffL
#define TP64XX_DSP_ONCHIP1_RG_SADDR				0x02000000L  // C64xx DSP on-chip registers set #1 (QDMA)
#define TP64XX_DSP_ONCHIP1_RG_EADDR				0x02000033L


	//------ recommended DSP EMIF configurations ----------
	// These defs are required in case C6x DSP is being initialized in HPI bootmode or
	// in case host PCI will need to access DSP memory areas via HPI port prior
	// DSP will initialize EMIF.

		//>>>>>>>>>> address defs for DSP on-chip EMIF control RGs
#define C6X_EMIF_GCR_RG    			0x01800000L
#define C6X_EMIF_CE0_RG				0x01800008L
#define C6X_EMIF_CE1_RG        		0x01800004L
#define C6X_EMIF_CE2_RG     	   	0x01800010L
#define C6X_EMIF_CE3_RG        		0x01800014L
#define C6X_EMIF_SDRAM_RG     	 	0x01800018L
#define C6X_EMIF_SDRAM_TIMER_RG		0x0180001cL

			// C64xx specific extension regs
#define C6X_EMIFA_GCR_RG    		C6X_EMIF_GCR_RG
#define C6X_EMIFA_CE0_RG			C6X_EMIF_CE0_RG
#define C6X_EMIFA_CE1_RG    		C6X_EMIF_CE1_RG
#define C6X_EMIFA_CE2_RG    		C6X_EMIF_CE2_RG
#define C6X_EMIFA_CE3_RG    		C6X_EMIF_CE3_RG
#define C6X_EMIFA_SDRAM_RG  		C6X_EMIF_SDRAM_RG
#define C6X_EMIFA_SDRAM_TIMER_RG	C6X_EMIF_SDRAM_TIMER_RG
#define	C6X_EMIFA_SDRAM_EXT_RG  	0x01800020
#define	C6X_EMIFA_CE0_SEC_RG    	0x01800048
#define	C6X_EMIFA_CE1_SEC_RG    	0x01800044
#define	C6X_EMIFA_CE2_SEC_RG    	0x01800050
#define	C6X_EMIFA_CE3_SEC_RG    	0x01800054

#define	C6X_EMIFB_GCR_RG        	0x01A80000
#define	C6X_EMIFB_CE0_RG        	0x01A80008
#define	C6X_EMIFB_CE1_RG        	0x01A80004
#define	C6X_EMIFB_CE2_RG        	0x01A80010
#define	C6X_EMIFB_CE3_RG        	0x01A80014
#define	C6X_EMIFB_SDRAM_RG      	0x01A80018
#define	C6X_EMIFB_SDRAM_TIMER_RG	0x01A8001c
#define	C6X_EMIFB_SDRAM_EXT_RG    	0x01A80020
#define	C6X_EMIFB_CE0_SEC_RG     	0x01A80048
#define	C6X_EMIFB_CE1_SEC_RG     	0x01A80044
#define	C6X_EMIFB_CE2_SEC_RG     	0x01A80050
#define	C6X_EMIFB_CE3_SEC_RG     	0x01A80054




		//-- old TP62/TP67 DSP EMIF settings, which are provided for
		//   compatibility with API rev.1 software only. These settings will run
		//   on TP6202/TP6203 with limited functionality.
#define TP6X_DSP_EMIF_GCR_DATA				0x3078          // 1/2 CLK SBSRAM
#define TP6X_DSP_EMIF_CE3_DATA				0x30		// SDRAM
#define TP6X_DSP_EMIF_CE2_DATA				0x40		// SBSRAM
#define TP6X_DSP_EMIF_CE1_DATA				0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define TP6X_DSP_EMIF_CE0_DATA				0x31b3c623	// DPRAM, PIOX and UTIL area (stb=6, setup=3,hold=3)
#define TP6X_DSP_EMIF_SDRAM_DATA			0x7339000	// Trc=90ns, Trp=30ns, Trcd=30ns for Micron 100MHz SDRAM
#define	TP6X_DSP_EMIF_SDRAM_TIMER_DATA		1000		// REFR period = 10us

		//-- dedicated DSP EMIF settings for TP62 (DSP clock = 200MHz)
#define TP62_DSP_EMIF_GCR_DATA				0x3078          // 1/2 CLK SBSRAM
#define TP62_DSP_EMIF_CE3_DATA				0x30		// SDRAM
#define TP62_DSP_EMIF_CE2_DATA				0x40		// SBSRAM
#define TP62_DSP_EMIF_CE1_DATA				0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define TP62_DSP_EMIF_CE0_DATA				0x31b3c623	// DPRAM, PIOX and UTIL area (stb=6, setup=3,hold=3)
#define TP62_DSP_EMIF_SDRAM_DATA			0x7339000	// Trc=90ns, Trp=30ns, Trcd=30ns for Micron 100MHz SDRAM
#define	TP62_DSP_EMIF_SDRAM_TIMER_DATA		1000		// REFR period = 10us @ 100MHz(10ns)

		//-- dedicated DSP EMIF settings for TP67(DSP clock = 167 MHz)
#define TP67_DSP_EMIF_GCR_DATA				0x3078      // 1/2 CLK SBSRAM
#define TP67_DSP_EMIF_CE3_DATA				0x30		// SDRAM
#define TP67_DSP_EMIF_CE2_DATA				0x40		// SBSRAM
#define TP67_DSP_EMIF_CE1_DATA				0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define TP67_DSP_EMIF_CE0_DATA				0x2162c522	// DPRAM, PIOX and UTIL area (stb=5, setup=2,hold=2)
#define TP67_DSP_EMIF_SDRAM_DATA			0x7338000	// Trc=90ns, Trp=30ns, Trcd=30ns for Micron 100MHz SDRAM
#define	TP67_DSP_EMIF_SDRAM_TIMER_DATA		833			// REFR period = 10us @ 83.5MHz(12ns)

		//-- dedicated TP6202 DSP EMIF settings (DSP clock = 250 MHz)
		//  note!!! EMIF CE2 and CE3 registers shall be configured identically
#define TP6202_DSP_EMIF_GCR_DATA			0x3078		// set as default
#define TP6202_DSP_EMIF_CE3_SDRAM_DATA		0x30		// enable CE3 as SDRAM
#define TP6202_DSP_EMIF_CE3_SBSRAM_DATA		0x40		// enable CE3 as SBSRAM (reserved)
#define TP6202_DSP_EMIF_CE2_SDRAM_DATA		0x30		// enable CE2 as SDRAM
#define TP6202_DSP_EMIF_CE2_SBSRAM_DATA		0x40		// enable CE2 as SBSRAM
#define TP6202_DSP_EMIF_SDRAM_DATA			0x7339000	// Trc=70ns, Trp=20ns, Trcd=20ns for Micron 125MHz SDRAM
#define	TP6202_DSP_EMIF_SDRAM_TIMER_DATA 	1250		// REFR period = 10us @ 125MHz(8ns)
#define TP6202_DSP_EMIF_CE1_DATA			0x87b81e23	// FLASH area (stb=30, setup=8,hold=3)
#define TP6202_DSP_EMIF_CE0_DATA			0x31f30723	// DPRAM, PIOX and UTIL area (stb=7, setup=3,hold=3)

		//-- dedicated TP6203 DSP EMIF settings (DSP clock = 300 MHz)
		//  note!!! EMIF CE2 and CE3 registers shall be configured identically
#define TP6203_DSP_EMIF_GCR_DATA			0x3078      // set as default
#define TP6203_DSP_EMIF_CE3_SDRAM_DATA		0x30		// enable CE3 as SDRAM
#define TP6203_DSP_EMIF_CE3_SBSRAM_DATA		0x40		// enable CE3 as SBSRAM (reserved)
#define TP6203_DSP_EMIF_CE2_SDRAM_DATA		0x30		// enable CE2 as SDRAM
#define TP6203_DSP_EMIF_CE2_SBSRAM_DATA		0x40		// enable CE2 as SBSRAM
#define TP6203_DSP_EMIF_CE1_DATA			0xc93c2423	// FLASH area (stb=36, setup=12,hold=3)
#define TP6203_DSP_EMIF_CE0_DATA			0x42740923	// DPRAM, PIOX and UTIL area (stb=9, setup=4,hold=3)
#define TP6203_DSP_EMIF_SDRAM_DATA			0x733a000	// Trc=60ns, Trp=15ns, Trcd=15ns for Micron 150MHz SDRAM
#define	TP6203_DSP_EMIF_SDRAM_TIMER_DATA 	1500		// REFR period = 10us @ 150MHz(6.6ns)


		//-- dedicated TP64xx EMIF settings
		//   EMIF registers setting for TP64xx boards is done upon the f/w revision
			// common EMIF-A/B registers settings, whcich are used for all f/w revisions
#define TP64XX_DSP_EMIFA_GCR_DATA			0x0001207c  // default GCR setting
#define TP64XX_DSP_EMIFA_CE0_DATA			0xe0		// configure CE0 as 64-bit SBSRAM
#define TP64XX_DSP_EMIFA_CE0_SEC_DATA		0x00000002	// CE-0 secondary ext
#define TP64XX_DSP_EMIFA_CE1_DATA			0x40		// configure CE0 as 32-bit SDPRAM
#define TP64XX_DSP_EMIFA_CE1_SEC_DATA		0x00000002	// CE-0 secondary ext
#define TP64XX_DSP_EMIFA_CE2_DATA			0xd0		// configure CE2 as 64-bit SDRAM
#define TP64XX_DSP_EMIFA_CE2_SEC_DATA		0x00000002	// CE-2 secondary ext
#define TP64XX_DSP_EMIFA_CE3_DATA			0xffffff03	// reserved, leave as default
#define TP64XX_DSP_EMIFB_GCR_DATA			0x0001207c  // default GCR setting
#define TP64XX_DSP_EMIFB_CE2_DATA			0xffffff03	// reserved, leave as default

			// TP64xx boards f/w rev.1A: 600MHz C64xx DSP, EMIF-A/B clock 100MHz (DSP_CLK/6)
#define TP64XX_DSP_EMIFA_SDRAM16M_DATA_REV1A_600MHZ			0x63115000	// 128MB: Trc=60ns, Trp=15ns, Trcd=15ns for Micron -8ns SDRAM
#define TP64XX_DSP_EMIFA_SDRAM4M_DATA_REV1A_600MHZ			0x57115000	// 32MB:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron -8ns SDRAM
#define TP64XX_DSP_EMIFA_SDRAM_EXT_DATA_REV1A_600MHZ		0x00014d29	// SDRAM ext
#define	TP64XX_DSP_EMIFA_SDRAM16M_TIMER_DATA_REV1A_600MHZ	781			// REFR period = 64mS@8192rows => 7.81us @ 100MHz(10ns)
#define	TP64XX_DSP_EMIFA_SDRAM4M_TIMER_DATA_REV1A_600MHZ	1562		// REFR period = 64mS@4096rows => 15.62us @ 100MHz(10ns)
#define TP64XX_DSP_EMIFB_CE0_DATA_REV1A_600MHZ				0x10d14311	// IOX/SIOX_revC area (16-bit ASYNC, stb=3, setup=1,hold=1, TA=1)
#define TP64XX_DSP_EMIFB_CE1_DATA_REV1A_600MHZ				0x22924a01	// FLASH area (8-bit async,stb=12, setup=2,hold=1, TA=1)
#define TP64XX_DSP_EMIFB_CE3_DATA_REV1A_600MHZ				0x10d14311	// PIOX area (16-bit ASYNC, stb=3, setup=1,hold=1, TA=1)

			// TP64xx boards f/w rev.1A-1: 600MHz/720MHz/1GHz C64xx DSP, EMIF-A/B clock 125MHz (external)
#define TP64XX_DSP_EMIFA_SDRAM16M_DATA_REV1A1				0x63117000	// 128MB: Trc=60ns, Trp=15ns, Trcd=15ns for Micron -8ns SDRAM
#define TP64XX_DSP_EMIFA_SDRAM4M_DATA_REV1A1				0x57117000	// 32MB:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron -8ns SDRAM
#define TP64XX_DSP_EMIFA_SDRAM_EXT_DATA_REV1A1				0x00014d29	// SDRAM ext
#define	TP64XX_DSP_EMIFA_SDRAM16M_TIMER_DATA_REV1A1			976			// REFR period = 64mS@8192rows => 7.81us @ 125MHz(8ns)
#define	TP64XX_DSP_EMIFA_SDRAM4M_TIMER_DATA_REV1A1			1952		// REFR period = 64mS@4096rows => 15.62us @ 125MHz(8ns)
#define TP64XX_DSP_EMIFB_CE0_DATA_REV1A1					0x21124411	// IOX/SIOX_revC/PCICarea (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)
#define TP64XX_DSP_EMIFB_CE1_DATA_REV1A1					0x23524d01	// FLASH area (8-bit,stb=13, setup=2,hold=1)
#define TP64XX_DSP_EMIFB_CE3_DATA_REV1A1					0x21124411	// PIOX area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)

			// TP64xx boards f/w rev.1A-2: 600MHz/720MHz/1GHz C64xx DSP, EMIF-A/B clock 133MHz (external)
#define TP64XX_DSP_EMIFA_SDRAM16M_DATA_REV1A2				0x63117000	// 16Mx64: Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_DSP_EMIFA_SDRAM4M_DATA_REV1A2				0x57117000	// 4Mx64:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_DSP_EMIFA_SDRAM_EXT_DATA_REV1A2				0x00014d29	// SDRAM ext
#define	TP64XX_DSP_EMIFA_SDRAM16M_TIMER_DATA_REV1A2			1041		// REFR period = 64mS@8192rows => 7.81us @ 125MHz(7.5ns)
#define	TP64XX_DSP_EMIFA_SDRAM4M_TIMER_DATA_REV1A2			2082		// REFR period = 64mS@4096rows => 15.62us @ 125MHz(7.5ns)
#define TP64XX_DSP_EMIFB_CE0_DATA_REV1A2					0x21124411	// IOX/SIOX_revC/PCIC area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)
#define TP64XX_DSP_EMIFB_CE1_DATA_REV1A2					0x23d24e01	// FLASH area (8-bit,stb=14, setup=2,hold=1)
#define TP64XX_DSP_EMIFB_CE3_DATA_REV1A2					0x21124411	// PIOX area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)

			// TP64xx boards f/w rev.1B: 600/720/1000MHz DSP, EMIF-A/B_clk=external=133MHz@7.5ns
#define TP64XX_DSP_EMIFA_SDRAM16M_DATA_REV1B				0x63117000	// 16Mx64: Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_DSP_EMIFA_SDRAM4M_DATA_REV1B					0x57117000	// 4Mx64:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_DSP_EMIFA_SDRAM_EXT_DATA_REV1B				0x00014d29	// SDRAM ext
#define	TP64XX_DSP_EMIFA_SDRAM16M_TIMER_DATA_REV1B			1041		// REFR period = 64mS@8192rows => 7.81us @ 125MHz(7.5ns)
#define	TP64XX_DSP_EMIFA_SDRAM4M_TIMER_DATA_REV1B			2082		// REFR period = 64mS@4096rows => 15.62us @ 125MHz(7.5ns)
#define TP64XX_DSP_EMIFB_CE0_DATA_REV1B						0x21124411	// IOX/SIOX_revC/PCIC area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)
#define TP64XX_DSP_EMIFB_CE1_DATA_REV1B						0x24125001	// FLASH area (8-bit,120ns access, stb=16, setup=2,hold=1, TA=1)
#define TP64XX_DSP_EMIFB_CE3_DATA_REV1B						0x21124411	// PIOX area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)

#endif
