/*******************************************************************
   PCI BIOS I/F Library.    v.1.1
   (C) MicroLAB Systems Ltd, 1998-1999

   file: MAIN BODY (.C)
*******************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <string.h>

#include "_pci.h"

#define	_ON	1
#define	_OFF	0

#define _PCI_FX_FUNCTION_ID	0xb1
#define	_PCI_FX_BIOS_PRESENT	0x01
#define	_PCI_FX_FIND_DEVICE	0x02
#define	_PCI_FX_READ_CNF_BYTE	0x08
#define	_PCI_FX_READ_CNF_WORD	0x09
#define	_PCI_FX_READ_CNF_DWORD	0x0a

#define	_PCI_ERR_BASE		-3000

static int __pci_last_err__	=_PCI_NO_ERR;
static int __pci_open__		=_OFF;


int _PCI_open(BYTE *last_pci_bus_number)
{  union REGS inreg,outreg;

   __pci_open__=_OFF;

   inreg.h.ah=_PCI_FX_FUNCTION_ID;
   inreg.h.al=_PCI_FX_BIOS_PRESENT;
   int86(0x1a,&inreg,&outreg);

   if(   outreg.x.cflag!=0
      || outreg.h.ah!=0
      || outreg.h.dl!='P'
      || outreg.h.dh!='C'
     )
	return(__pci_last_err__=_PCI_BIOS_NOT_FOUND_ERR);

   if(last_pci_bus_number!=NULL) *last_pci_bus_number=outreg.h.cl;

   __pci_open__=_ON;
   return(__pci_last_err__=_PCI_NO_ERR);
}


int _PCI_find_device(unsigned vendor_id, unsigned dev_id, 
		    unsigned index,
		    BYTE *bus, BYTE *dev_and_function )
{  union REGS inreg,outreg;
   int r;

   if(__pci_open__!=_ON)
       { if((r=_PCI_open(NULL))!=_PCI_NO_ERR)
	   return(r);
       }

   inreg.h.ah=_PCI_FX_FUNCTION_ID;
   inreg.h.al=_PCI_FX_FIND_DEVICE;
   inreg.x.si=index;
   inreg.x.dx=vendor_id;
   inreg.x.cx=dev_id;
   int86(0x1a,&inreg,&outreg);

/*-???
   The following operator returns !=0 value in case any error occures.
   This results in impossibility to detect what particular error occurs.
*/
   if(outreg.x.cflag!=0) return(__pci_last_err__=_PCI_DEV_NOT_FOUND_ERR);
   if((r=outreg.h.ah)!=0) return(__pci_last_err__=-r+_PCI_ERR_BASE);

   if(bus!=NULL) *bus=outreg.h.bh;
   if(dev_and_function!=NULL) *dev_and_function=outreg.h.bl;

   return(__pci_last_err__=_PCI_NO_ERR);
}


int _PCI_read_cnf_byte(BYTE bus,  BYTE dev_and_function,
		      BYTE reg_number,
		      BYTE *d )
{  union REGS inreg,outreg;
   int r;

   if(__pci_open__!=_ON)
       { if((r=_PCI_open(NULL))!=_PCI_NO_ERR)
	   return(r);
       }

   inreg.h.ah=_PCI_FX_FUNCTION_ID;
   inreg.h.al=_PCI_FX_READ_CNF_BYTE;
   inreg.x.di=reg_number;
   inreg.h.bl=dev_and_function;
   inreg.h.bh=bus;

   int86(0x1a,&inreg,&outreg);

/*-???
   The following operator returns !=0 value in case any error occures.
   This results in impossibility to detect what particular error occurs.
*/
   if(outreg.x.cflag!=0) return(__pci_last_err__=_PCI_CNF_RG_READ_ERR);
   if((r=outreg.h.ah)!=0) return(__pci_last_err__=-r+_PCI_ERR_BASE);

   if (d!=NULL) *d=outreg.h.cl;
   return(__pci_last_err__=_PCI_NO_ERR);
}


int _PCI_read_cnf_word(BYTE bus,  BYTE dev_and_function,
		      BYTE reg_number,
		      unsigned *d )
{  union REGS inreg,outreg;
   int r;

   if(__pci_open__!=_ON)
       { if((r=_PCI_open(NULL))!=_PCI_NO_ERR)
	   return(r);
       }

   inreg.h.ah=_PCI_FX_FUNCTION_ID;
   inreg.h.al=_PCI_FX_READ_CNF_WORD;
   inreg.x.di=reg_number;
   inreg.h.bl=dev_and_function;
   inreg.h.bh=bus;

   int86(0x1a,&inreg,&outreg);

   if(outreg.x.cflag!=0) return(__pci_last_err__=_PCI_CNF_RG_READ_ERR);
   if((r=outreg.h.ah)!=0) return(__pci_last_err__=-r+_PCI_ERR_BASE);

   if (d!=NULL) *d=outreg.x.cx;
   return(__pci_last_err__=_PCI_NO_ERR);
}


int _PCI_read_cnf_dword(BYTE bus,  BYTE dev_and_function,
		      BYTE reg_number,
		      unsigned long *d )
{  union REGS inreg,outreg;
   int r;
   unsigned _dl,_dh;

   if(__pci_open__!=_ON)
       { if((r=_PCI_open(NULL))!=_PCI_NO_ERR)
	   return(r);
       }

   inreg.h.ah=_PCI_FX_FUNCTION_ID;
   inreg.h.al=_PCI_FX_READ_CNF_WORD;
   inreg.x.di=reg_number;
   inreg.h.bl=dev_and_function;
   inreg.h.bh=bus;
   int86(0x1a,&inreg,&outreg);

   if(outreg.x.cflag!=0) return(__pci_last_err__=_PCI_CNF_RG_READ_ERR);
   if((r=outreg.h.ah)!=0) return(__pci_last_err__=-r+_PCI_ERR_BASE);

   _dl=outreg.x.cx;      	// save temp LSW

   inreg.h.ah=_PCI_FX_FUNCTION_ID;
   inreg.h.al=_PCI_FX_READ_CNF_WORD;
   inreg.x.di=reg_number+2;	// get MSW now
   inreg.h.bl=dev_and_function;
   inreg.h.bh=bus;
   int86(0x1a,&inreg,&outreg);

   if(outreg.x.cflag!=0) return(__pci_last_err__=_PCI_CNF_RG_READ_ERR);
   if((r=outreg.h.ah)!=0) return(__pci_last_err__=-r+_PCI_ERR_BASE);

   _dh=outreg.x.cx;      	// save temp MSW

   if (d!=NULL) *d=((((unsigned long)_dh)<<16) & 0xffff0000) | _dl  ;
   return(__pci_last_err__=_PCI_NO_ERR);
}

int _PCI_err_message(int err, char *s)
{ int r;

  if(s==NULL) return(_PCI_NO_ERR);

  r=_PCI_NO_ERR;
  strcpy(s,">>PCI: ");
  switch(err)
   {
	case _PCI_NO_ERR:
		strcat(s,"OK");
		break;

	case _PCI_BIOS_NOT_FOUND_ERR:
		strcat(s,"PCI BIOS not found");
		break;

	case _PCI_DEV_NOT_FOUND_ERR:
		strcat(s,"device not found");
		break;

	case _PCI_CNF_RG_READ_ERR:
		strcat(s,"cannot read PCI configuration register");
		break;

	case _PCI_BIOS_INV_FUNCTION_ERR:
		strcat(s,"invalid PCI BIOS function");
		break;

	case _PCI_BIOS_INV_VENDOR_ID_ERR:
		strcat(s,"invalid PCI vendor ID");
		break;

	case _PCI_BIOS_INV_DEV_ID_ERR:
		strcat(s,"invalid PCI device ID");
		break;

	case _PCI_BIOS_INV_RG_NUMBER_ERR:
		strcat(s,"invalid PCI configuration register");
		break;

	case _PCI_INV_PARAM_ERR:
		strcat(s,"invalid function parameter");
		break;

	default:
		strcat(s,"unknowm error code");
		r=_PCI_INV_PARAM_ERR;
		break;
    }
   return(r);

   }

int _PCI_last_err_message(char *s)
{
  _PCI_err_message(__pci_last_err__,s);
  return(_PCI_NO_ERR);
}
